@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2018, Intel Corporation                                **
+** Copyright (c) 2016-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -39,18 +39,17 @@
  */
 #define LIBXSMM_TARGET_ARCH_UNKNOWN 0
 #define LIBXSMM_TARGET_ARCH_GENERIC 1
-#define LIBXSMM_X86_IMCI         1001
 #define LIBXSMM_X86_GENERIC      1002
 #define LIBXSMM_X86_SSE3         1003
 #define LIBXSMM_X86_SSE4         1004
 #define LIBXSMM_X86_AVX          1005
 #define LIBXSMM_X86_AVX2         1006
 #define LIBXSMM_X86_AVX512       1007
-#define LIBXSMM_X86_AVX512_MIC   1010
+#define LIBXSMM_X86_AVX512_MIC   1010 /* KNL */
 #define LIBXSMM_X86_AVX512_KNM   1011
-#define LIBXSMM_X86_AVX512_CORE  1020
-#define LIBXSMM_X86_AVX512_ICL   1022
-
+#define LIBXSMM_X86_AVX512_CORE  1020 /* SKX */
+#define LIBXSMM_X86_AVX512_CLX   1021
+#define LIBXSMM_X86_AVX512_CPX   1022
 
 /**
  * Returns the target architecture and instruction set extensions, but *not* necessarily the
@@ -63,5 +62,8 @@ LIBXSMM_API int libxsmm_cpuid_x86(void);
 /** Similar to libxsmm_cpuid_x86, but conceptually not x86-specific. */
 LIBXSMM_API int libxsmm_cpuid(void);
 
+/** Names the CPU architecture given by CPUID. */
+LIBXSMM_API const char* libxsmm_cpuid_name(int id);
+
 #endif /*LIBXSMM_CPUID_H*/
 
