@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -34,51 +34,61 @@
 #include "libxsmm_typedefs.h"
 
 #define LIBXSMM_GEMM_NO_BYPASS(FLAGS, ALPHA, BETA) ( \
-  0 == ((FLAGS) & (LIBXSMM_GEMM_FLAG_TRANS_A | LIBXSMM_GEMM_FLAG_TRANS_B)) && \
+  0 == ((FLAGS) & (LIBXSMM_GEMM_FLAG_TRANS_A)) && \
         (LIBXSMM_FEQ(1, ALPHA) /*|| LIBXSMM_FEQ(-1, ALPHA)*/) && \
         (LIBXSMM_FEQ(1, BETA) || LIBXSMM_FEQ(0, BETA)))
 
-#define LIBXSMM_TRANS_NO_BYPASS_DIMS(M, N, LDO) \
-  ((((unsigned int)(M)) * (N)) <= ((LIBXSMM_AVG_M) * (LIBXSMM_AVG_N)))
-
 
 /** Initialize GEMM descriptor as used by low-level routines (type-specific). */
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_dgemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
-  double alpha, double beta, int flags, libxsmm_gemm_prefetch_type prefetch);
+  double alpha, double beta, int flags, int prefetch);
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_sgemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
-  float alpha, float beta, int flags, libxsmm_gemm_prefetch_type prefetch);
+  float alpha, float beta, int flags, int prefetch);
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_wigemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
-  int alpha, int beta, int flags, libxsmm_gemm_prefetch_type prefetch);
+  int alpha, int beta, int flags, int prefetch);
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_wsgemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
-  float alpha, float beta, int flags, libxsmm_gemm_prefetch_type prefetch);
+  float alpha, float beta, int flags, int prefetch);
+LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bsgemm_descriptor_init(libxsmm_descriptor_blob* blob,
+  libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+  libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
+  float alpha, float beta, int flags, int prefetch);
+LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bgemm_descriptor_init(libxsmm_descriptor_blob* blob,
+  libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+  libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
+  float alpha, float beta, int flags, int prefetch);
 
 /** Initialize GEMM descriptor (generic: double-precision alpha/beta). */
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_gemm_descriptor_dinit(libxsmm_descriptor_blob* blob,
   libxsmm_gemm_precision precision, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc, double alpha, double beta,
-  int flags, libxsmm_gemm_prefetch_type prefetch);
+  int flags, int prefetch);
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_gemm_descriptor_dinit2(libxsmm_descriptor_blob* blob,
   libxsmm_gemm_precision iprec, libxsmm_gemm_precision oprec, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
-  double alpha, double beta, int flags, libxsmm_gemm_prefetch_type prefetch);
+  double alpha, double beta, int flags, int prefetch);
 
 /** Initialize GEMM descriptor as used by low-level routines (generic). */
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_gemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_gemm_precision precision, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc, const void* alpha, const void* beta,
-  int flags, libxsmm_gemm_prefetch_type prefetch);
+  int flags, int prefetch);
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_gemm_descriptor_init2(libxsmm_descriptor_blob* blob,
   libxsmm_gemm_precision iprec, libxsmm_gemm_precision oprec, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc, const void* alpha, const void* beta,
-  int flags, libxsmm_gemm_prefetch_type prefetch);
+  int flags, int prefetch);
+/** Similar to libxsmm_gemm_descriptor_init2 with optional type-converted alpha/beta (dalpha/dbeta). */
+LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_gemm_descriptor_init3(libxsmm_descriptor_blob* blob,
+  libxsmm_gemm_precision iprec, libxsmm_gemm_precision oprec, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+  libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc, const void* alpha, const void* beta,
+  int flags, int prefetch, double* dalpha, double* dbeta);
 
 /** Initialize transpose descriptor as used by low-level routines. */
 LIBXSMM_API libxsmm_trans_descriptor* libxsmm_trans_descriptor_init(libxsmm_descriptor_blob* blob,
@@ -89,6 +99,25 @@ LIBXSMM_API libxsmm_mcopy_descriptor* libxsmm_mcopy_descriptor_init(libxsmm_desc
   unsigned int typesize, unsigned int m, unsigned int n, unsigned int ldo,
   unsigned int ldi, int flags, int prefetch, const int* unroll);
 
+/** Initialize packed trsm descriptor as used by low-level routines. */
+LIBXSMM_API libxsmm_trsm_descriptor* libxsmm_trsm_descriptor_init(libxsmm_descriptor_blob* blob,
+  unsigned int typesize, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint lda, libxsmm_blasint ldb,
+  const void* alpha, char transa, char diag, char side, char uplo, int layout);
+
+/** Initialize packed trmm descriptor as used by low-level routines. */
+LIBXSMM_API libxsmm_trmm_descriptor* libxsmm_trmm_descriptor_init(libxsmm_descriptor_blob* blob,
+  unsigned int typesize, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint lda, libxsmm_blasint ldb,
+  const void* alpha, char transa, char diag, char side, char uplo, int layout);
+
+/** Initialize packed getrf descriptor as used by low-level routines. */
+LIBXSMM_API libxsmm_getrf_descriptor* libxsmm_getrf_descriptor_init(libxsmm_descriptor_blob* blob,
+  unsigned int typesize, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint lda, int layout);
+
+/** Initialize packed pgemm descriptor as used by low-level routines. */
+LIBXSMM_API libxsmm_pgemm_descriptor* libxsmm_pgemm_descriptor_init(libxsmm_descriptor_blob* blob,
+  unsigned int typesize, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k, libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
+  const void* alpha, char transa, char transb, int layout);
+
 /** Structure referring to the generated code with some attached information. */
 LIBXSMM_EXTERN_C typedef struct libxsmm_generated_code {
   void* generated_code;       /** pointer to memory which can contain strings or binary code */
@@ -186,6 +215,36 @@ void libxsmm_generator_spgemm_csc_soa_kernel( libxsmm_generated_code*        io_
                                               const unsigned int*            i_column_idx,
                                               const void*                    i_values );
 
+/* @TODO change int based architecture value */
+LIBXSMM_API void libxsmm_generator_packed_gemm_ac_rm( libxsmm_generated_code*         io_generated_code,
+                                                      const libxsmm_gemm_descriptor*  i_xgemm_desc,
+                                                      const char*                     i_arch );
+
+/* @TODO change int based architecture value */
+LIBXSMM_API void libxsmm_generator_packed_gemm_bc_rm( libxsmm_generated_code*         io_generated_code,
+                                                      const libxsmm_gemm_descriptor*  i_xgemm_desc,
+                                                      const char*                     i_arch );
+
+LIBXSMM_API
+void libxsmm_generator_pgemm_kernel( libxsmm_generated_code*          io_generated_code,
+                                     const libxsmm_pgemm_descriptor*  i_packed_pgemm_desc,
+                                     int                              i_arch, ... );
+
+LIBXSMM_API
+void libxsmm_generator_getrf_kernel( libxsmm_generated_code*          io_generated_code,
+                                     const libxsmm_getrf_descriptor*  i_packed_pgemm_desc,
+                                     int                              i_arch );
+
+LIBXSMM_API
+void libxsmm_generator_trmm_kernel( libxsmm_generated_code*         io_generated_code,
+                                    const libxsmm_trmm_descriptor*  i_packed_trmm_desc,
+                                    const char*                     i_arch );
+
+LIBXSMM_API
+void libxsmm_generator_trsm_kernel( libxsmm_generated_code*         io_generated_code,
+                                    const libxsmm_trsm_descriptor*  i_packed_trsm_desc,
+                                    const char*                     i_arch );
+
 /* @TODO change int based architecture value */
 LIBXSMM_API
 void libxsmm_generator_matcopy_kernel( libxsmm_generated_code*            io_generated_code,
@@ -195,21 +254,7 @@ void libxsmm_generator_matcopy_kernel( libxsmm_generated_code*            io_gen
 LIBXSMM_API
 void libxsmm_generator_transpose_kernel( libxsmm_generated_code*          io_generated_code,
                                          const libxsmm_trans_descriptor*  i_trans_desc,
-                                         const char*                      i_arch );
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_forward_inlineasm(const char*                       i_file_out,
-                                                     const char*                       i_routine_name,
-                                                     const libxsmm_convolution_forward_descriptor* i_conv_desc,
-                                                     const char*                       i_arch);
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_forward_directasm(const char*                       i_file_out,
-                                                     const char*                       i_routine_name,
-                                                     const libxsmm_convolution_forward_descriptor* i_conv_desc,
-                                                     const char*                       i_arch);
+                                         int                              i_arch );
 
 /* @TODO change int based architecture value */
 LIBXSMM_API
@@ -229,45 +274,10 @@ void libxsmm_generator_convolution_weight_update_kernel(libxsmm_generated_code*
                                                         const libxsmm_convolution_weight_update_descriptor* i_conv_desc,
                                                         const char*                       i_arch);
 
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_winograd_weight_update_kernel(libxsmm_generated_code*                        io_generated_code,
-                                                                 const libxsmm_convolution_winograd_descriptor* i_conv_desc,
-                                                                 const char*                                    i_arch);
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_winograd_weight_update_inlineasm(const char*                                    i_file_out,
-                                                                    const char*                                    i_routine_name,
-                                                                    const libxsmm_convolution_winograd_descriptor* i_conv_desc,
-                                                                    const char*                                    i_arch);
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_winograd_weight_update_directasm(const char*                                    i_file_out,
-                                                                    const char*                                    i_routine_name,
-                                                                    const libxsmm_convolution_winograd_descriptor* i_conv_desc,
-                                                                    const char*                                    i_arch);
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_winograd_forward_kernel(libxsmm_generated_code*                        io_generated_code,
-                                                           const libxsmm_convolution_winograd_descriptor* i_conv_desc,
-                                                           const char*                                    i_arch);
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_winograd_forward_inlineasm(const char*                                    i_file_out,
-                                                              const char*                                    i_routine_name,
-                                                              const libxsmm_convolution_winograd_descriptor* i_conv_desc,
-                                                              const char*                                    i_arch);
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_winograd_forward_directasm(const char*                                    i_file_out,
-                                                              const char*                                    i_routine_name,
-                                                              const libxsmm_convolution_winograd_descriptor* i_conv_desc,
-                                                              const char*                                    i_arch);
+/** Initialization counter that can be used to check whether the library is initialized (!=0) or not (==0). */
+LIBXSMM_APIVAR_ALIGNED(unsigned int libxsmm_ninit);
+/** Verbosity level (0: quiet, 1: errors, 2: warnings, 3: info, neg.: all/dump). */
+LIBXSMM_APIVAR_ALIGNED(int libxsmm_verbosity);
 
 #endif /*LIBXSMM_GENERATOR_H*/
 
