@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2014-2018, Intel Corporation                                **
+** Copyright (c) 2014-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -34,7 +34,7 @@
 #include "libxsmm_intrinsics_x86.h"
 
 #if !defined(LIBXSMM_TLS)
-# if !defined(LIBXSMM_NO_SYNC) && !defined(LIBXSMM_NO_TLS)
+# if (0 != LIBXSMM_SYNC) && !defined(LIBXSMM_NO_TLS)
 #   if defined(__CYGWIN__) && defined(__clang__)
 #     define LIBXSMM_NO_TLS
 #     define LIBXSMM_TLS
@@ -57,22 +57,26 @@
 # endif
 #endif
 
-#if !defined(LIBXSMM_GCC_BASELINE) && defined(__GNUC__) && \
+#if !defined(LIBXSMM_GCC_BASELINE) && !defined(LIBXSMM_SYNC_LEGACY) && defined(__GNUC__) && \
   LIBXSMM_VERSION3(4, 7, 0) <= LIBXSMM_VERSION3(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__)
 # define LIBXSMM_GCC_BASELINE
 #endif
 
 #if defined(__MIC__)
 # define LIBXSMM_SYNC_PAUSE _mm_delay_32(8/*delay*/)
-#elif !defined(LIBXSMM_INTRINSICS_NONE) && !defined(LIBXSMM_INTRINSICS_STATIC)
-# define LIBXSMM_SYNC_PAUSE _mm_pause()
-#elif defined(LIBXSMM_GCC_BASELINE) && !defined(__PGI)
-# define LIBXSMM_SYNC_PAUSE __builtin_ia32_pause()
+#elif !defined(LIBXSMM_INTRINSICS_NONE)
+# if defined(LIBXSMM_GCC_BASELINE) && !defined(__INTEL_COMPILER)
+#   define LIBXSMM_SYNC_PAUSE __builtin_ia32_pause()
+# else
+#   define LIBXSMM_SYNC_PAUSE _mm_pause()
+# endif
 #else
 # define LIBXSMM_SYNC_PAUSE
 #endif
 
-#if !defined(LIBXSMM_SYNC_SYSTEM) && (defined(__MINGW32__) || defined(__PGI))
+#if !defined(LIBXSMM_SYNC_SYSTEM) && \
+  (defined(__MINGW32__) || defined(__PGI) || \
+  (defined(_CRAYC) && !defined(__GNUC__)))
 # define LIBXSMM_SYNC_SYSTEM
 #endif
 #if !defined(LIBXSMM_ATOMIC_TRYLOCK_CMPSWP) && 0
@@ -81,20 +85,23 @@
 #if !defined(LIBXSMM_ATOMIC_ZERO_STORE) && defined(_CRAYC)
 # define LIBXSMM_ATOMIC_ZERO_STORE
 #endif
-#if defined(__ATOMIC_RELAXED)
-# define LIBXSMM_ATOMIC_RELAXED __ATOMIC_RELAXED
-#else
-# define LIBXSMM_ATOMIC_RELAXED 0
-#endif
-#if defined(__ATOMIC_SEQ_CST)
-# define LIBXSMM_ATOMIC_SEQ_CST __ATOMIC_SEQ_CST
-#else
-# define LIBXSMM_ATOMIC_SEQ_CST 0
-#endif
 #if !defined(LIBXSMM_ATOMIC_LOCKTYPE)
 # define LIBXSMM_ATOMIC_LOCKTYPE char
 #endif
 
+typedef enum libxsmm_atomic_kind {
+#if defined(__ATOMIC_SEQ_CST)
+  LIBXSMM_ATOMIC_SEQ_CST = __ATOMIC_SEQ_CST,
+#else
+  LIBXSMM_ATOMIC_SEQ_CST = 0,
+#endif
+#if defined(__ATOMIC_RELAXED)
+  LIBXSMM_ATOMIC_RELAXED = __ATOMIC_RELAXED
+#else
+  LIBXSMM_ATOMIC_RELAXED = LIBXSMM_ATOMIC_SEQ_CST
+#endif
+} libxsmm_atomic_kind;
+
 #define LIBXSMM_NONATOMIC_LOCKTYPE LIBXSMM_ATOMIC_LOCKTYPE
 #define LIBXSMM_NONATOMIC_LOAD(SRC_PTR, KIND) (*(SRC_PTR))
 #define LIBXSMM_NONATOMIC_STORE(DST_PTR, VALUE, KIND) { LIBXSMM_UNUSED(KIND); *(DST_PTR) = VALUE; }
@@ -109,14 +116,14 @@
 #define LIBXSMM_NONATOMIC_CMPSWP(DST_PTR, OLDVAL, NEWVAL, KIND) ((NEWVAL) == (*(DST_PTR) == (OLDVAL) ? (*(DST_PTR) = (NEWVAL)) : (OLDVAL)))
 #define LIBXSMM_NONATOMIC_TRYLOCK(DST_PTR, KIND) LIBXSMM_NONATOMIC_CMPSWP(DST_PTR, 0, 1, KIND)
 #define LIBXSMM_NONATOMIC_ACQUIRE(DST_PTR, NPAUSE, KIND) { LIBXSMM_UNUSED(NPAUSE); \
-          LIBXSMM_ASSERT(0 == *(DST_PTR) && *"LIBXSMM_NONATOMIC_ACQUIRE"); LIBXSMM_NONATOMIC_STORE(DST_PTR, 1, KIND); \
-          LIBXSMM_ASSERT(0 != *(DST_PTR) && *"LIBXSMM_NONATOMIC_ACQUIRE"); }
+          LIBXSMM_ASSERT_MSG(0 == *(DST_PTR), "LIBXSMM_NONATOMIC_ACQUIRE"); LIBXSMM_NONATOMIC_STORE(DST_PTR, 1, KIND); \
+          LIBXSMM_ASSERT_MSG(0 != *(DST_PTR), "LIBXSMM_NONATOMIC_ACQUIRE"); }
 #define LIBXSMM_NONATOMIC_RELEASE(DST_PTR, KIND) { LIBXSMM_UNUSED(DST_PTR); LIBXSMM_UNUSED(KIND); \
-          LIBXSMM_ASSERT(0 != *(DST_PTR) && *"LIBXSMM_NONATOMIC_RELEASE"); LIBXSMM_NONATOMIC_STORE(DST_PTR, 0, KIND); \
-          LIBXSMM_ASSERT(0 == *(DST_PTR) && *"LIBXSMM_NONATOMIC_RELEASE"); }
+          LIBXSMM_ASSERT_MSG(0 != *(DST_PTR), "LIBXSMM_NONATOMIC_RELEASE"); LIBXSMM_NONATOMIC_STORE(DST_PTR, 0, KIND); \
+          LIBXSMM_ASSERT_MSG(0 == *(DST_PTR), "LIBXSMM_NONATOMIC_RELEASE"); }
 #define LIBXSMM_NONATOMIC_SYNC(KIND) LIBXSMM_UNUSED(KIND)
 
-#if defined(LIBXSMM_NO_SYNC)
+#if (0 == LIBXSMM_SYNC)
 # define LIBXSMM_ATOMIC(FN, BITS) FN
 # define LIBXSMM_ATOMIC_LOAD LIBXSMM_NONATOMIC_LOAD
 # define LIBXSMM_ATOMIC_STORE LIBXSMM_NONATOMIC_STORE
@@ -136,7 +143,8 @@
 #   define LIBXSMM_SYNC_NPAUSE 0
 # endif
 #else
-# if defined(__GNUC__) && !defined(LIBXSMM_SYNC_SYSTEM)
+# if defined(__GNUC__) && !defined(LIBXSMM_SYNC_SYSTEM) && /* check for legacy GCC (no atomics) */ \
+  LIBXSMM_VERSION3(4, 1, 0) <= LIBXSMM_VERSION3(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__)
 #   define LIBXSMM_ATOMIC(FN, BITS) FN
 #   if defined(LIBXSMM_GCC_BASELINE)
 #     define LIBXSMM_ATOMIC_LOAD(SRC_PTR, KIND) __atomic_load_n(SRC_PTR, KIND)
@@ -159,10 +167,10 @@
 #       define LIBXSMM_ATOMIC_TRYLOCK(DST_PTR, KIND) (!__atomic_test_and_set(DST_PTR, KIND))
 #     endif
 #     if defined(__PGI) /* ICE: __atomic_clear not implemented */
-#       define LIBXSMM_ATOMIC_RELEASE(DST_PTR, KIND) { LIBXSMM_ASSERT(0 != *(DST_PTR) && *"LIBXSMM_ATOMIC_RELEASE"); \
+#       define LIBXSMM_ATOMIC_RELEASE(DST_PTR, KIND) { LIBXSMM_ASSERT_MSG(0 != *(DST_PTR), "LIBXSMM_ATOMIC_RELEASE"); \
                 LIBXSMM_ATOMIC_STORE_ZERO(DST_PTR, KIND); }
 #     else
-#       define LIBXSMM_ATOMIC_RELEASE(DST_PTR, KIND) { LIBXSMM_ASSERT(0 != *(DST_PTR) && *"LIBXSMM_ATOMIC_RELEASE"); \
+#       define LIBXSMM_ATOMIC_RELEASE(DST_PTR, KIND) { LIBXSMM_ASSERT_MSG(0 != *(DST_PTR), "LIBXSMM_ATOMIC_RELEASE"); \
                 __atomic_clear(DST_PTR, KIND); }
 #     endif
 #     if 0 /* __atomic_thread_fence: incorrect behavior in libxsmm_barrier (even with LIBXSMM_ATOMIC_SEQ_CST) */
@@ -187,7 +195,7 @@
 #     if defined(LIBXSMM_ATOMIC_TRYLOCK_CMPSWP)
 #       define LIBXSMM_ATOMIC_TRYLOCK(DST_PTR, KIND) (0 == __sync_lock_test_and_set(DST_PTR, 1))
 #     endif
-#     define LIBXSMM_ATOMIC_RELEASE(DST_PTR, KIND) { LIBXSMM_ASSERT(0 != *(DST_PTR) && *"LIBXSMM_ATOMIC_RELEASE"); \
+#     define LIBXSMM_ATOMIC_RELEASE(DST_PTR, KIND) { LIBXSMM_ASSERT_MSG(0 != *(DST_PTR), "LIBXSMM_ATOMIC_RELEASE"); \
               __sync_lock_release(DST_PTR); }
 #     define LIBXSMM_ATOMIC_SYNC(KIND) __sync_synchronize()
 #   endif
@@ -200,7 +208,7 @@
 #   define LIBXSMM_ATOMIC_ACQUIRE(DST_PTR, NPAUSE, KIND) { LIBXSMM_SYNC_CYCLE_DECL(libxsmm_atomic_acquire_counter_); \
             LIBXSMM_ASSERT(1 == sizeof(LIBXSMM_ATOMIC_LOCKTYPE)); LIBXSMM_ASSERT(0 == LIBXSMM_MOD2((uintptr_t)(DST_PTR), 4)); \
             while (!LIBXSMM_ATOMIC_TRYLOCK(DST_PTR, KIND)) LIBXSMM_SYNC_CYCLE(libxsmm_atomic_acquire_counter_, NPAUSE); \
-            LIBXSMM_ASSERT(0 != *(DST_PTR) && *"LIBXSMM_ATOMIC_ACQUIRE"); }
+            LIBXSMM_ASSERT_MSG(0 != *(DST_PTR), "LIBXSMM_ATOMIC_ACQUIRE"); }
 #   define LIBXSMM_ATOMIC_SYNC_NOFENCE(KIND) __asm__ __volatile__ ("" ::: "memory")
 #   if !defined(LIBXSMM_SYNC_NPAUSE)
 #     define LIBXSMM_SYNC_NPAUSE 4096
@@ -230,11 +238,11 @@
 #   define LIBXSMM_ATOMIC_FETCH_OR8(DST_PTR, VALUE, KIND) _InterlockedOr8((volatile char*)(DST_PTR), VALUE)
 #   define LIBXSMM_ATOMIC_ADD_FETCH(DST_PTR, VALUE, KIND) (LIBXSMM_ATOMIC_FETCH_ADD(DST_PTR, VALUE, KIND) + (VALUE))
 #   define LIBXSMM_ATOMIC_ADD_FETCH64(DST_PTR, VALUE, KIND) (LIBXSMM_ATOMIC_FETCH_ADD64(DST_PTR, VALUE, KIND) + (VALUE))
-#   define LIBXSMM_ATOMIC_SUB_FETCH(DST_PTR, VALUE, KIND) (LIBXSMM_ATOMIC_FETCH_SUB(DST_PTR, VALUE, KIND) - (VALUE))
+#   define LIBXSMM_ATOMIC_SUB_FETCH(DST_PTR, VALUE, KIND) ((size_t)LIBXSMM_ATOMIC_FETCH_SUB(DST_PTR, VALUE, KIND) - ((size_t)VALUE))
 #   define LIBXSMM_ATOMIC_SUB_FETCH64(DST_PTR, VALUE, KIND) (LIBXSMM_ATOMIC_FETCH_SUB64(DST_PTR, VALUE, KIND) - (VALUE))
 #   define LIBXSMM_ATOMIC_FETCH_ADD(DST_PTR, VALUE, KIND) InterlockedExchangeAdd((volatile LONG*)(DST_PTR), VALUE)
 #   define LIBXSMM_ATOMIC_FETCH_ADD64(DST_PTR, VALUE, KIND) InterlockedExchangeAdd64((volatile LONGLONG*)(DST_PTR), VALUE)
-#   define LIBXSMM_ATOMIC_FETCH_SUB(DST_PTR, VALUE, KIND) LIBXSMM_ATOMIC_FETCH_ADD(DST_PTR, -(VALUE), KIND)
+#   define LIBXSMM_ATOMIC_FETCH_SUB(DST_PTR, VALUE, KIND) LIBXSMM_ATOMIC_FETCH_ADD(DST_PTR, -1 * (VALUE), KIND)
 #   define LIBXSMM_ATOMIC_CMPSWP(DST_PTR, OLDVAL, NEWVAL, KIND) (((LONG)(OLDVAL)) == InterlockedCompareExchange((volatile LONG*)(DST_PTR), NEWVAL, OLDVAL))
 #   define LIBXSMM_ATOMIC_CMPSWP8(DST_PTR, OLDVAL, NEWVAL, KIND) ((OLDVAL) == _InterlockedCompareExchange8((volatile char*)(DST_PTR), NEWVAL, OLDVAL))
 #   if defined(LIBXSMM_ATOMIC_TRYLOCK_CMPSWP)
@@ -245,9 +253,9 @@
 #   define LIBXSMM_ATOMIC_ACQUIRE(DST_PTR, NPAUSE, KIND) { LIBXSMM_SYNC_CYCLE_DECL(libxsmm_atomic_acquire_counter_); \
             LIBXSMM_ASSERT(1 == sizeof(LIBXSMM_ATOMIC_LOCKTYPE)); LIBXSMM_ASSERT(0 == LIBXSMM_MOD2((uintptr_t)(DST_PTR), 4)); \
             while (!LIBXSMM_ATOMIC_TRYLOCK(DST_PTR, KIND)) LIBXSMM_SYNC_CYCLE(libxsmm_atomic_acquire_counter_, NPAUSE); \
-            LIBXSMM_ASSERT(0 != *(DST_PTR) && *"LIBXSMM_ATOMIC_ACQUIRE"); }
+            LIBXSMM_ASSERT_MSG(0 != *(DST_PTR), "LIBXSMM_ATOMIC_ACQUIRE"); }
 #   define LIBXSMM_ATOMIC_RELEASE(DST_PTR, KIND) { \
-            LIBXSMM_ASSERT(0 != *(DST_PTR) && *"LIBXSMM_ATOMIC_RELEASE"); \
+            LIBXSMM_ASSERT_MSG(0 != *(DST_PTR), "LIBXSMM_ATOMIC_RELEASE"); \
             LIBXSMM_ATOMIC(LIBXSMM_ATOMIC_STORE_ZERO, 8)(DST_PTR, KIND); }
 #   define LIBXSMM_ATOMIC_SYNC(KIND) _ReadWriteBarrier()
 #   if !defined(LIBXSMM_SYNC_NPAUSE)
@@ -292,8 +300,12 @@
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
-#if !defined(LIBXSMM_NO_SYNC) /** Default lock-kind */
-# define LIBXSMM_LOCK_DEFAULT LIBXSMM_LOCK_SPINLOCK
+#if (0 != LIBXSMM_SYNC) /** Default lock-kind */
+# if defined(_MSC_VER)
+#   define LIBXSMM_LOCK_DEFAULT LIBXSMM_LOCK_SPINLOCK
+# else
+#   define LIBXSMM_LOCK_DEFAULT LIBXSMM_LOCK_MUTEX
+# endif
 # if !defined(LIBXSMM_LOCK_SYSTEM_SPINLOCK) && (defined(LIBXSMM_SYNC_SYSTEM) || 1)
 #   define LIBXSMM_LOCK_SYSTEM_SPINLOCK
 # endif
@@ -306,6 +318,7 @@
   /* Lock type, initialization, destruction, (try-)lock, unlock, etc */
 # define LIBXSMM_LOCK_ACQUIRED(KIND) LIBXSMM_CONCATENATE(LIBXSMM_LOCK_ACQUIRED_, KIND)
 # define LIBXSMM_LOCK_TYPE_ISPOD(KIND) LIBXSMM_CONCATENATE(LIBXSMM_LOCK_TYPE_ISPOD_, KIND)
+# define LIBXSMM_LOCK_TYPE_ISRW(KIND) LIBXSMM_CONCATENATE(LIBXSMM_LOCK_TYPE_ISRW_, KIND)
 # define LIBXSMM_LOCK_TYPE(KIND) LIBXSMM_CONCATENATE(LIBXSMM_LOCK_TYPE_, KIND)
 # define LIBXSMM_LOCK_INIT(KIND, LOCK, ATTR) LIBXSMM_CONCATENATE(LIBXSMM_LOCK_INIT_, KIND)(LOCK, ATTR)
 # define LIBXSMM_LOCK_DESTROY(KIND, LOCK) LIBXSMM_CONCATENATE(LIBXSMM_LOCK_DESTROY_, KIND)(LOCK)
@@ -362,6 +375,7 @@
 #   endif
 #   if defined(LIBXSMM_LOCK_SYSTEM_MUTEX) && !(defined(_OPENMP) && defined(LIBXSMM_OMP))
 #     define LIBXSMM_LOCK_TYPE_ISPOD_mutex 0
+#     define LIBXSMM_LOCK_TYPE_ISRW_mutex 0
 #     define LIBXSMM_LOCK_TYPE_mutex HANDLE
 #     define LIBXSMM_LOCK_INIT_mutex(LOCK, ATTR) (*(LOCK) = CreateMutex(*(ATTR), FALSE, NULL))
 #     define LIBXSMM_LOCK_DESTROY_mutex(LOCK) CloseHandle(*(LOCK))
@@ -380,6 +394,7 @@
 #   endif
 #   if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && !(defined(_OPENMP) && defined(LIBXSMM_OMP))
 #     define LIBXSMM_LOCK_TYPE_ISPOD_rwlock 1
+#     define LIBXSMM_LOCK_TYPE_ISRW_rwlock 1
 #     define LIBXSMM_LOCK_TYPE_rwlock SRWLOCK
 #     define LIBXSMM_LOCK_INIT_rwlock(LOCK, ATTR) { LIBXSMM_UNUSED(ATTR); InitializeSRWLock(LOCK); }
 #     define LIBXSMM_LOCK_DESTROY_rwlock(LOCK) LIBXSMM_UNUSED(LOCK)
@@ -416,6 +431,7 @@
 #   endif
 #   if defined(LIBXSMM_LOCK_SYSTEM_SPINLOCK) && !(defined(_OPENMP) && defined(LIBXSMM_OMP))
 #     define LIBXSMM_LOCK_TYPE_ISPOD_spin 0
+#     define LIBXSMM_LOCK_TYPE_ISRW_spin 0
 #     define LIBXSMM_LOCK_TYPE_spin pthread_spinlock_t
 #     define LIBXSMM_LOCK_INIT_spin(LOCK, ATTR) LIBXSMM_EXPECT(0, pthread_spin_init(LOCK, *(ATTR)))
 #     define LIBXSMM_LOCK_DESTROY_spin(LOCK) LIBXSMM_EXPECT(0, pthread_spin_destroy(LOCK))
@@ -434,6 +450,7 @@
 #   endif
 #   if defined(LIBXSMM_LOCK_SYSTEM_MUTEX) && !(defined(_OPENMP) && defined(LIBXSMM_OMP))
 #     define LIBXSMM_LOCK_TYPE_ISPOD_mutex 0
+#     define LIBXSMM_LOCK_TYPE_ISRW_mutex 0
 #     define LIBXSMM_LOCK_TYPE_mutex pthread_mutex_t
 #     define LIBXSMM_LOCK_INIT_mutex(LOCK, ATTR) LIBXSMM_EXPECT(0, pthread_mutex_init(LOCK, ATTR))
 #     define LIBXSMM_LOCK_DESTROY_mutex(LOCK) LIBXSMM_EXPECT(0, pthread_mutex_destroy(LOCK))
@@ -458,6 +475,7 @@
 #   endif
 #   if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && !(defined(_OPENMP) && defined(LIBXSMM_OMP))
 #     define LIBXSMM_LOCK_TYPE_ISPOD_rwlock 0
+#     define LIBXSMM_LOCK_TYPE_ISRW_rwlock 1
 #     define LIBXSMM_LOCK_TYPE_rwlock pthread_rwlock_t
 #     define LIBXSMM_LOCK_INIT_rwlock(LOCK, ATTR) LIBXSMM_EXPECT(0, pthread_rwlock_init(LOCK, ATTR))
 #     define LIBXSMM_LOCK_DESTROY_rwlock(LOCK) LIBXSMM_EXPECT(0, pthread_rwlock_destroy(LOCK))
@@ -481,6 +499,7 @@
 # if !defined(LIBXSMM_LOCK_SYSTEM_SPINLOCK)
 #   if 1 /* directly based on atomic primitives */
 #     define LIBXSMM_LOCK_TYPE_ISPOD_spin 1
+#     define LIBXSMM_LOCK_TYPE_ISRW_spin 0
 #     define LIBXSMM_LOCK_TYPE_spin volatile LIBXSMM_ATOMIC_LOCKTYPE
 #     define LIBXSMM_LOCK_INIT_spin(LOCK, ATTR) { LIBXSMM_UNUSED(ATTR); (*(LOCK) = 0); }
 #     define LIBXSMM_LOCK_DESTROY_spin(LOCK) LIBXSMM_UNUSED(LOCK)
@@ -495,6 +514,7 @@
 #     define LIBXSMM_LOCK_ATTR_DESTROY_spin(ATTR) LIBXSMM_UNUSED(ATTR)
 #   else /* rely on LIBXSMM's portable locks */
 #     define LIBXSMM_LOCK_TYPE_ISPOD_spin 0
+#     define LIBXSMM_LOCK_TYPE_ISRW_spin 0
 #     define LIBXSMM_LOCK_TYPE_spin libxsmm_spinlock*
 #     define LIBXSMM_LOCK_INIT_spin(LOCK, ATTR) { LIBXSMM_UNUSED(ATTR); (*(LOCK) = libxsmm_spinlock_create()); }
 #     define LIBXSMM_LOCK_DESTROY_spin(LOCK) libxsmm_spinlock_destroy(*(LOCK))
@@ -511,6 +531,7 @@
 # elif defined(_OPENMP) && defined(LIBXSMM_OMP)
 #   define LIBXSMM_LOCK_ACQUIRED_spin 1
 #   define LIBXSMM_LOCK_TYPE_ISPOD_spin 0
+#   define LIBXSMM_LOCK_TYPE_ISRW_spin 0
 #   define LIBXSMM_LOCK_TYPE_spin omp_lock_t
 #   define LIBXSMM_LOCK_INIT_spin(LOCK, ATTR) { LIBXSMM_UNUSED(ATTR); omp_init_lock(LOCK); }
 #   define LIBXSMM_LOCK_DESTROY_spin(LOCK) omp_destroy_lock(LOCK)
@@ -526,6 +547,7 @@
 # endif
 # if !defined(LIBXSMM_LOCK_SYSTEM_MUTEX)
 #   define LIBXSMM_LOCK_TYPE_ISPOD_mutex 0
+#   define LIBXSMM_LOCK_TYPE_ISRW_mutex 0
 #   define LIBXSMM_LOCK_TYPE_mutex libxsmm_mutex*
 #   define LIBXSMM_LOCK_INIT_mutex(LOCK, ATTR) { LIBXSMM_UNUSED(ATTR); (*(LOCK) = libxsmm_mutex_create()); }
 #   define LIBXSMM_LOCK_DESTROY_mutex(LOCK) libxsmm_mutex_destroy(*(LOCK))
@@ -541,6 +563,7 @@
 # elif defined(_OPENMP) && defined(LIBXSMM_OMP)
 #   define LIBXSMM_LOCK_ACQUIRED_mutex 1
 #   define LIBXSMM_LOCK_TYPE_ISPOD_mutex 0
+#   define LIBXSMM_LOCK_TYPE_ISRW_mutex 0
 #   define LIBXSMM_LOCK_TYPE_mutex omp_lock_t
 #   define LIBXSMM_LOCK_INIT_mutex(LOCK, ATTR) { LIBXSMM_UNUSED(ATTR); omp_init_lock(LOCK); }
 #   define LIBXSMM_LOCK_DESTROY_mutex(LOCK) omp_destroy_lock(LOCK)
@@ -556,6 +579,7 @@
 # endif
 # if !defined(LIBXSMM_LOCK_SYSTEM_RWLOCK)
 #   define LIBXSMM_LOCK_TYPE_ISPOD_rwlock 0
+#   define LIBXSMM_LOCK_TYPE_ISRW_rwlock 1
 #   define LIBXSMM_LOCK_TYPE_rwlock libxsmm_rwlock*
 #   define LIBXSMM_LOCK_INIT_rwlock(LOCK, ATTR) { LIBXSMM_UNUSED(ATTR); (*(LOCK) = libxsmm_rwlock_create()); }
 #   define LIBXSMM_LOCK_DESTROY_rwlock(LOCK) libxsmm_rwlock_destroy(*(LOCK))
@@ -571,6 +595,7 @@
 # elif defined(_OPENMP) && defined(LIBXSMM_OMP)
 #   define LIBXSMM_LOCK_ACQUIRED_rwlock 1
 #   define LIBXSMM_LOCK_TYPE_ISPOD_rwlock 0
+#   define LIBXSMM_LOCK_TYPE_ISRW_rwlock 0
 #   define LIBXSMM_LOCK_TYPE_rwlock omp_lock_t
 #   define LIBXSMM_LOCK_INIT_rwlock(LOCK, ATTR) { LIBXSMM_UNUSED(ATTR); omp_init_lock(LOCK); }
 #   define LIBXSMM_LOCK_DESTROY_rwlock(LOCK) omp_destroy_lock(LOCK)
@@ -589,6 +614,8 @@
 # define LIBXSMM_LOCK_MUTEX
 # define LIBXSMM_LOCK_RWLOCK
 # define LIBXSMM_LOCK_ACQUIRED(KIND) 0
+# define LIBXSMM_LOCK_TYPE_ISPOD(KIND) 1
+# define LIBXSMM_LOCK_TYPE_ISRW(KIND) 0
 # define LIBXSMM_LOCK_ATTR_TYPE(KIND) int
 # define LIBXSMM_LOCK_ATTR_INIT(KIND, ATTR) LIBXSMM_UNUSED(ATTR)
 # define LIBXSMM_LOCK_ATTR_DESTROY(KIND, ATTR) LIBXSMM_UNUSED(ATTR)
