@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2009-2018, Intel Corporation                                **
+** Copyright (c) 2009-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -33,6 +33,8 @@
 
 #include "libxsmm_macros.h"
 
+#define libxsmm_timer_diff(TICK0, TICK1) libxsmm_timer_cycles(TICK0, TICK1)
+
 
 typedef unsigned long long libxsmm_timer_tickint;
 
@@ -42,10 +44,8 @@ typedef unsigned long long libxsmm_timer_tickint;
  */
 LIBXSMM_API libxsmm_timer_tickint libxsmm_timer_tick(void);
 
-/** Helper function to receive the difference between two timer ticks; avoids potential side-effects of LIBXSMM_DIFF. */
-LIBXSMM_API_INLINE libxsmm_timer_tickint libxsmm_timer_diff(libxsmm_timer_tickint tick0, libxsmm_timer_tickint tick1) {
-  return LIBXSMM_DIFF(tick0, tick1);
-}
+/** Returns the difference between two timer ticks (cycles); avoids potential side-effects of LIBXSMM_DIFF. */
+LIBXSMM_API libxsmm_timer_tickint libxsmm_timer_cycles(libxsmm_timer_tickint tick0, libxsmm_timer_tickint tick1);
 
 /** Returns the duration (in seconds) between two values received by libxsmm_timer_tick. */
 LIBXSMM_API double libxsmm_timer_duration(libxsmm_timer_tickint tick0, libxsmm_timer_tickint tick1);
