@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -51,21 +51,41 @@
 #define LIBXSMM_PREFETCH_SIGONLY 1
 #define LIBXSMM_PREFETCH_NONE 0
 
-/** Helper macros for type postfixes. */
-#define LIBXSMM_TPOSTFIX_NAME(TYPE) LIBXSMM_CONCATENATE(LIBXSMM_TPOSTFIX_, TYPE)
-#define LIBXSMM_TPOSTFIX(TYPE, SYMBOL) LIBXSMM_CONCATENATE(SYMBOL, LIBXSMM_TPOSTFIX_NAME(TYPE))
-#define LIBXSMM_TPOSTFIX_double F64
-#define LIBXSMM_TPOSTFIX_float F32
-#define LIBXSMM_TPOSTFIX_int I32
-#define LIBXSMM_TPOSTFIX_short I16
+/** Helper macro for type names. */
+#define LIBXSMM_TYPENAME(TYPE) LIBXSMM_STRINGIFY(LIBXSMM_CONCATENATE(LIBXSMM_TYPENAME_, TYPE))
+#define LIBXSMM_TYPENAME_double f64
+#define LIBXSMM_TYPENAME_float f32
+#define LIBXSMM_TYPENAME_libxsmm_bfloat16 bf16
+#define LIBXSMM_TYPENAME_int i32
+#define LIBXSMM_TYPENAME_short i16
+#define LIBXSMM_TYPENAME_char i8
+
+/** Helper macro for type information: INFO := { FP }. */
+#define LIBXSMM_TYPEINFO(TYPE, INFO) LIBXSMM_CONCATENATE3(LIBXSMM_TYPEINFO_, INFO, _, TYPE)
+#define LIBXSMM_TYPEINFO_FP_double 1
+#define LIBXSMM_TYPEINFO_FP_float 1
+#define LIBXSMM_TYPEINFO_FP_libxsmm_bfloat16 1
+#define LIBXSMM_TYPEINFO_FP_int 0
+#define LIBXSMM_TYPEINFO_FP_short 0
+#define LIBXSMM_TYPEINFO_FP_char 0
+
+/** Helper macro for type postfixes. */
+#define LIBXSMM_TYPESYMBOL(TYPE) LIBXSMM_CONCATENATE(LIBXSMM_TYPESYMBOL_, TYPE)
+#define LIBXSMM_TYPESYMBOL_double F64
+#define LIBXSMM_TYPESYMBOL_float F32
+#define LIBXSMM_TYPESYMBOL_libxsmm_bfloat16 BF16
+#define LIBXSMM_TYPESYMBOL_int I32
+#define LIBXSMM_TYPESYMBOL_short I16
+#define LIBXSMM_TYPESYMBOL_char I8
 
 #define LIBXSMM_TYPESIZE(ENUM) ( \
-  ((int)(ENUM)) == LIBXSMM_DATATYPE_F64 ? 8 : ( \
-  ((int)(ENUM)) == LIBXSMM_DATATYPE_F32 ? 4 : ( \
-  ((int)(ENUM)) == LIBXSMM_DATATYPE_I32 ? 4 : ( \
-  ((int)(ENUM)) == LIBXSMM_DATATYPE_I16 ? 2 : ( \
-  ((int)(ENUM)) == LIBXSMM_DATATYPE_I8  ? 1 : ( \
-  0/*invalid*/))))))
+  ((int)(ENUM)) == LIBXSMM_DATATYPE_F64  ? 8 : ( \
+  ((int)(ENUM)) == LIBXSMM_DATATYPE_F32  ? 4 : ( \
+  ((int)(ENUM)) == LIBXSMM_DATATYPE_BF16 ? 2 : ( \
+  ((int)(ENUM)) == LIBXSMM_DATATYPE_I32  ? 4 : ( \
+  ((int)(ENUM)) == LIBXSMM_DATATYPE_I16  ? 2 : ( \
+  ((int)(ENUM)) == LIBXSMM_DATATYPE_I8   ? 1 : ( \
+  0/*invalid*/)))))))
 
 /* Get input or output precision */
 #define LIBXSMM_GETENUM_INP(SRC) ((SRC) & 0x0F)
@@ -75,26 +95,48 @@
 #define LIBXSMM_SETENUM(DST, INP, OUT) DST = LIBXSMM_GETENUM(INP, OUT)
 
 /* Construct an enumerator (libxsmm_datatype) from a built-in type (float, double, etc.). */
-#define LIBXSMM_DATATYPE(TYPE) LIBXSMM_TPOSTFIX(TYPE, LIBXSMM_DATATYPE_)
+#define LIBXSMM_DATATYPE(TYPE) LIBXSMM_CONCATENATE(LIBXSMM_DATATYPE_, LIBXSMM_TYPESYMBOL(TYPE))
 /* Construct a type-id from built-in input/output types (float, double, etc.). */
 #define LIBXSMM_DATATYPE2(ITYPE, OTYPE) LIBXSMM_GETENUM(LIBXSMM_DATATYPE(ITYPE), LIBXSMM_DATATYPE(OTYPE))
 
 /* Construct an enumerator (libxsmm_gemm_precision) from a built-in type (float, double, etc.). */
-#define LIBXSMM_GEMM_PRECISION(TYPE) LIBXSMM_TPOSTFIX(TYPE, LIBXSMM_GEMM_PRECISION_)
+#define LIBXSMM_GEMM_PRECISION(TYPE) LIBXSMM_CONCATENATE(LIBXSMM_GEMM_PRECISION_, LIBXSMM_TYPESYMBOL(TYPE))
 /* Construct GEMM-precision from built-in input/output types (float, double, etc.). */
-#define LIBXSMM_GEMM_PRECISION2(ITYPE, OTYPE) LIBXSMM_GETENUM(LIBXSMM_GEMM_PRECISION(ITYPE), \
-                                                              LIBXSMM_GEMM_PRECISION(OTYPE))
+#define LIBXSMM_GEMM_PRECISION2(ITYPE, OTYPE) (libxsmm_gemm_precision)LIBXSMM_GETENUM( \
+  LIBXSMM_GEMM_PRECISION(ITYPE), LIBXSMM_GEMM_PRECISION(OTYPE))
 
-/** Necessary size to store a descriptor/blob (GEMM, MCOPY, TRANS). */
-#define LIBXSMM_GEMM_DESCRIPTOR_SIZE 32
+/** Maximum size available to store a descriptor/blob (GEMM, MCOPY, TRANS, TRSM, TRMM). */
+#if !defined(LIBXSMM_DESCRIPTOR_MAXSIZE)
+# define LIBXSMM_DESCRIPTOR_MAXSIZE 64
+#endif
+/** Size of the descriptor considered as unique signature. */
+#if !defined(LIBXSMM_DESCRIPTOR_SIGSIZE)
+# define LIBXSMM_DESCRIPTOR_SIGSIZE LIBXSMM_DESCRIPTOR_MAXSIZE
+#endif
 
 
+/* Support for Bfloat16 */
+typedef unsigned short libxsmm_bfloat16;
+
+LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE libxsmm_bfloat16_hp {
+  libxsmm_bfloat16 i[2];
+  float f;
+} libxsmm_bfloat16_hp;
+
+#if defined(__cplusplus)
+namespace tensorflow { struct bfloat16; }
+#endif /*__cplusplus*/
+
 /** Integer type for LAPACK/BLAS (LP64: 32-bit, and ILP64: 64-bit). */
 typedef LIBXSMM_BLASINT libxsmm_blasint;
 
+/** Type representing sufficient storage space for a GEMM handle. */
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_gemm_blob { char data[128]; } libxsmm_gemm_blob;
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_gemm_handle libxsmm_gemm_handle;
+
 /** Type representing sufficient storage space for descriptors (GEMM, TCOPY, MCOPY). */
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_descriptor_blob {
-  char data[LIBXSMM_GEMM_DESCRIPTOR_SIZE];
+  char data[LIBXSMM_DESCRIPTOR_MAXSIZE];
 } libxsmm_descriptor_blob;
 
 /** Structure storing arguments of GEMM-like routines. */
@@ -103,20 +145,32 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_gemm_descriptor lib
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_mcopy_descriptor libxsmm_mcopy_descriptor;
 /** Structure storing arguments of the transpose routine. */
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_trans_descriptor libxsmm_trans_descriptor;
+/** Structure storing arguments of packed TRSM. */
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_trsm_descriptor libxsmm_trsm_descriptor;
+/** Structure storing arguments of packed TRMM. */
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_trmm_descriptor libxsmm_trmm_descriptor;
+/** Structure storing arguments of packed GETRF. */
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_getrf_descriptor libxsmm_getrf_descriptor;
+/** Structure storing arguments of packed GEMM. */
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_pgemm_descriptor libxsmm_pgemm_descriptor;
 
 /** Enumerates element/data types. */
 typedef enum libxsmm_datatype {
   LIBXSMM_DATATYPE_F64,
   LIBXSMM_DATATYPE_F32,
+  LIBXSMM_DATATYPE_BF16,
+  LIBXSMM_DATATYPE_I64,
   LIBXSMM_DATATYPE_I32,
   LIBXSMM_DATATYPE_I16,
-  LIBXSMM_DATATYPE_I8
+  LIBXSMM_DATATYPE_I8,
+  LIBXSMM_DATATYPE_UNSUPPORTED
 } libxsmm_datatype;
 
 /** Denotes the precision/data type of GEMM. */
 typedef enum libxsmm_gemm_precision {
   LIBXSMM_GEMM_PRECISION_F64  = LIBXSMM_DATATYPE_F64,
   LIBXSMM_GEMM_PRECISION_F32  = LIBXSMM_DATATYPE_F32,
+  LIBXSMM_GEMM_PRECISION_BF16 = LIBXSMM_DATATYPE_BF16,
   LIBXSMM_GEMM_PRECISION_I32  = LIBXSMM_DATATYPE_I32,
   LIBXSMM_GEMM_PRECISION_I16  = LIBXSMM_DATATYPE_I16,
   LIBXSMM_GEMM_PRECISION_I8   = LIBXSMM_DATATYPE_I8
@@ -124,29 +178,48 @@ typedef enum libxsmm_gemm_precision {
 
 /** Flag enumeration which can be binary ORed. */
 typedef enum libxsmm_gemm_flags {
-  LIBXSMM_GEMM_FLAG_NONE    = 0,
+  LIBXSMM_GEMM_FLAG_NONE = 0,
   /** Transpose matrix A. */
   LIBXSMM_GEMM_FLAG_TRANS_A = 1,
   /** Transpose matrix B. */
   LIBXSMM_GEMM_FLAG_TRANS_B = 2,
+  /** Transpose matrix A and B. */
+  LIBXSMM_GEMM_FLAG_TRANS_AB = LIBXSMM_GEMM_FLAG_TRANS_A | LIBXSMM_GEMM_FLAG_TRANS_B,
 #if 0
   /** Alpha=0|1 */
   LIBXSMM_GEMM_FLAG_ALPHA_0 = 4,
   /** Alpha=neg|pos */
   LIBXSMM_GEMM_FLAG_ALPHA_S = 8,
+#endif
   /** Beta=0|1 */
-  LIBXSMM_GEMM_FLAG_BETA_0  = 16,
+  LIBXSMM_GEMM_FLAG_BETA_0 = 16,
+#if 0
   /** Beta=neg|pos */
-  LIBXSMM_GEMM_FLAG_BETA_S  = 32,
+  LIBXSMM_GEMM_FLAG_BETA_S = 32,
 #endif
   /** Generate aligned load instructions. */
   LIBXSMM_GEMM_FLAG_ALIGN_A = 64,
   /** Aligned load/store instructions. */
   LIBXSMM_GEMM_FLAG_ALIGN_C = 128,
+  /** Batch-reduce Ai * Bi. */
+  LIBXSMM_GEMM_FLAG_BATCH_REDUCE = 256,
+  /** Aligned C matrix, but using NTS Hint when storing */
+  LIBXSMM_GEMM_FLAG_ALIGN_C_NTS_HINT = 640,
+  LIBXSMM_GEMM_FLAG_ALIGN_C_NTS_HINT_BATCH_REDUCE        = LIBXSMM_GEMM_FLAG_BATCH_REDUCE | LIBXSMM_GEMM_FLAG_ALIGN_C_NTS_HINT,
+  LIBXSMM_GEMM_FLAG_ALIGN_C_NTS_HINT_BETA_0              = LIBXSMM_GEMM_FLAG_BETA_0       | LIBXSMM_GEMM_FLAG_ALIGN_C_NTS_HINT,
+  LIBXSMM_GEMM_FLAG_ALIGN_C_NTS_HINT_BETA_0_BATCH_REDUCE = LIBXSMM_GEMM_FLAG_BETA_0       | LIBXSMM_GEMM_FLAG_ALIGN_C_NTS_HINT | LIBXSMM_GEMM_FLAG_BATCH_REDUCE,
   /** Marker flag; do not use. */
-  LIBXSMM_GEMM_FLAG_INVALID = 256
+  LIBXSMM_GEMM_FLAG_INVALID = 1024
 } libxsmm_gemm_flags;
 
+/** Flag enumeration which can be binary ORed. */
+typedef enum libxsmm_gemm_handle_flags {
+  LIBXSMM_GEMM_HANDLE_FLAG_AUTO   = 0,
+  LIBXSMM_GEMM_HANDLE_FLAG_COPY_A = 1,
+  LIBXSMM_GEMM_HANDLE_FLAG_COPY_B = 2,
+  LIBXSMM_GEMM_HANDLE_FLAG_COPY_C = 4
+} libxsmm_gemm_handle_flags;
+
 /** Auto-batch flags (can be ORed) applicable to mmbatch_begin/mmbatch_end. */
 typedef enum libxsmm_mmbatch_flags {
   /** Handle recorded batch unsynchronized-parallel. */
@@ -238,15 +311,19 @@ typedef enum libxsmm_convolution_prefetch_type {
 
 typedef enum libxsmm_dnn_tensor_format {
   /* use LIBXSMM internal format, we need to copy data into that */
-  LIBXSMM_DNN_TENSOR_FORMAT_LIBXSMM = 1,
+  LIBXSMM_DNN_TENSOR_FORMAT_LIBXSMM  = 1,
   /* use NHWC format internally, this allows no-copy operations */
-  LIBXSMM_DNN_TENSOR_FORMAT_NHWC = 2,
+  LIBXSMM_DNN_TENSOR_FORMAT_NHWC     = 2,
   /* use NCHW format internally, this will include shadow copies, not preferred */
-  LIBXSMM_DNN_TENSOR_FORMAT_NCHW = 4,
-  /* use RSCK format internally, this allows no-copy operations  */
-  LIBXSMM_DNN_TENSOR_FORMAT_RSCK = 8,
+  LIBXSMM_DNN_TENSOR_FORMAT_NCHW     = 4,
+  /* use RSCK format internally, this allows no-copy operations */
+  LIBXSMM_DNN_TENSOR_FORMAT_RSCK     = 8,
   /* use KCRS format internally, this will include shadow copies, not preferred */
-  LIBXSMM_DNN_TENSOR_FORMAT_KCRS = 16
+  LIBXSMM_DNN_TENSOR_FORMAT_KCRS     = 16,
+  LIBXSMM_DNN_TENSOR_FORMAT_CK       = 32,
+  LIBXSMM_DNN_TENSOR_FORMAT_CKPACKED = 64,
+  LIBXSMM_DNN_TENSOR_FORMAT_NCPACKED = 128,
+  LIBXSMM_DNN_TENSOR_FORMAT_NC       = 256
 } libxsmm_dnn_tensor_format;
 
 typedef enum libxsmm_dnn_internal_format {
@@ -262,6 +339,7 @@ typedef enum libxsmm_dnn_internal_format {
 typedef enum libxsmm_dnn_datatype {
   LIBXSMM_DNN_DATATYPE_F64  = LIBXSMM_DATATYPE_F64,
   LIBXSMM_DNN_DATATYPE_F32  = LIBXSMM_DATATYPE_F32,
+  LIBXSMM_DNN_DATATYPE_BF16 = LIBXSMM_DATATYPE_BF16,
   LIBXSMM_DNN_DATATYPE_I32  = LIBXSMM_DATATYPE_I32,
   LIBXSMM_DNN_DATATYPE_I16  = LIBXSMM_DATATYPE_I16,
   LIBXSMM_DNN_DATATYPE_I8   = LIBXSMM_DATATYPE_I8
@@ -270,21 +348,67 @@ typedef enum libxsmm_dnn_datatype {
 typedef enum libxsmm_dnn_conv_option {
   /* we get default settings */
   LIBXSMM_DNN_CONV_OPTION_NONE = 0,
-  /* activations are stored unsigned */
-  LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED = 1,
-  /* reduce filters externally to op */
-  LIBXSMM_DNN_CONV_OPTION_WU_EXT_FILTER_REDUCE = 2,
-  /* use 16 bit accumulate instead of 32 bit accumulate for fix point */
-  LIBXSMM_DNN_CONV_OPTION_16BIT_ACC = 4,
   /* overwrite results buffer (set it to zero before running the operations) */
-  LIBXSMM_DNN_CONV_OPTION_OVERWRITE = 8,
+  LIBXSMM_DNN_CONV_OPTION_OVERWRITE = 1,
+  /* activations are stored unsigned */
+  /* @TODO check if we still need this option */
+  LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED = 2,
+  /* reduce filters externally to op */
+  LIBXSMM_DNN_CONV_OPTION_UPD_NO_FILTER_REDUCE = 4,
+  /* external filter transpose to bwd convolutions */
+  LIBXSMM_DNN_CONV_OPTION_BWD_NO_FILTER_TRANSPOSE = 8,
+  /* external filter transpose to bwd convolutions */
+  LIBXSMM_DNN_CONV_OPTION_UPD_NO_INPUT_TRANSPOSE = 16,
+  /* Down-convert for BF16 using RNE rounding */
+  LIBXSMM_DNN_CONV_OPTION_F32_BF16_CVT_RNE = 32,
   /* compound types */
-  LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED_16BIT_ACC = LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED | LIBXSMM_DNN_CONV_OPTION_16BIT_ACC,
-  LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED_16BIT_ACC_OVERWRITE = LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED | LIBXSMM_DNN_CONV_OPTION_16BIT_ACC | LIBXSMM_DNN_CONV_OPTION_OVERWRITE,
+  LIBXSMM_DNN_CONV_OPTION_F32_BF16_CVT_RNE_OVERWRITE = LIBXSMM_DNN_CONV_OPTION_OVERWRITE | LIBXSMM_DNN_CONV_OPTION_F32_BF16_CVT_RNE,
   LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED_OVERWRITE = LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED | LIBXSMM_DNN_CONV_OPTION_OVERWRITE,
-  LIBXSMM_DNN_CONV_OPTION_WU_EXT_FILTER_REDUCE_OVERWRITE = LIBXSMM_DNN_CONV_OPTION_WU_EXT_FILTER_REDUCE | LIBXSMM_DNN_CONV_OPTION_OVERWRITE
+  LIBXSMM_DNN_CONV_OPTION_UPD_NO_FILTER_REDUCE_OVERWRITE = LIBXSMM_DNN_CONV_OPTION_UPD_NO_FILTER_REDUCE | LIBXSMM_DNN_CONV_OPTION_OVERWRITE,
+  LIBXSMM_DNN_CONV_OPTION_BWD_NO_FILTER_TRANSPOSE_OVERWRITE = LIBXSMM_DNN_CONV_OPTION_OVERWRITE | LIBXSMM_DNN_CONV_OPTION_BWD_NO_FILTER_TRANSPOSE,
+  LIBXSMM_DNN_CONV_OPTION_UPD_NO_INPUT_TRANSPOSE_OVERWRITE = LIBXSMM_DNN_CONV_OPTION_OVERWRITE | LIBXSMM_DNN_CONV_OPTION_UPD_NO_INPUT_TRANSPOSE,
+  LIBXSMM_DNN_CONV_OPTION_NO_TRANSPOSES_OVERWRITE = LIBXSMM_DNN_CONV_OPTION_OVERWRITE | LIBXSMM_DNN_CONV_OPTION_BWD_NO_FILTER_TRANSPOSE | LIBXSMM_DNN_CONV_OPTION_UPD_NO_INPUT_TRANSPOSE
 } libxsmm_dnn_conv_option;
 
+typedef enum libxsmm_dnn_fusedbatchnorm_fuse_order {
+  /* the fuse order is: 1. BN, 2. element-wise 3. RELU */
+  LIBXSMM_DNN_FUSEDBN_ORDER_BN_ELTWISE_RELU = 0
+} libxsmm_dnn_fusedbatchnorm_fuse_order;
+
+typedef enum libxsmm_dnn_fusedbatchnorm_fuse_op {
+  /* the fuse order is: 1. BN, 2. element-wise 3. RELU */
+  LIBXSMM_DNN_FUSEDBN_OPS_BN = 1,
+  LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE = 2,
+  LIBXSMM_DNN_FUSEDBN_OPS_ELTWISE = 4,
+  LIBXSMM_DNN_FUSEDBN_OPS_RELU = 8,
+  LIBXSMM_DNN_FUSEDBN_OPS_BN_ELTWISE = LIBXSMM_DNN_FUSEDBN_OPS_BN | LIBXSMM_DNN_FUSEDBN_OPS_ELTWISE,
+  LIBXSMM_DNN_FUSEDBN_OPS_BN_RELU = LIBXSMM_DNN_FUSEDBN_OPS_BN | LIBXSMM_DNN_FUSEDBN_OPS_RELU,
+  LIBXSMM_DNN_FUSEDBN_OPS_BN_ELTWISE_RELU = LIBXSMM_DNN_FUSEDBN_OPS_BN | LIBXSMM_DNN_FUSEDBN_OPS_ELTWISE | LIBXSMM_DNN_FUSEDBN_OPS_RELU,
+  LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE_ELTWISE = LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE | LIBXSMM_DNN_FUSEDBN_OPS_ELTWISE,
+  LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE_RELU = LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE | LIBXSMM_DNN_FUSEDBN_OPS_RELU,
+  LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE_ELTWISE_RELU = LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE | LIBXSMM_DNN_FUSEDBN_OPS_ELTWISE | LIBXSMM_DNN_FUSEDBN_OPS_RELU
+} libxsmm_dnn_fusedbatchnorm_fuse_op;
+
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_dnn_fusedbatchnorm_desc {
+  int N;                                     /* number of images in mini-batch */
+  int C;                                     /* number of input feature maps */
+  int H;                                     /* height of input image */
+  int W;                                     /* width of input image */
+  int u;                                     /* vertical stride */
+  int v;                                     /* horizontal stride */
+  int pad_h_in;                              /* height of physical zero-padding in input buffer */
+  int pad_w_in;                              /* width of physical zero-padding in input buffer */
+  int pad_h_out;                             /* height of physical zero-padding in output buffer */
+  int pad_w_out;                             /* width of physical zero-padding in output buffer */
+  int threads;                               /* number of threads used */
+  libxsmm_dnn_datatype datatype_in;          /* datatype used for all input related buffers */
+  libxsmm_dnn_datatype datatype_out;         /* datatype used for all output related buffers */
+  libxsmm_dnn_datatype datatype_stats;       /* datatype used for all stats related buffers */
+  libxsmm_dnn_tensor_format buffer_format;   /* format which is for activation buffers */
+  libxsmm_dnn_fusedbatchnorm_fuse_order fuse_order; /* additional options */
+  libxsmm_dnn_fusedbatchnorm_fuse_op fuse_ops;      /* used ops into convolutions */
+} libxsmm_dnn_fusedbatchnorm_desc;
+
 /** Structure storing the convolution argument description. */
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_forward_descriptor {
   unsigned int kh;                              /* kernel height */
@@ -296,6 +420,8 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_forward_de
   unsigned int blocks_ifm_blocking;
   unsigned int ofm_block;                       /* should be VLEN */
   unsigned int ifm_block;                       /* should be VLEN */
+  unsigned int ifm_block_hp;
+  unsigned int ofm_block_lp;
   unsigned int ofh_padded;                      /* this we need for 2D register block */
   unsigned int ofw_padded;                      /* this we use for 1D and 2D register block */
   unsigned int ofh_rb;                          /* UR, register block of ofh */
@@ -306,6 +432,22 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_forward_de
   unsigned int stride_w;                        /* this we use for offsets in the input */
   unsigned int fm_lp_block;                     /* additional blocking for low precision datatypes of ifm */
   unsigned int use_nts;                         /* non-zero if intent is to overwrite the output buffer using streaming stores */
+  unsigned int weight_stride;
+  unsigned int use_fwd_generator_for_bwd;
+  unsigned int stride_h_store;
+  unsigned int stride_w_store;
+  unsigned int extra_L2_prefetching;
+  unsigned int input_L2_prefetching;
+  unsigned int lookahead;
+  unsigned int compute_batch_stats_fwd;
+  unsigned int compute_batch_stats_bwd;
+  unsigned int compute_eltwise;
+  libxsmm_dnn_fusedbatchnorm_desc* pre_bn;
+  libxsmm_dnn_fusedbatchnorm_desc* post_bn;
+  unsigned int compute_max;
+  unsigned int perform_relu_in_kernel;
+  unsigned int n_variants;
+  unsigned int f32_bf16_cvt_rne;                /* non-zero if in case of bf16 we perform RNE rounding when converting down from f32 in JIT sequence */
   libxsmm_dnn_tensor_format format;
   libxsmm_dnn_conv_option option;
   libxsmm_dnn_datatype datatype;
@@ -343,12 +485,13 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_backward_d
 /** Structure storing the convolution weight update argument description. */
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_weight_update_descriptor {
   unsigned int kw;                              /* kernel width */
-  unsigned int unroll_kw;                       /* kernel width, unrolled */
   unsigned int kh;                              /* kernel height */
   unsigned int blocks_ofm;
   unsigned int blocks_ifm;
   unsigned int ofm_block;                       /* should be VLEN */
   unsigned int ifm_block;                       /* should be VLEN */
+  unsigned int ifm_block_hp;
+  unsigned int ofm_block_lp;
   unsigned int ofh_padded;                      /* this we need for 2D register block */
   unsigned int ofw_padded;                      /* this we use for 1D and 2D register block */
   unsigned int ofh_rb;                          /* UR, register block of ofh */
@@ -357,13 +500,21 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_weight_upd
   unsigned int ifw_padded;                      /* this we use for 1D and 2D register block */
   unsigned int stride_h;                        /* this we use for offsets in the input */
   unsigned int stride_w;                        /* this we use for offsets in the input */
-
+  unsigned int fm_lp_block;
   unsigned int ifm_unroll;                      /* this we use to unroll ifm loop */
   unsigned int ofh;                             /* upper bound of oj loop */
   unsigned int ofh_unroll;                      /* this we use to unroll ofh loop */
   unsigned int ofw;                             /* upper bound of oi loop */
   unsigned int ofw_unroll;                      /* this we use to unroll ofw loop */
+  unsigned int blocks_h;
+  unsigned int blocks_img;
+  unsigned int use_nts;
   unsigned int transpose_ofw_ifm;               /* transpose ofw and ifm */
+  unsigned int ofw_fake_pixels;
+  unsigned int use_fastpath;
+  unsigned int ncopies;                         /* number of reduction copies, probably nthreads */
+  unsigned int avoid_output_trans;
+  unsigned int f32_bf16_cvt_rne;                /* non-zero if in case of bf16 we perform RNE rounding when converting down from f32 in JIT sequence */
   libxsmm_dnn_tensor_format format;
   libxsmm_dnn_conv_option option;
   libxsmm_dnn_datatype datatype;
@@ -371,28 +522,6 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_weight_upd
   libxsmm_convolution_prefetch_type prefetch;   /* prefetch type, can be ORed vales of libxsmm_convolution_prefetch_type */
 } libxsmm_convolution_weight_update_descriptor;
 
-/**
- * Structure storing the convolution Winograd argument description.
- */
-LIBXSMM_EXTERN_C typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_winograd_descriptor {
-  /** alpha determines the tile size */
-  unsigned int alpha;
-  /** number of itiles */
-  unsigned int itiles;
-  /** number of jtiles */
-  unsigned int jtiles;
-  /** number of images in a block */
-  unsigned int bimg;
-  /** unroll factor */
-  unsigned int ur;
-  /** number of ifm blocks to unroll */
-  unsigned int ur_ifm;
-  /** number of ifm blocks. When ur_ifm equals to this, do additional optimizations */
-  unsigned int blocks_ifm;
-  /** prefetch type, can be ORed vales of libxsmm_convolution_prefetch_type */
-  libxsmm_convolution_prefetch_type prefetch;
-} libxsmm_convolution_winograd_descriptor;
-
 /** Specialized function with fused alpha and beta arguments, and optional prefetch locations (double-precision). */
 LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_dmmfunction)(const double* a, const double* b, double* c, ...);
 /** Specialized function with fused alpha and beta arguments, and optional prefetch locations (single-precision). */
@@ -401,10 +530,22 @@ LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_smmfunction)(const
 LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_wimmfunction)(const short* a, const short* b, int* c, ...);
 /** Specialized function with fused alpha and beta arguments, and optional prefetch locations (low-precision). */
 LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_wsmmfunction)(const short* a, const short* b, float* c, ...);
+/** Specialized function with fused alpha and beta arguments, and optional prefetch locations (bf16, fp32-accumulate). */
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_bsmmfunction)(const libxsmm_bfloat16* a, const libxsmm_bfloat16* b, float* c, ...);
+/** Specialized function with fused alpha and beta arguments, and optional prefetch locations (bf16, fp32-accumulate). */
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_bmmfunction)(const libxsmm_bfloat16* a, const libxsmm_bfloat16* b, libxsmm_bfloat16* c, ...);
+
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_dmmfunction_reducebatch)(const double** a, const double** b, double* c, const unsigned long long* count, ...);
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_smmfunction_reducebatch)(const float** a, const float** b, float* c, const unsigned long long* count, ...);
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_bsmmfunction_reducebatch)(const libxsmm_bfloat16** a, const libxsmm_bfloat16** b, float* c, const unsigned long long* count, ...);
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_bmmfunction_reducebatch)(const libxsmm_bfloat16** a, const libxsmm_bfloat16** b, libxsmm_bfloat16* c, const unsigned long long* count, ...);
+
 /** Function type which is either libxsmm_smmfunction or libxsmm_dmmfunction (weak-typed). */
 LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE libxsmm_xmmfunction {
   void (*xmm)(const void* a, const void* b, void* c, ...);
-  libxsmm_dmmfunction dmm; libxsmm_smmfunction smm; libxsmm_wimmfunction wimm; libxsmm_wsmmfunction wsmm;
+  void (*xbm)(const void** a, const void** b, void* c, const unsigned long long* count, ...);
+  libxsmm_dmmfunction dmm; libxsmm_smmfunction smm; libxsmm_wimmfunction wimm; libxsmm_wsmmfunction wsmm; libxsmm_bsmmfunction bsmm; libxsmm_bmmfunction bmm;
+  libxsmm_dmmfunction_reducebatch dmr; libxsmm_smmfunction_reducebatch smr; libxsmm_bsmmfunction_reducebatch bsmr; libxsmm_bmmfunction_reducebatch bmr;
 } libxsmm_xmmfunction;
 
 /** Determines the kernel kind. */
@@ -415,8 +556,16 @@ typedef enum libxsmm_kernel_kind {
   LIBXSMM_KERNEL_KIND_MCOPY   = 1,
   /** Transpose kernel kind */
   LIBXSMM_KERNEL_KIND_TRANS   = 2,
+  /** GEMM/packed kernel kind */
+  LIBXSMM_KERNEL_KIND_PGEMM   = 3,
+  /** GEMM/packed kernel kind */
+  LIBXSMM_KERNEL_KIND_GETRF   = 4,
+  /** TRMM kernel kind */
+  LIBXSMM_KERNEL_KIND_TRMM    = 5,
+  /** TRSM kernel kind */
+  LIBXSMM_KERNEL_KIND_TRSM    = 6,
   /** Not a JIT kernel */
-  LIBXSMM_KERNEL_KIND_INVALID = 3
+  LIBXSMM_KERNEL_KIND_INVALID = 7
 } libxsmm_kernel_kind;
 
 /** Specialized function for matrix-copy (weak-typed). */
@@ -427,6 +576,22 @@ LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_xmcopyfunction)(
 LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_xtransfunction)(
   const void* in, const unsigned int* ldi, void* out, const unsigned int* ldo);
 
+/** Specialized function for packed GEMM (weak-typed). */
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_pgemm_xfunction)(
+  const void* a, const void* b, void* c);
+
+/** Specialized function for packed GEMM (weak-typed). */
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_getrf_xfunction)(
+  const void* a, const void* b, void* c);
+
+/** Specialized function for TRMM (weak-typed). */
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_trmm_xfunction)(
+  const void* a, const void* b, void* c);
+
+/** Specialized function for TRSM (weak-typed). */
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_trsm_xfunction)(
+  const void* a, const void* b, void* c);
+
 /** Structure to receive information about GEMM-kernels (libxsmm_get_mmkernel_info). */
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_mmkernel_info {
   /** Input/output data-type */
