@@ -1,6 +1,6 @@
 #!/bin/sh
 #############################################################################
-# Copyright (c) 2015-2018, Intel Corporation                                #
+# Copyright (c) 2015-2019, Intel Corporation                                #
 # All rights reserved.                                                      #
 #                                                                           #
 # Redistribution and use in source and binary forms, with or without        #
@@ -30,9 +30,9 @@
 # Hans Pabst (Intel Corp.)
 #############################################################################
 
-SORT=$(which sort)
-SED=$(which sed)
-CUT=$(which cut)
+SORT=$(command -v sort)
+SED=$(command -v sed)
+CUT=$(command -v cut)
 
 VARIANT=Specialized
 
@@ -44,14 +44,14 @@ fi
 HERE=$(cd $(dirname $0); pwd -P)
 FILE=${HERE}/cp2k-perf.txt
 
-GREP=$(which grep)
+GREP=$(command -v grep)
 PERF=$(${GREP} -A1 -i "${VARIANT}" ${FILE} | \
   ${GREP} -e "performance" | \
   ${CUT} -d" " -f2 | \
   ${SORT} -n)
 
-ECHO=$(which echo)
-NUM=$(${ECHO} "${PERF}" | wc -l)
+ECHO=$(command -v echo)
+NUM=$(${ECHO} "${PERF}" | wc -l | tr -d " ")
 MIN=$(${ECHO} ${PERF} | ${CUT} -d" " -f1)
 MAX=$(${ECHO} ${PERF} | ${CUT} -d" " -f${NUM})
 
@@ -59,7 +59,7 @@ ${ECHO} "num=${NUM}"
 ${ECHO} "min=${MIN}"
 ${ECHO} "max=${MAX}"
 
-BC=$(which bc 2>/dev/null)
+BC=$(command -v bc)
 if [ "" != "${BC}" ]; then
   AVG=$(${ECHO} "$(${ECHO} -n "scale=3;(${PERF})/${NUM}" | tr "\n" "+")" | ${BC})
   NUM2=$((NUM / 2))
@@ -83,7 +83,7 @@ elif [ -f /cygdrive/c/Program\ Files\ \(x86\)/gnuplot/bin/wgnuplot ]; then
   WGNUPLOT=/cygdrive/c/Program\ Files\ \(x86\)/gnuplot/bin/wgnuplot
   GNUPLOT=/cygdrive/c/Program\ Files\ \(x86\)/gnuplot/bin/gnuplot
 else
-  GNUPLOT=$(which gnuplot 2>/dev/null)
+  GNUPLOT=$(command -v gnuplot)
   WGNUPLOT=${GNUPLOT}
 fi
 
