@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2018, Intel Corporation                                **
+** Copyright (c) 2016-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -82,12 +82,12 @@ int main(int argc, char *argv[])
 
   #pragma omp parallel for
   for (i = 0; i < lda*m; i++) {
-    a[i] = drand48();
+    a[i] = libxsmm_rng_f64();
   }
 
   #pragma omp parallel for
   for (i = 0; i < ldb*k; i++) {
-    b[i] = drand48();
+    b[i] = libxsmm_rng_f64();
   }
 
   #pragma omp parallel for
