@@ -1,6 +1,6 @@
-#!/bin/sh
+#!/bin/bash
 #############################################################################
-# Copyright (c) 2015-2018, Intel Corporation                                #
+# Copyright (c) 2015-2019, Intel Corporation                                #
 # All rights reserved.                                                      #
 #                                                                           #
 # Redistribution and use in source and binary forms, with or without        #
@@ -46,7 +46,7 @@ elif [ -f /cygdrive/c/Program\ Files\ \(x86\)/gnuplot/bin/wgnuplot ]; then
   WGNUPLOT=/cygdrive/c/Program\ Files\ \(x86\)/gnuplot/bin/wgnuplot
   GNUPLOT=/cygdrive/c/Program\ Files\ \(x86\)/gnuplot/bin/gnuplot
 else
-  GNUPLOT=$(which gnuplot 2>/dev/null)
+  GNUPLOT=$(command -v gnuplot)
   WGNUPLOT=${GNUPLOT}
 fi
 
@@ -58,7 +58,7 @@ if [ -f "${GNUPLOT}" ]; then
 fi
 GNUPLOT_VERSION=$((GNUPLOT_MAJOR * 10000 + GNUPLOT_MINOR * 100))
 
-SED=$(which sed)
+SED=$(command -v sed)
 function capturedTxtToDataFile {
   ${SED} \
     -e "/^m=/,/${VARIANT}/{//!d}" \
@@ -81,7 +81,7 @@ function capturedTxtToDataFile {
 }
 
 if [ "40600" -le "${GNUPLOT_VERSION}" ]; then
-  RM=$(which rm)
+  RM=$(command -v rm)
   if [ "" = "$1" ]; then
     FILENAME=smm-$(echo ${VARIANT} | tr ' ,' '-' | tr -d '()' | tr '[:upper:]' '[:lower:]').pdf
   else
