@@ -1,5 +1,5 @@
 !*****************************************************************************!
-!* Copyright (c) 2015-2018, Intel Corporation                                *!
+!* Copyright (c) 2015-2019, Intel Corporation                                *!
 !* All rights reserved.                                                      *!
 !*                                                                           *!
 !* Redistribution and use in source and binary forms, with or without        *!
@@ -39,12 +39,13 @@
         REAL(T), ALLOCATABLE, TARGET :: a(:,:,:), b(:,:,:)
         REAL(T), ALLOCATABLE, TARGET :: c(:,:), d(:,:)
         REAL(T), ALLOCATABLE, TARGET, SAVE :: tmp(:,:)
-        !DIR$ ATTRIBUTES ALIGN:LIBXSMM_ALIGNMENT :: a, b, c, tmp
+        !DIR$ ATTRIBUTES ALIGN:64 :: a, b, c, tmp
         !$OMP THREADPRIVATE(tmp)
         TYPE(LIBXSMM_DMMFUNCTION) :: xmm
-        DOUBLE PRECISION :: duration, scale, max_diff, diff
         INTEGER(8) :: i, r, s, size0, size1, size, repetitions, start
+        TYPE(LIBXSMM_MATDIFF_INFO) :: diff, max_diff
         INTEGER(LIBXSMM_BLASINT_KIND) :: m, n, k
+        DOUBLE PRECISION :: duration, scale
         CHARACTER(32) :: argv
         INTEGER :: argc
 
@@ -87,7 +88,7 @@
         CALL libxsmm_dispatch(xmm, m, n, k)
 
         ! workload is about 2 GByte in memory by default
-        size0 = (m * k + k * n) * T ! size of a single stream element in Byte
+        size0 = (m * k + k * n + m * n) * T ! size of a single stream element in Byte
         size1 = MERGE(2048_8, MERGE(size1, ISHFT(ABS(size0 * size1)     &
      &            + ISHFT(1, 20) - 1, -20), 0.LE.size1), 0.EQ.size1)
         size = ISHFT(MERGE(MAX(size, size1), ISHFT(ABS(size) * size0    &
@@ -96,8 +97,8 @@
         repetitions = size / s
         scale = 1D0 / s
         duration = 0
-        max_diff = 0
 
+        CALL libxsmm_matdiff_clear(max_diff)
         ALLOCATE(c(m,n))
         ALLOCATE(a(m,k,s))
         ALLOCATE(b(k,n,s))
@@ -111,7 +112,7 @@
 
         WRITE(*, "(3(A,I0),A,I0,A,I0,A,I0)")                            &
      &    "m=", m, " n=", n, " k=", k, " elements=", UBOUND(a, 3),      &
-     &    " size=", size1, "MB repetitions=", repetitions
+     &    " size=", size1, " MB repetitions=", repetitions
 
         ! compute reference solution and warmup BLAS library
         ALLOCATE(d(m,n))
@@ -190,9 +191,10 @@
         DEALLOCATE(tmp)
         !$OMP END PARALLEL
         CALL performance(duration, m, n, k, size)
-        diff = MAXVAL((c(:,:) - d(:,:)) * (c(:,:) - d(:,:)))
-        WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "diff:       ", diff
-        max_diff = MAX(diff, max_diff)
+        CALL libxsmm_matdiff(diff, LIBXSMM_DATATYPE_F64, m, n,          &
+     &    libxsmm_ptr2(d), libxsmm_ptr2(c))
+        WRITE(*, "(1A,A,F10.1)") CHAR(9), "diff:      ", diff%l2_abs
+        CALL libxsmm_matdiff_reduce(max_diff, diff)
 
         IF (libxsmm_available(xmm)) THEN
           c(:,:) = 0
@@ -220,9 +222,10 @@
           DEALLOCATE(tmp)
           !$OMP END PARALLEL
           CALL performance(duration, m, n, k, size)
-          diff = MAXVAL((c(:,:) - d(:,:)) * (c(:,:) - d(:,:)))
-          WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "diff:       ", diff
-          max_diff = MAX(diff, max_diff)
+          CALL libxsmm_matdiff(diff, LIBXSMM_DATATYPE_F64, m, n,        &
+     &      libxsmm_ptr2(d), libxsmm_ptr2(c))
+          WRITE(*, "(1A,A,F10.1)") CHAR(9), "diff:      ", diff%l2_abs
+          CALL libxsmm_matdiff_reduce(max_diff, diff)
         END IF
 
         ! Deallocate global arrays
@@ -234,10 +237,10 @@
         ! finalize LIBXSMM
         CALL libxsmm_finalize()
 
-        IF (1.LT.max_diff) STOP 1
+        IF (1.LT.(max_diff%l2_rel)) STOP 1
 
       CONTAINS
-        SUBROUTINE init(seed, matrix, scale, n)
+        PURE SUBROUTINE init(seed, matrix, scale, n)
           INTEGER, INTENT(IN) :: seed
           REAL(T), INTENT(OUT) :: matrix(:,:)
           REAL(8), INTENT(IN) :: scale
