@@ -1,6 +1,6 @@
 #!/bin/sh
 #############################################################################
-# Copyright (c) 2015-2018, Intel Corporation                                #
+# Copyright (c) 2015-2019, Intel Corporation                                #
 # All rights reserved.                                                      #
 #                                                                           #
 # Redistribution and use in source and binary forms, with or without        #
@@ -32,7 +32,7 @@
 
 cat << EOM
 /******************************************************************************
-** Copyright (c) 2016-2018, Intel Corporation                                **
+** Copyright (c) 2016-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -88,11 +88,18 @@ cat << EOM
 EOM
 
 HERE=$(cd $(dirname $0); pwd -P)
+SRCDIR=../src
+
+if [ "" = "$1" ]; then
+  DSTDIR=${SRCDIR}
+else
+  DSTDIR=$1
+fi
 
 # good-enough pattern to match a main function, and to exclude this translation unit
-for FILE in $(grep -L "main\s*(.*)" ${HERE}/../src/*.c); do
+for FILE in $(grep -L "main[[:space:]]*(.*)" ${HERE}/${SRCDIR}/*.c); do
   BASENAME=$(basename ${FILE})
-  echo "#include \"../src/${BASENAME}\""
+  echo "#include \"${DSTDIR}/${BASENAME}\""
 done
 
 cat << EOM
