@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -275,37 +275,6 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_SHUFPS:
       libxsmm_strncpy(o_instr_name, "shufps", i_instr_name_max_length, 6 );
       break;
-    /* IMCI special */
-    case LIBXSMM_X86_INSTR_VLOADUNPACKLPD:
-      libxsmm_strncpy(o_instr_name, "vloadunpacklpd", i_instr_name_max_length, 14 );
-      break;
-    case LIBXSMM_X86_INSTR_VLOADUNPACKHPD:
-      libxsmm_strncpy(o_instr_name, "vloadunpackhpd", i_instr_name_max_length, 14 );
-      break;
-    case LIBXSMM_X86_INSTR_VLOADUNPACKLPS:
-      libxsmm_strncpy(o_instr_name, "vloadunpacklps", i_instr_name_max_length, 14 );
-      break;
-    case LIBXSMM_X86_INSTR_VLOADUNPACKHPS:
-      libxsmm_strncpy(o_instr_name, "vloadunpackhps", i_instr_name_max_length, 14 );
-      break;
-    case LIBXSMM_X86_INSTR_VPACKSTORELPD:
-      libxsmm_strncpy(o_instr_name, "vpackstorelpd", i_instr_name_max_length, 13 );
-      break;
-    case LIBXSMM_X86_INSTR_VPACKSTOREHPD:
-      libxsmm_strncpy(o_instr_name, "vpackstorehpd", i_instr_name_max_length, 13 );
-      break;
-    case LIBXSMM_X86_INSTR_VPACKSTORELPS:
-      libxsmm_strncpy(o_instr_name, "vpackstorelps", i_instr_name_max_length, 13 );
-      break;
-    case LIBXSMM_X86_INSTR_VPACKSTOREHPS:
-      libxsmm_strncpy(o_instr_name, "vpackstorehps", i_instr_name_max_length, 13 );
-      break;
-    case LIBXSMM_X86_INSTR_VPREFETCH1:
-      libxsmm_strncpy(o_instr_name, "vprefetch1", i_instr_name_max_length, 10 );
-      break;
-    case LIBXSMM_X86_INSTR_VPREFETCH0:
-      libxsmm_strncpy(o_instr_name, "vprefetch0", i_instr_name_max_length, 10 );
-      break;
     /* Gather/scatter single precision */
     case LIBXSMM_X86_INSTR_VGATHERDPS:
       libxsmm_strncpy(o_instr_name, "vgatherdps", i_instr_name_max_length, 10 );
@@ -408,7 +377,7 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_ADDSS:
       libxsmm_strncpy(o_instr_name, "addss", i_instr_name_max_length, 5 );
       break;
-    /* XOR AVX512,IMCI */
+    /* XOR AVX512F */
     case LIBXSMM_X86_INSTR_VPXORD:
       libxsmm_strncpy(o_instr_name, "vpxord", i_instr_name_max_length, 6 );
       break;
@@ -433,6 +402,15 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_VPSRAVD:
       libxsmm_strncpy(o_instr_name, "vpsravd", i_instr_name_max_length, 7 );
       break;
+    case LIBXSMM_X86_INSTR_VPSRAD:
+      libxsmm_strncpy(o_instr_name, "vpsrad", i_instr_name_max_length, 6 );
+      break;
+    case LIBXSMM_X86_INSTR_VPSLLD:
+      libxsmm_strncpy(o_instr_name, "vpslld", i_instr_name_max_length, 6 );
+      break;
+    case LIBXSMM_X86_INSTR_VPCMPD:
+      libxsmm_strncpy(o_instr_name, "vpcmpd", i_instr_name_max_length, 6 );
+      break;
     /* AVX512, QFMA */
     case LIBXSMM_X86_INSTR_V4FMADDPS:
       libxsmm_strncpy(o_instr_name, "v4fmaddps", i_instr_name_max_length, 9 );
@@ -447,10 +425,10 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
       libxsmm_strncpy(o_instr_name, "v4fnmaddss", i_instr_name_max_length, 10 );
       break;
     case LIBXSMM_X86_INSTR_VP4DPWSSD:
-      libxsmm_strncpy(o_instr_name, "vp4dpwssd", i_instr_name_max_length, 13 );
+      libxsmm_strncpy(o_instr_name, "vp4dpwssd", i_instr_name_max_length, 9 );
       break;
     case LIBXSMM_X86_INSTR_VP4DPWSSDS:
-      libxsmm_strncpy(o_instr_name, "vp4dpwssds", i_instr_name_max_length, 15 );
+      libxsmm_strncpy(o_instr_name, "vp4dpwssds", i_instr_name_max_length, 10 );
       break;
     /* AVX512, VNNI */
     case LIBXSMM_X86_INSTR_VPDPWSSD:
@@ -465,6 +443,16 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_VPDPBUSDS:
       libxsmm_strncpy(o_instr_name, "vpdpbusds", i_instr_name_max_length, 9 );
       break;
+    /* AVX512, BF16 */
+    case LIBXSMM_X86_INSTR_VDPBF16PS:
+      libxsmm_strncpy(o_instr_name, "vdpbf16ps", i_instr_name_max_length, 9 );
+      break;
+    case LIBXSMM_X86_INSTR_VCVTNEPS2BF16:
+      libxsmm_strncpy(o_instr_name, "vcvtneps2bf16", i_instr_name_max_length, 13 );
+      break;
+    case LIBXSMM_X86_INSTR_VCVTNE2PS2BF16:
+      libxsmm_strncpy(o_instr_name, "vcvtne2ps2bf16", i_instr_name_max_length, 14 );
+      break;
     /* GP instructions */
     case LIBXSMM_X86_INSTR_ADDQ:
       libxsmm_strncpy(o_instr_name, "addq", i_instr_name_max_length, 4 );
@@ -481,6 +469,27 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_JL:
       libxsmm_strncpy(o_instr_name, "jl", i_instr_name_max_length, 2 );
       break;
+    case LIBXSMM_X86_INSTR_JE:
+      libxsmm_strncpy(o_instr_name, "je", i_instr_name_max_length, 2 );
+      break;
+    case LIBXSMM_X86_INSTR_JZ:
+      libxsmm_strncpy(o_instr_name, "jz", i_instr_name_max_length, 2 );
+      break;
+    case LIBXSMM_X86_INSTR_JG:
+      libxsmm_strncpy(o_instr_name, "jg", i_instr_name_max_length, 2 );
+      break;
+    case LIBXSMM_X86_INSTR_JNE:
+      libxsmm_strncpy(o_instr_name, "jne", i_instr_name_max_length, 3 );
+      break;
+    case LIBXSMM_X86_INSTR_JNZ:
+      libxsmm_strncpy(o_instr_name, "jnz", i_instr_name_max_length, 3 );
+      break;
+    case LIBXSMM_X86_INSTR_JGE:
+      libxsmm_strncpy(o_instr_name, "jge", i_instr_name_max_length, 3 );
+      break;
+    case LIBXSMM_X86_INSTR_JLE:
+      libxsmm_strncpy(o_instr_name, "jle", i_instr_name_max_length, 3 );
+      break;
     case LIBXSMM_X86_INSTR_PREFETCHT0:
       libxsmm_strncpy(o_instr_name, "prefetcht0", i_instr_name_max_length, 10 );
       break;
@@ -584,31 +593,6 @@ unsigned int libxsmm_is_x86_vec_instr_single_precision( const unsigned int i_ins
     case LIBXSMM_X86_INSTR_SHUFPS:
       l_return = 1;
       break;
-    /* IMCI special */
-    case LIBXSMM_X86_INSTR_VLOADUNPACKLPD:
-      l_return = 0;
-      break;
-    case LIBXSMM_X86_INSTR_VLOADUNPACKHPD:
-      l_return = 0;
-      break;
-    case LIBXSMM_X86_INSTR_VLOADUNPACKLPS:
-      l_return = 1;
-      break;
-    case LIBXSMM_X86_INSTR_VLOADUNPACKHPS:
-      l_return = 1;
-      break;
-    case LIBXSMM_X86_INSTR_VPACKSTORELPD:
-      l_return = 0;
-      break;
-    case LIBXSMM_X86_INSTR_VPACKSTOREHPD:
-      l_return = 0;
-      break;
-    case LIBXSMM_X86_INSTR_VPACKSTORELPS:
-      l_return = 1;
-      break;
-    case LIBXSMM_X86_INSTR_VPACKSTOREHPS:
-      l_return = 1;
-      break;
     /* Gather/Scatter single precision */
     case LIBXSMM_X86_INSTR_VGATHERDPS:
       l_return = 1;
@@ -763,14 +747,20 @@ void libxsmm_reset_loop_label_tracker( libxsmm_loop_label_tracker* io_loop_label
   memset(io_loop_label_tracker, 0, sizeof(*io_loop_label_tracker));
 }
 
+LIBXSMM_API_INTERN
+void libxsmm_reset_jump_label_tracker( libxsmm_jump_label_tracker* io_jump_label_tracker ) {
+  memset(io_jump_label_tracker, 0, sizeof(*io_jump_label_tracker));
+}
+
 LIBXSMM_API_INTERN
 void libxsmm_mmfunction_signature( libxsmm_generated_code*         io_generated_code,
-                                  const char*                     i_routine_name,
-                                  const libxsmm_gemm_descriptor* i_xgemm_desc ) {
+                                   const char*                     i_routine_name,
+                                   const libxsmm_gemm_descriptor*  i_xgemm_desc ) {
   char l_new_code[512];
   int l_max_code_length = 511;
   int l_code_length = 0;
 
+  LIBXSMM_ASSERT_MSG(NULL != i_xgemm_desc, "Invalid descriptor");
   if ( io_generated_code->code_type > 1 ) {
     return;
   } else if ( io_generated_code->code_type == 1 ) {
@@ -838,12 +828,11 @@ void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_co
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     } else if ( (strcmp( i_arch, "knl" ) == 0) ||
                 (strcmp( i_arch, "knm" ) == 0) ||
-                (strcmp( i_arch, "skx" ) == 0) ) {
+                (strcmp( i_arch, "skx" ) == 0) ||
+                (strcmp( i_arch, "clx" ) == 0) ||
+                (strcmp( i_arch, "cpx" ) == 0)    ) {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX512F__\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "knc" ) == 0) ) {
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __MIC__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     } else if ( (strcmp( i_arch, "noarch" ) == 0) ) {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#pragma message (\"LIBXSMM KERNEL COMPILATION WARNING: compiling arch-independent gemm kernel in: \" __FILE__)\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
@@ -865,10 +854,11 @@ void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_co
     if ( (strcmp( i_arch, "wsm" ) == 0) ||
          (strcmp( i_arch, "snb" ) == 0) ||
          (strcmp( i_arch, "hsw" ) == 0) ||
-         (strcmp( i_arch, "knc" ) == 0) ||
          (strcmp( i_arch, "knl" ) == 0) ||
          (strcmp( i_arch, "knm" ) == 0) ||
-         (strcmp( i_arch, "skx" ) == 0))
+         (strcmp( i_arch, "skx" ) == 0) ||
+         (strcmp( i_arch, "clx" ) == 0) ||
+         (strcmp( i_arch, "cpx" ) == 0)    )
     {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#else\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
@@ -894,14 +884,14 @@ void libxsmm_handle_error( libxsmm_generated_code* io_generated_code,
   static LIBXSMM_TLS unsigned int last_error_code;
   if (i_error_code != last_error_code) {
     if (0 != emit_message) {
-      LIBXSMM_FLOCK(stderr);
+      LIBXSMM_STDIO_ACQUIRE();
       if (0 != context && 0 != *context && '0' != *context) {
         fprintf(stderr, "LIBXSMM ERROR (%s): %s\n", context, libxsmm_strerror(i_error_code));
       }
       else {
         fprintf(stderr, "LIBXSMM ERROR: %s\n", libxsmm_strerror(i_error_code));
       }
-      LIBXSMM_FLOCK(stderr);
+      LIBXSMM_STDIO_RELEASE();
     }
     last_error_code = i_error_code;
   }
@@ -931,7 +921,7 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       break;
     case LIBXSMM_ERR_ARCH_PREC:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "unknown architecture or precision (error #%u)!", i_error_code );
+        "unknown architecture or unsupported precision (error #%u)!", i_error_code );
       break;
     case LIBXSMM_ERR_ARCH:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
@@ -959,7 +949,7 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       break;
     case LIBXSMM_ERR_CSC_INPUT:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "could not open the specified CSC input file (error #%u)!", i_error_code );
+        "could not open the CSC input file, or invalid file content found (error #%u)!", i_error_code );
       break;
     case LIBXSMM_ERR_CSC_READ_LEN:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
@@ -989,21 +979,13 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "invalid K blocking in microkernel (error #%u)!", i_error_code );
       break;
-    case LIBXSMM_ERR_NO_IMCI:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "IMCI architecture requested but called for a different one (error #%u)!", i_error_code );
-      break;
     case LIBXSMM_ERR_REG_BLOCK:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "invalid MxN register blocking was specified (error #%u)!", i_error_code );
       break;
-    case LIBXSMM_ERR_VEC_MOVE_IMCI:
+    case LIBXSMM_ERR_NO_AVX512_BCAST:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "invalid vec move instruction for IMCI instruction replacement (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_NO_IMCI_AVX512_BCAST:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "fused memory broadcast is not supported on other platforms than AVX512/IMCI (error #%u)!", i_error_code );
+        "fused memory broadcast is not supported on other platforms than AVX512 (error #%u)!", i_error_code );
       break;
     case LIBXSMM_ERR_NO_AVX512_QFMA:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
@@ -1045,6 +1027,10 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "too many nested loops, exceeding loop label tracker (error #%u)!", i_error_code );
       break;
+    case LIBXSMM_ERR_JMPLBL_USED:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
+        "attempted to use an already used jump label (error #%u)!", i_error_code );
+      break;
     case LIBXSMM_ERR_CSC_ALLOC_DATA:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "could not allocate temporary memory for reading CSC file (error #%u)!", i_error_code );
@@ -1129,6 +1115,14 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "input vector register parameter must be undefined here (error #%u)!", i_error_code );
       break;
+    case LIBXSMM_ERR_TRANS_B:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
+        "GEMM kernel with trans B requested, but target/datatype not supported (error #%u)!", i_error_code );
+      break;
+    case LIBXSMM_ERR_LDB_TRANS:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
+        "ldb needs to be greater than or equal to n (error #%u)!", i_error_code );
+      break;
     default: /* we do not know what happened */
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "an unknown error occurred (error #%u)!", i_error_code );
@@ -1138,6 +1132,7 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
   return error_message;
 }
 
+LIBXSMM_API_INTERN
 void libxsmm_convfunction_signature_fp32( libxsmm_generated_code*         io_generated_code,
                                           const char*                     i_routine_name ) {
   char l_new_code[512];
@@ -1172,3 +1167,56 @@ void libxsmm_convfunction_signature_int16( libxsmm_generated_code*         io_ge
 
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
 }
+
+
+LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
+  unsigned int i_size, unsigned int i_max_block,
+  unsigned int* o_range_1, unsigned int* o_block_1,
+  unsigned int* o_range_2, unsigned int* o_block_2 )
+{
+  unsigned int l_number_of_chunks = 1+((i_size-1)/i_max_block);
+  unsigned int l_modulo = i_size%l_number_of_chunks;
+  unsigned int l_n2 = i_size/l_number_of_chunks;
+  unsigned int l_n1 = l_n2 + 1;
+  unsigned int l_N2 = 0;
+  unsigned int l_N1 = 0;
+  unsigned int l_chunk = 0;
+  unsigned int l_ret = 0;
+
+  /* ranges */
+  if (l_n1 > i_max_block) l_n1 = i_max_block;
+  for (l_chunk = 0; l_chunk < l_number_of_chunks; l_chunk++) {
+    if (l_chunk < l_modulo) {
+      l_N1 += l_n1;
+    } else {
+      l_N2 += l_n2;
+    }
+  }
+
+  /* if we have perfect blocking, let's swap n2 and n1, set  */
+  if (l_modulo == 0 ) {
+    l_n1 = l_n2;
+    l_N1 = l_N2;
+    l_n2 = 0;
+    l_N2 = 0;
+  }
+
+  /* some checks */
+  if ( l_N1 % l_n1 != 0 ) {
+    l_ret = 1;
+  }
+  if ( l_n2 != 0 ) {
+    if ( l_N2 % l_n2 != 0 ) {
+      l_ret = 1;
+    }
+  }
+
+  /* set output variables */
+  *o_range_1 = l_N1;
+  *o_block_1 = l_n1;
+  *o_range_2 = l_N2;
+  *o_block_2 = l_n2;
+
+  return l_ret;
+}
+
