@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -32,12 +32,7 @@
 #define GENERATOR_COMMON_H
 
 #include <libxsmm_generator.h>
-
-/* Fallback to MKL and not to triple nested loops in the generator (noarch)
- **/
-#if 0
-#define LIBXSMM_GENERATOR_MKL_BLAS_FALLBACK
-#endif
+#include <libxsmm_cpuid.h>
 
 /*@TODO check if we want to use enums here? Has this implications in the encoder? */
 /* defining register mappings */
@@ -63,7 +58,10 @@
    using this values as the third operand means SSE */
 #define LIBXSMM_X86_VEC_REG_UNDEF          255
 #define LIBXSMM_X86_MASK_REG_UNDEF         255
-#define LIBXSMM_X86_IMCI_AVX512_MASK         1  /* this specifies k1 */
+#define LIBXSMM_X86_AVX512_MASK              1  /* this specifies k1 */
+
+/* special value for undefined immediate */
+#define LIBXSMM_X86_IMM_UNDEF             1024
 
 /* special instruction */
 #define LIBXSMM_X86_INSTR_UNDEF           9999
@@ -104,15 +102,6 @@
 #define LIBXSMM_X86_INSTR_MOVSS          10014
 #define LIBXSMM_X86_INSTR_MOVDDUP        10015
 #define LIBXSMM_X86_INSTR_SHUFPS         10016
-/* IMCI */
-#define LIBXSMM_X86_INSTR_VLOADUNPACKLPD 10017
-#define LIBXSMM_X86_INSTR_VLOADUNPACKHPD 10018
-#define LIBXSMM_X86_INSTR_VLOADUNPACKLPS 10019
-#define LIBXSMM_X86_INSTR_VLOADUNPACKHPS 10020
-#define LIBXSMM_X86_INSTR_VPACKSTORELPD  10021
-#define LIBXSMM_X86_INSTR_VPACKSTOREHPD  10022
-#define LIBXSMM_X86_INSTR_VPACKSTORELPS  10023
-#define LIBXSMM_X86_INSTR_VPACKSTOREHPS  10024
 
 /* Gather/Scatter instructions */
 #define LIBXSMM_X86_INSTR_VGATHERDPS     11000
@@ -179,7 +168,7 @@
 #define LIBXSMM_X86_INSTR_MULSS          20042
 #define LIBXSMM_X86_INSTR_ADDSS          20043
 #define LIBXSMM_X86_INSTR_SUBSS          20044
-/* AVX512, IMCI Integer XOR as there is no FP */
+/* AVX512F: Integer XOR as there is no FP */
 #define LIBXSMM_X86_INSTR_VPXORD         20045
 /* additional integer stuff */
 #define LIBXSMM_X86_INSTR_VPADDQ         20046
@@ -190,26 +179,35 @@
 #define LIBXSMM_X86_INSTR_VPMADDUBSW     20051
 #define LIBXSMM_X86_INSTR_VPADDSW        20052
 #define LIBXSMM_X86_INSTR_VPADDSB        20053
+#define LIBXSMM_X86_INSTR_VPSUBD         20054
 /* Additional vector manipulations */
-#define LIBXSMM_X86_INSTR_VUNPCKLPD      20054
-#define LIBXSMM_X86_INSTR_VUNPCKLPS      20055
-#define LIBXSMM_X86_INSTR_VUNPCKHPD      20056
-#define LIBXSMM_X86_INSTR_VUNPCKHPS      20057
-#define LIBXSMM_X86_INSTR_VPSRAVD        20058
-#define LIBXSMM_X86_INSTR_VCVTDQ2PS      20059
-#define LIBXSMM_X86_INSTR_VDIVPS         20060
-#define LIBXSMM_X86_INSTR_VDIVPD         20061
-#define LIBXSMM_X86_INSTR_VFMADD213PS    20062
-#define LIBXSMM_X86_INSTR_VCVTPS2PD      20063
-#define LIBXSMM_X86_INSTR_VBLENDMPS      20064
-#define LIBXSMM_X86_INSTR_VCMPPS         20065
-#define LIBXSMM_X86_INSTR_VPANDD         20066
-#define LIBXSMM_X86_INSTR_VPANDQ         20067
-#define LIBXSMM_X86_INSTR_VMAXPD         20068
-#define LIBXSMM_X86_INSTR_VMAXPS         20069
-#define LIBXSMM_X86_INSTR_VCVTPS2PH      20070
-#define LIBXSMM_X86_INSTR_VCVTPH2PS      20071
-#define LIBXSMM_X86_INSTR_VPERMD         20072
+#define LIBXSMM_X86_INSTR_VUNPCKLPD      20055
+#define LIBXSMM_X86_INSTR_VUNPCKLPS      20056
+#define LIBXSMM_X86_INSTR_VUNPCKHPD      20057
+#define LIBXSMM_X86_INSTR_VUNPCKHPS      20058
+#define LIBXSMM_X86_INSTR_VPSRAVD        20059
+#define LIBXSMM_X86_INSTR_VCVTDQ2PS      20060
+#define LIBXSMM_X86_INSTR_VDIVPS         20061
+#define LIBXSMM_X86_INSTR_VDIVPD         20062
+#define LIBXSMM_X86_INSTR_VFMADD213PS    20063
+#define LIBXSMM_X86_INSTR_VCVTPS2PD      20064
+#define LIBXSMM_X86_INSTR_VBLENDMPS      20065
+#define LIBXSMM_X86_INSTR_VCMPPS         20066
+#define LIBXSMM_X86_INSTR_VPANDD         20067
+#define LIBXSMM_X86_INSTR_VPANDQ         20068
+#define LIBXSMM_X86_INSTR_VMAXPD         20069
+#define LIBXSMM_X86_INSTR_VMAXPS         20070
+#define LIBXSMM_X86_INSTR_VCVTPS2PH      20071
+#define LIBXSMM_X86_INSTR_VCVTPH2PS      20072
+#define LIBXSMM_X86_INSTR_VPERMD         20073
+#define LIBXSMM_X86_INSTR_VPMOVDW        20074
+#define LIBXSMM_X86_INSTR_VPSRAD         20075
+#define LIBXSMM_X86_INSTR_VPSLLD         20076
+#define LIBXSMM_X86_INSTR_VPCMPD         20077
+#define LIBXSMM_X86_INSTR_VPORD          20078
+#define LIBXSMM_X86_INSTR_VPSRLD         20079
+#define LIBXSMM_X86_INSTR_VPERMT2W       20080
+#define LIBXSMM_X86_INSTR_VPMOVSXWD      20081
 
 /* AVX512, QUAD MADD, QUAD VNNI and VNNI */
 #define LIBXSMM_X86_INSTR_V4FMADDPS      26000
@@ -223,6 +221,11 @@
 #define LIBXSMM_X86_INSTR_VPDPBUSD       26008
 #define LIBXSMM_X86_INSTR_VPDPBUSDS      26009
 
+/* AVX512 BF16 */
+#define LIBXSMM_X86_INSTR_VDPBF16PS      27000
+#define LIBXSMM_X86_INSTR_VCVTNEPS2BF16  27001
+#define LIBXSMM_X86_INSTR_VCVTNE2PS2BF16 27002
+
 /* GP instructions */
 #define LIBXSMM_X86_INSTR_ADDQ           30000
 #define LIBXSMM_X86_INSTR_SUBQ           30001
@@ -241,6 +244,17 @@
 #define LIBXSMM_X86_INSTR_IMUL           30014
 #define LIBXSMM_X86_INSTR_CMOVZ          30015
 #define LIBXSMM_X86_INSTR_CMOVNZ         30016
+#define LIBXSMM_X86_INSTR_JE             30017
+#define LIBXSMM_X86_INSTR_JZ             30018
+#define LIBXSMM_X86_INSTR_JG             30019
+#define LIBXSMM_X86_INSTR_JNE            30020
+#define LIBXSMM_X86_INSTR_JNZ            30021
+#define LIBXSMM_X86_INSTR_JGE            30022
+#define LIBXSMM_X86_INSTR_JLE            30023
+#define LIBXSMM_X86_INSTR_JMP            30024
+#define LIBXSMM_X86_INSTR_POPCNT         30025
+#define LIBXSMM_X86_INSTR_TZCNT          30026
+#define LIBXSMM_X86_INSTR_LEAQ           30027
 
 /* Mask move instructions */
 #define LIBXSMM_X86_INSTR_KMOV           40000
@@ -272,10 +286,8 @@
 #define LIBXSMM_ERR_N_BLOCK              90016
 #define LIBXSMM_ERR_M_BLOCK              90017
 #define LIBXSMM_ERR_K_BLOCK              90018
-#define LIBXSMM_ERR_NO_IMCI              90019
 #define LIBXSMM_ERR_REG_BLOCK            90020
-#define LIBXSMM_ERR_VEC_MOVE_IMCI        90021
-#define LIBXSMM_ERR_NO_IMCI_AVX512_BCAST 90022
+#define LIBXSMM_ERR_NO_AVX512_BCAST      90022
 #define LIBXSMM_ERR_NO_AVX512_QFMA       90023
 #define LIBXSMM_ERR_CALLEE_SAVE_A        90024
 #define LIBXSMM_ERR_CALLEE_SAVE_B        90025
@@ -307,21 +319,21 @@
 #define LIBXSMM_ERR_INVALID_GEMM_CONFIG  90051
 #define LIBXSMM_ERR_UNIQUE_VAL           90052
 #define LIBXSMM_ERR_VEC_REG_MUST_BE_UNDEF 90053
+#define LIBXSMM_ERR_JMPLBL_USED           90054
+#define LIBXSMM_ERR_TRANS_B               90055
+#define LIBXSMM_ERR_LDB_TRANS             90056
 
-#if !defined(LIBXSMM_VERBOSE_BACKEND) && !defined(NDEBUG)
-# define LIBXSMM_VERBOSE_BACKEND
-#endif
-#if defined(LIBXSMM_VERBOSE_BACKEND)
-# define LIBXSMM_BACKEND_VERBOSITY 1
+#if defined(LIBXSMM_HANDLE_ERROR_QUIET)
+# define LIBXSMM_HANDLE_ERROR(GENERATED_CODE, ERROR_CODE)
+# define LIBXSMM_HANDLE_ERROR_VERBOSE(GENERATED_CODE, ERROR_CODE)
 #else
-# define LIBXSMM_BACKEND_VERBOSITY 0
+# define LIBXSMM_HANDLE_ERROR(GENERATED_CODE, ERROR_CODE) libxsmm_handle_error( \
+    GENERATED_CODE, ERROR_CODE, LIBXSMM_FUNCNAME, 0 != libxsmm_ninit ? libxsmm_verbosity : 1)
+# define LIBXSMM_HANDLE_ERROR_VERBOSE(GENERATED_CODE, ERROR_CODE) libxsmm_handle_error( \
+    GENERATED_CODE, ERROR_CODE, LIBXSMM_FUNCNAME, 1)
 #endif
-#define LIBXSMM_HANDLE_ERROR(GENERATED_CODE, ERROR_CODE) libxsmm_handle_error( \
-  GENERATED_CODE, ERROR_CODE, LIBXSMM_CALLER, LIBXSMM_BACKEND_VERBOSITY)
-#define LIBXSMM_HANDLE_ERROR_VERBOSE(GENERATED_CODE, ERROR_CODE) libxsmm_handle_error( \
-  GENERATED_CODE, ERROR_CODE, LIBXSMM_CALLER, 1)
 
-/* micro kernel config */
+/* micro kernel configuration */
 LIBXSMM_EXTERN_C typedef struct libxsmm_micro_kernel_config {
   unsigned int instruction_set;
   unsigned int vector_reg_count;
@@ -331,6 +343,7 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_micro_kernel_config {
   unsigned int b_vmove_instruction;
   unsigned int b_shuff_instruction;
   unsigned int c_vmove_instruction;
+  unsigned int c_vmove_nts_instruction;
   unsigned int use_masking_a_c;
   unsigned int prefetch_instruction;
   unsigned int vxor_instruction;
@@ -344,7 +357,7 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_micro_kernel_config {
   char vector_name;
 } libxsmm_micro_kernel_config;
 
-/* struct for storing the current gp reg mapping */
+/* structure for storing the current gp reg mapping */
 LIBXSMM_EXTERN_C typedef struct libxsmm_gp_reg_mapping_struct {
   unsigned int gp_reg_a;
   unsigned int gp_reg_b;
@@ -355,6 +368,8 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_gp_reg_mapping_struct {
   unsigned int gp_reg_mloop;
   unsigned int gp_reg_nloop;
   unsigned int gp_reg_kloop;
+  unsigned int gp_reg_reduce_count;
+  unsigned int gp_reg_reduce_loop;
   unsigned int gp_reg_help_0;
   unsigned int gp_reg_help_1;
   unsigned int gp_reg_help_2;
@@ -363,7 +378,7 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_gp_reg_mapping_struct {
   unsigned int gp_reg_help_5;
 } libxsmm_gp_reg_mapping;
 
-/* convolution kernel config */
+/* convolution kernel configuration */
 LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_kernel_config_struct {
   unsigned int instruction_set;
   unsigned int vector_reg_count;
@@ -392,13 +407,15 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_kernel_config_struct {
   unsigned int l_ld_ofm_fil;
 } libxsmm_convolution_kernel_config;
 
-/* struct for storing the current gp reg mapping for convolution */
+/* structure for storing the current gp reg mapping for convolution */
 LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_forward_gp_reg_mapping_struct {
   unsigned int gp_reg_input;
   unsigned int gp_reg_weight;
   unsigned int gp_reg_output;
   unsigned int gp_reg_input_pf;
+  unsigned int gp_reg_input_pf_L2;
   unsigned int gp_reg_weight_pf;
+  unsigned int gp_reg_weight_pf_L2;
   unsigned int gp_reg_output_pf;
   unsigned int gp_reg_kw_loop;
   unsigned int gp_reg_kh_loop;
@@ -413,28 +430,7 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_forward_gp_reg_mapping_struc
   unsigned int gp_reg_help_6;
 } libxsmm_convolution_forward_gp_reg_mapping;
 
-/* struct for storing the current gp reg mapping for convolution */
-LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_backward_gp_reg_mapping_struct {
-  unsigned int gp_reg_input;
-  unsigned int gp_reg_weight;
-  unsigned int gp_reg_output;
-  unsigned int gp_reg_input_pf;
-  unsigned int gp_reg_weight_pf;
-  unsigned int gp_reg_output_pf;
-  unsigned int gp_reg_kw_loop;
-  unsigned int gp_reg_oi_loop;
-  unsigned int gp_reg_kh_loop;
-  unsigned int gp_reg_ofmInner_loop;
-  unsigned int gp_reg_help_0;
-  unsigned int gp_reg_help_1;
-  unsigned int gp_reg_help_2;
-  unsigned int gp_reg_help_3;
-  unsigned int gp_reg_help_4;
-  unsigned int gp_reg_help_5;
-  unsigned int gp_reg_help_6;
-} libxsmm_convolution_backward_gp_reg_mapping;
-
-/* struct for storing the current gp reg mapping for convolution */
+/* structure for storing the current gp reg mapping for convolution */
 LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_weight_update_gp_reg_mapping_struct {
   unsigned int gp_reg_input;
   unsigned int gp_reg_weight;
@@ -454,7 +450,7 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_weight_update_gp_reg_mapping
   unsigned int gp_reg_help_6;
 } libxsmm_convolution_weight_update_gp_reg_mapping;
 
-/* struct for storing the current gp reg mapping for matcopy */
+/* structure for storing the current gp reg mapping for matcopy */
 LIBXSMM_EXTERN_C typedef struct libxsmm_matcopy_gp_reg_mapping_struct {
   unsigned int gp_reg_a;
   unsigned int gp_reg_lda;
@@ -467,7 +463,7 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_matcopy_gp_reg_mapping_struct {
   unsigned int gp_reg_help_0;
 } libxsmm_matcopy_gp_reg_mapping;
 
-/* matcopy kernel config */
+/* matcopy kernel configuration */
 LIBXSMM_EXTERN_C typedef struct libxsmm_matcopy_kernel_config_struct {
   unsigned int instruction_set;
   unsigned int vector_reg_count;
@@ -483,7 +479,7 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_matcopy_kernel_config_struct {
   char vector_name;
 } libxsmm_matcopy_kernel_config;
 
-/* struct for storing the current gp reg mapping for transpose */
+/* structure for storing the current gp reg mapping for transpose */
 LIBXSMM_EXTERN_C typedef struct libxsmm_transpose_gp_reg_mapping_struct {
   unsigned int gp_reg_a;
   unsigned int gp_reg_lda;
@@ -499,23 +495,38 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_transpose_gp_reg_mapping_struct {
   unsigned int gp_reg_help_5;
 } libxsmm_transpose_gp_reg_mapping;
 
-/* transpose kernel config */
+/* transpose kernel configuration */
 LIBXSMM_EXTERN_C typedef struct libxsmm_transpose_kernel_config_struct {
   unsigned int instruction_set;
   unsigned int vector_reg_count;
   char vector_name;
 } libxsmm_transpose_kernel_config;
 
-/* struct for tracking local labels in assembly
-   we don't allow overlapping loops */
+/* structure for tracking local labels in assembly we don't allow overlapping loops */
 LIBXSMM_EXTERN_C typedef struct libxsmm_loop_label_tracker_struct {
   unsigned int label_address[32];
   unsigned int label_count;
 } libxsmm_loop_label_tracker;
 
+/* structure to save jump properties to the same destination */
+LIBXSMM_EXTERN_C typedef struct libxsmm_jump_source_struct {
+  unsigned int instr_type[32];
+  unsigned int instr_addr[32];
+  unsigned int ref_count;
+} libxsmm_jump_source;
+
+/* structure for tracking arbitrary jump labels in assembly code */
+LIBXSMM_EXTERN_C typedef struct libxsmm_jump_label_tracker_struct {
+  unsigned int        label_address[32];
+  libxsmm_jump_source label_source[32];
+} libxsmm_jump_label_tracker;
+
 LIBXSMM_API_INTERN
 void libxsmm_reset_loop_label_tracker( libxsmm_loop_label_tracker* io_loop_label_tracker );
 
+LIBXSMM_API_INTERN
+void libxsmm_reset_jump_label_tracker( libxsmm_jump_label_tracker* io_jump_lable_tracker );
+
 LIBXSMM_API_INTERN
 void libxsmm_get_x86_gp_reg_name( const unsigned int i_gp_reg_number,
                                   char*              o_gp_reg_name,
@@ -535,8 +546,7 @@ void libxsmm_reset_x86_gp_reg_mapping( libxsmm_gp_reg_mapping* io_gp_reg_mapping
 LIBXSMM_API_INTERN
 unsigned int libxsmm_is_x86_vec_instr_single_precision( const unsigned int i_instr_number );
 
-/* some string manipulation helper needed to
-   generated code */
+/* some string manipulation helper needed to generated code */
 LIBXSMM_API_INTERN
 void libxsmm_append_code_as_string( libxsmm_generated_code* io_generated_code,
                                     const char*             i_code_to_append,
@@ -572,5 +582,10 @@ LIBXSMM_API_INTERN
 void libxsmm_convfunction_signature_int16( libxsmm_generated_code*         io_generated_code,
                                            const char*                     i_routine_name     );
 
+LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
+  unsigned int i_size, unsigned int i_max_block,
+  unsigned int* o_range_1, unsigned int* o_block_1,
+  unsigned int* o_range_2, unsigned int* o_block_2 );
+
 #endif /* GENERATOR_COMMON_H */
 
