@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -80,22 +80,25 @@ void libxsmm_generator_convolution_weight_update_transpose_avx512_ofwloop( libxs
                                                            const int                                               no_unroll_no_block);
 
 LIBXSMM_API_INTERN
-void libxsmm_generator_convolution_weight_update_transpose_avx512_ofwloop_sfma( libxsmm_generated_code*                           io_generated_code,
-                                                                const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
-                                                                const libxsmm_convolution_kernel_config*                i_conv_kernel_config,
-                                                                const libxsmm_convolution_weight_update_descriptor*                       i_conv_desc,
-                                                                const unsigned int                                      i_ofh_unroll,
-            const unsigned int ofh_trip_counter,
-                                                                const int                                               no_unroll_no_block);
+void libxsmm_generator_convolution_weight_update_transpose_avx512_ofwloop_all_pixels_inside( libxsmm_generated_code*               io_generated_code,
+                                                                                const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
+                                                                                const libxsmm_convolution_kernel_config*                i_conv_kernel_config,
+                                                                                const libxsmm_convolution_weight_update_descriptor*     i_conv_desc,
+                                                                                unsigned int                                            is_last_call);
 
 LIBXSMM_API_INTERN
-void libxsmm_generator_convolution_weight_update_transpose_avx512_ofwloop_qfma( libxsmm_generated_code*                           io_generated_code,
-                                                                const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
-                                                                const libxsmm_convolution_kernel_config*                i_conv_kernel_config,
-                                                                const libxsmm_convolution_weight_update_descriptor*                       i_conv_desc,
-                                                                const unsigned int                                      i_ofh_unroll,
-            const unsigned int ofh_trip_counter,
-                                                                const int                                               no_unroll_no_block);
+void libxsmm_generator_convolution_weight_update_avx512_ofwloop_all_pixels_inside( libxsmm_generated_code*               io_generated_code,
+                                                                                const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
+                                                                                const libxsmm_convolution_kernel_config*                i_conv_kernel_config,
+                                                                                const libxsmm_convolution_weight_update_descriptor*     i_conv_desc,
+                                                                                unsigned int                                            is_last_call);
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_convolution_weight_update_avx512_c3k64s2_bf16_all_pixels_inside( libxsmm_generated_code* io_generated_code,
+    const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
+    const libxsmm_convolution_kernel_config*                i_conv_kernel_config,
+    const libxsmm_convolution_weight_update_descriptor*     i_conv_desc,
+    libxsmm_loop_label_tracker* loop_label_tracker);
 
 #endif /* GENERATOR_CONVOLUTION_WEIGHT_UPDATE_AVX512_H */
 
