@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2018, Intel Corporation                                **
+** Copyright (c) 2016-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -59,6 +59,8 @@ void libxsmm_generator_gemm_avx512_microkernel_nofsdbcst( libxsmm_generated_code
   unsigned int l_m = 0;
   /* start register of accumulator */
   unsigned int l_vec_reg_acc_start = i_micro_kernel_config->vector_reg_count - (i_n_blocking * l_m_blocking);
+  /* temp variable for b-offset to handle no-trans/trans B */
+  int l_b_offset = 0;
 
 #if !defined(NDEBUG)
   if ( (i_n_blocking > 6) || (i_n_blocking < 1) ) {
@@ -80,7 +82,7 @@ void libxsmm_generator_gemm_avx512_microkernel_nofsdbcst( libxsmm_generated_code
                                   LIBXSMM_X86_GP_REG_UNDEF, 0,
                                   0,
                                   i_micro_kernel_config->vector_name,
-                                  i_n_blocking, i_micro_kernel_config->use_masking_a_c, 0 );
+                                  i_n_blocking, i_micro_kernel_config->use_masking_a_c, 1, 0 );
     /* loop over columns of B */
     for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
       /* post increment of a pointer early */
@@ -92,28 +94,49 @@ void libxsmm_generator_gemm_avx512_microkernel_nofsdbcst( libxsmm_generated_code
       }
       /* different ways of using B */
       if ( i_offset != (-1) ) {
+        /* handle trans B */
+        if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+          l_b_offset = (i_micro_kernel_config->datatype_size * i_offset * i_xgemm_desc->ldb) + (l_n * i_micro_kernel_config->datatype_size);
+        } else {
+          l_b_offset = (i_micro_kernel_config->datatype_size * i_offset) + (i_xgemm_desc->ldb * l_n * i_micro_kernel_config->datatype_size);
+        }
+
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->b_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_b,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      ((i_micro_kernel_config->datatype_size) * i_offset) + (i_xgemm_desc->ldb * l_n * (i_micro_kernel_config->datatype_size)),
+                                      l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
       } else {
+        /* handle trans B */
+        if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+          l_b_offset = l_n * i_micro_kernel_config->datatype_size;
+        } else {
+          l_b_offset = i_xgemm_desc->ldb * l_n * i_micro_kernel_config->datatype_size;
+        }
+
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->b_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_b,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      i_xgemm_desc->ldb * l_n *  i_micro_kernel_config->datatype_size,
+                                      l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
         if ( l_n == (i_n_blocking -1) ) {
+          /* handle trans B */
+          if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+            l_b_offset = i_xgemm_desc->ldb * i_micro_kernel_config->datatype_size;
+          } else {
+            l_b_offset = i_micro_kernel_config->datatype_size;
+          }
+
           libxsmm_x86_instruction_alu_imm( io_generated_code,
                                        i_micro_kernel_config->alu_add_instruction,
                                        i_gp_reg_mapping->gp_reg_b,
-                                       i_micro_kernel_config->datatype_size );
+                                       l_b_offset );
         }
       }
       /* issue fma */
@@ -129,30 +152,51 @@ void libxsmm_generator_gemm_avx512_microkernel_nofsdbcst( libxsmm_generated_code
     /* broadcast from B -> into vec registers 0 to i_n_blocking */
     if ( i_offset != (-1) ) {
       for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
+        /* handle trans B */
+        if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+          l_b_offset = (i_micro_kernel_config->datatype_size * i_offset * i_xgemm_desc->ldb) + (l_n * i_micro_kernel_config->datatype_size);
+        } else {
+          l_b_offset = (i_micro_kernel_config->datatype_size * i_offset) + (i_xgemm_desc->ldb * l_n * i_micro_kernel_config->datatype_size);
+        }
+
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->b_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_b,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      ((i_micro_kernel_config->datatype_size) * i_offset) + (i_xgemm_desc->ldb * l_n * (i_micro_kernel_config->datatype_size)),
+                                      l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
       }
     } else {
       for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
+        /* handle trans B */
+        if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+          l_b_offset = l_n * i_micro_kernel_config->datatype_size;
+        } else {
+          l_b_offset = i_xgemm_desc->ldb * l_n * i_micro_kernel_config->datatype_size;
+        }
+
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->b_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_b,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      i_xgemm_desc->ldb * l_n *  i_micro_kernel_config->datatype_size,
+                                      l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 0 );
-     }
-     libxsmm_x86_instruction_alu_imm( io_generated_code,
+                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+      }
+      /* handle trans B */
+      if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+        l_b_offset = i_xgemm_desc->ldb * i_micro_kernel_config->datatype_size;
+      } else {
+        l_b_offset = i_micro_kernel_config->datatype_size;
+      }
+
+      libxsmm_x86_instruction_alu_imm( io_generated_code,
                                   i_micro_kernel_config->alu_add_instruction,
                                   i_gp_reg_mapping->gp_reg_b,
-                                  i_micro_kernel_config->datatype_size );
+                                  l_b_offset );
     }
 
     if (l_m_blocking == 4) {
@@ -165,55 +209,69 @@ void libxsmm_generator_gemm_avx512_microkernel_nofsdbcst( libxsmm_generated_code
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       (i_micro_kernel_config->datatype_size) * (i_micro_kernel_config->vector_length) * l_m,
                                       i_micro_kernel_config->vector_name,
-                                      i_n_blocking, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      i_n_blocking, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+
+        /* In case of batch reduce try to prefetch a few more columns ahead...  */
+        if (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_BATCH_REDUCE) {
+          unsigned int pf_a_cols_ahead = 16;
+          if (i_xgemm_desc->lda == 1024) {
+            pf_a_cols_ahead = 4;
+          }
+          libxsmm_x86_instruction_prefetch( io_generated_code,
+              LIBXSMM_X86_INSTR_PREFETCHT0,
+              i_gp_reg_mapping->gp_reg_a,
+              LIBXSMM_X86_GP_REG_UNDEF, 0,
+              (i_micro_kernel_config->datatype_size) * (i_micro_kernel_config->vector_length) * l_m + pf_a_cols_ahead * i_xgemm_desc->lda * i_micro_kernel_config->datatype_size);
+        }
 
         for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
           /* post increment early */
           if ( (l_m == (l_m_blocking-1)) && (l_n == 0) ) {
             libxsmm_x86_instruction_alu_imm( io_generated_code,
-                                         i_micro_kernel_config->alu_add_instruction,
-                                         i_gp_reg_mapping->gp_reg_a,
-                                         (i_xgemm_desc->lda)*(i_micro_kernel_config->datatype_size) );
+                i_micro_kernel_config->alu_add_instruction,
+                i_gp_reg_mapping->gp_reg_a,
+                (i_xgemm_desc->lda)*(i_micro_kernel_config->datatype_size) );
           }
+
           /* issue fma */
           libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
-                                               i_micro_kernel_config->instruction_set,
-                                               i_micro_kernel_config->vmul_instruction,
-                                               i_micro_kernel_config->vector_name,
-                                               i_n_blocking,
-                                               l_n,
-                                               l_vec_reg_acc_start + l_m + (l_m_blocking * l_n) );
+              i_micro_kernel_config->instruction_set,
+              i_micro_kernel_config->vmul_instruction,
+              i_micro_kernel_config->vector_name,
+              i_n_blocking,
+              l_n,
+              l_vec_reg_acc_start + l_m + (l_m_blocking * l_n) );
         }
       }
     } else {
       /* load column vectors of A and multiply with all broadcasted row entries of B */
       for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
         libxsmm_x86_instruction_vec_move( io_generated_code,
-                                      i_micro_kernel_config->instruction_set,
-                                      i_micro_kernel_config->a_vmove_instruction,
-                                      i_gp_reg_mapping->gp_reg_a,
-                                      LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      (i_micro_kernel_config->datatype_size) * (i_micro_kernel_config->vector_length) * l_m,
-                                      i_micro_kernel_config->vector_name,
-                                      i_n_blocking+l_m, i_micro_kernel_config->use_masking_a_c, 0 );
+            i_micro_kernel_config->instruction_set,
+            i_micro_kernel_config->a_vmove_instruction,
+            i_gp_reg_mapping->gp_reg_a,
+            LIBXSMM_X86_GP_REG_UNDEF, 0,
+            (i_micro_kernel_config->datatype_size) * (i_micro_kernel_config->vector_length) * l_m,
+            i_micro_kernel_config->vector_name,
+            i_n_blocking+l_m, i_micro_kernel_config->use_masking_a_c, 1, 0 );
       }
       for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
         for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
           /* post increment early */
           if ( (l_m == (l_m_blocking-1)) && (l_n == 0) ) {
             libxsmm_x86_instruction_alu_imm( io_generated_code,
-                                         i_micro_kernel_config->alu_add_instruction,
-                                         i_gp_reg_mapping->gp_reg_a,
-                                         (i_xgemm_desc->lda)*(i_micro_kernel_config->datatype_size) );
+                i_micro_kernel_config->alu_add_instruction,
+                i_gp_reg_mapping->gp_reg_a,
+                (i_xgemm_desc->lda)*(i_micro_kernel_config->datatype_size) );
           }
           /* issue fma */
           libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
-                                               i_micro_kernel_config->instruction_set,
-                                               i_micro_kernel_config->vmul_instruction,
-                                               i_micro_kernel_config->vector_name,
-                                               i_n_blocking+l_m,
-                                               l_n,
-                                               l_vec_reg_acc_start + l_m + (l_m_blocking * l_n) );
+              i_micro_kernel_config->instruction_set,
+              i_micro_kernel_config->vmul_instruction,
+              i_micro_kernel_config->vector_name,
+              i_n_blocking+l_m,
+              l_n,
+              l_vec_reg_acc_start + l_m + (l_m_blocking * l_n) );
         }
       }
     }
