@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -74,6 +74,19 @@ void libxsmm_generator_gemm_footer_kloop( libxsmm_generated_code*             io
                                            const unsigned int                 i_max_blocked_k,
                                            const unsigned int                 i_kloop_complete );
 
+LIBXSMM_API_INTERN
+void libxsmm_generator_gemm_header_reduceloop( libxsmm_generated_code*             io_generated_code,
+                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config);
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_gemm_footer_reduceloop( libxsmm_generated_code*             io_generated_code,
+                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                           const libxsmm_gemm_descriptor*     i_xgemm_desc);
+
 LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_header_nloop( libxsmm_generated_code*             io_generated_code,
                                            libxsmm_loop_label_tracker*        io_loop_label_tracker,
