@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -48,34 +48,8 @@ void libxsmm_generator_gemm_noarch_kernel( libxsmm_generated_code*        io_gen
   char l_new_code[512];
   int l_max_code_length = 511;
   int l_code_length = 0;
-
   LIBXSMM_UNUSED(i_arch);
 
-#ifdef LIBXSMM_GENERATOR_MKL_BLAS_FALLBACK
-  if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype )  ) {
-    double l_alpha = 1.0;
-    double l_beta = 1.0;
-    if ( i_xgemm_desc->beta == 0 ) {
-      l_beta = 0.0;
-    }
-    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length,
-       "cblas_dgemm(CblasColMajor, CblasNoTrans, CblasNoTrans, %u, %u, %u, %f, A, %u, B, %u, %f, C, %u);\n",
-       (unsigned int)i_xgemm_desc->m, (unsigned int)i_xgemm_desc->n, (unsigned int)i_xgemm_desc->k, l_alpha,
-       (unsigned int)i_xgemm_desc->m, (unsigned int)i_xgemm_desc->k, l_beta, (unsigned int)i_xgemm_desc->m);
-       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-  } else {
-    float l_alpha = 1.0f;
-    float l_beta = 1.0f;
-    if ( i_xgemm_desc->beta == 0 ) {
-      l_beta = 0.0f;
-    }
-    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length,
-       "cblas_sgemm(CblasColMajor, CblasNoTrans, CblasNoTrans, %u, %u, %u, %f, A, %u, B, %u, %f, C, %u);\n",
-       (unsigned int)i_xgemm_desc->m, (unsigned int)i_xgemm_desc->n, (unsigned int)i_xgemm_desc->k, l_alpha,
-       (unsigned int)i_xgemm_desc->m, (unsigned int)i_xgemm_desc->k, l_beta, (unsigned int)i_xgemm_desc->m);
-       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-  }
-#else
   l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  unsigned int l_m = 0;\n");
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
   l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  unsigned int l_n = 0;\n");
@@ -85,7 +59,7 @@ void libxsmm_generator_gemm_noarch_kernel( libxsmm_generated_code*        io_gen
 
   l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  for ( l_n = 0; l_n < %u; l_n++ ) {\n", (unsigned int)i_xgemm_desc->n);
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-  if ( i_xgemm_desc->beta == 0 ) {
+  if (0 != (LIBXSMM_GEMM_FLAG_BETA_0 & i_xgemm_desc->flags)) { /* Beta=0 */
     if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype )  ) {
       l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "    for ( l_m = 0; l_m < %u; l_m++ ) { C[(l_n*%u)+l_m] = 0.0; }\n\n", (unsigned int)i_xgemm_desc->m, (unsigned int)i_xgemm_desc->ldc);
     } else {
@@ -107,6 +81,5 @@ void libxsmm_generator_gemm_noarch_kernel( libxsmm_generated_code*        io_gen
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
   l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  }\n");
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-#endif
 }
 
