@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -59,6 +59,8 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
   unsigned int l_m = 0;
   /* start register of accumulator */
   unsigned int l_vec_reg_acc_start = 16 - (i_n_blocking * l_m_blocking);
+  /* temp variable for b-offset to handle no-trans/trans B */
+  int l_b_offset = 0;
 
 #if !defined(NDEBUG)
   if ( (i_n_blocking > 3) || (i_n_blocking < 1) ) {
@@ -80,7 +82,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                   LIBXSMM_X86_GP_REG_UNDEF, 0,
                                   0,
                                   i_micro_kernel_config->vector_name,
-                                  i_n_blocking, i_micro_kernel_config->use_masking_a_c, 0 );
+                                  i_n_blocking, i_micro_kernel_config->use_masking_a_c, 1, 0 );
     /* loop over columns of B */
     for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
       /* post increment of a pointer early */
@@ -92,14 +94,21 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
       }
       /* different ways of using B */
       if ( i_offset != (-1) ) {
+        /* handle trans B */
+        if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+          l_b_offset = (i_micro_kernel_config->datatype_size * i_offset * i_xgemm_desc->ldb) + (l_n * i_micro_kernel_config->datatype_size);
+        } else {
+          l_b_offset = (i_micro_kernel_config->datatype_size * i_offset) + (i_xgemm_desc->ldb * l_n * i_micro_kernel_config->datatype_size);
+        }
+
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->b_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_b,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      ((i_micro_kernel_config->datatype_size) * i_offset) + (i_xgemm_desc->ldb * l_n * (i_micro_kernel_config->datatype_size)),
+                                      l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
         /* generate shuffle as SSE3 has no broadcast load for single precision */
         if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && ( i_micro_kernel_config->b_shuff_instruction != LIBXSMM_X86_INSTR_UNDEF ) ) {
           libxsmm_x86_instruction_vec_shuffle_reg( io_generated_code,
@@ -112,14 +121,21 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                                0 );
         }
       } else {
+        /* handle trans B */
+        if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+          l_b_offset = l_n * i_micro_kernel_config->datatype_size;
+        } else {
+          l_b_offset = i_xgemm_desc->ldb * l_n * i_micro_kernel_config->datatype_size;
+        }
+
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->b_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_b,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      i_xgemm_desc->ldb * l_n *  i_micro_kernel_config->datatype_size,
+                                      l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
         /* generate shuffle as SSE3 has no broadcast load for single precision */
         if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && ( i_micro_kernel_config->b_shuff_instruction != LIBXSMM_X86_INSTR_UNDEF ) ) {
           libxsmm_x86_instruction_vec_shuffle_reg( io_generated_code,
@@ -132,10 +148,17 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                                0 );
         }
         if ( l_n == (i_n_blocking -1) ) {
+          /* handle trans B */
+          if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+            l_b_offset = i_xgemm_desc->ldb * i_micro_kernel_config->datatype_size;
+          } else {
+            l_b_offset = i_micro_kernel_config->datatype_size;
+          }
+
           libxsmm_x86_instruction_alu_imm( io_generated_code,
                                        i_micro_kernel_config->alu_add_instruction,
                                        i_gp_reg_mapping->gp_reg_b,
-                                       i_micro_kernel_config->datatype_size );
+                                       l_b_offset );
         }
       }
       /* issue mul-add */
@@ -158,14 +181,21 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
     /* broadcast from B -> into vec registers 0 to i_n_blocking */
     if ( i_offset != (-1) ) {
       for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
+        /* handle trans B */
+        if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+          l_b_offset = (i_micro_kernel_config->datatype_size * i_offset * i_xgemm_desc->ldb) + (l_n * i_micro_kernel_config->datatype_size);
+        } else {
+          l_b_offset = (i_micro_kernel_config->datatype_size * i_offset) + (i_xgemm_desc->ldb * l_n * i_micro_kernel_config->datatype_size);
+        }
+
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->b_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_b,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      ((i_micro_kernel_config->datatype_size) * i_offset) + (i_xgemm_desc->ldb * l_n * (i_micro_kernel_config->datatype_size)),
+                                      l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
         /* generate shuffle as SSE3 has no broadcast load for single precision */
         if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && ( i_micro_kernel_config->b_shuff_instruction != LIBXSMM_X86_INSTR_UNDEF ) ) {
           libxsmm_x86_instruction_vec_shuffle_reg( io_generated_code,
@@ -180,14 +210,21 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
       }
     } else {
       for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
+        /* handle trans B */
+        if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+          l_b_offset = l_n * i_micro_kernel_config->datatype_size;
+        } else {
+          l_b_offset = i_xgemm_desc->ldb * l_n * i_micro_kernel_config->datatype_size;
+        }
+
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->b_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_b,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                      i_xgemm_desc->ldb * l_n *  i_micro_kernel_config->datatype_size,
+                                      l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
         /* generate shuffle as SSE3 has no broadcast load for single precision */
         if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) ) {
           libxsmm_x86_instruction_vec_shuffle_reg( io_generated_code,
@@ -199,11 +236,18 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                                LIBXSMM_X86_VEC_REG_UNDEF,
                                                0 );
         }
-     }
-     libxsmm_x86_instruction_alu_imm( io_generated_code,
+      }
+      /* handle trans B */
+      if ( (i_xgemm_desc->flags & LIBXSMM_GEMM_FLAG_TRANS_B) > 0 ) {
+        l_b_offset = i_xgemm_desc->ldb * i_micro_kernel_config->datatype_size;
+      } else {
+        l_b_offset = i_micro_kernel_config->datatype_size;
+      }
+
+      libxsmm_x86_instruction_alu_imm( io_generated_code,
                                   i_micro_kernel_config->alu_add_instruction,
                                   i_gp_reg_mapping->gp_reg_b,
-                                  i_micro_kernel_config->datatype_size );
+                                  l_b_offset );
     }
 
     if (l_m_blocking == 3) {
@@ -216,7 +260,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       (i_micro_kernel_config->datatype_size) * (i_micro_kernel_config->vector_length) * l_m,
                                       i_micro_kernel_config->vector_name,
-                                      i_n_blocking, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      i_n_blocking, i_micro_kernel_config->use_masking_a_c, 1, 0 );
 
         for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
           /* post increment early */
@@ -263,7 +307,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       (i_micro_kernel_config->datatype_size) * (i_micro_kernel_config->vector_length) * l_m,
                                       i_micro_kernel_config->vector_name,
-                                      i_n_blocking + l_m, i_micro_kernel_config->use_masking_a_c, 0 );
+                                      i_n_blocking + l_m, i_micro_kernel_config->use_masking_a_c, 1, 0 );
       }
       for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
         for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
