@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -103,7 +103,7 @@ void libxsmm_generator_spgemm_csc_bsparse( libxsmm_generated_code*         io_ge
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
 
   /* reset C if beta is zero */
-  if ( i_xgemm_desc->beta == 0 ) {
+  if (0 != (LIBXSMM_GEMM_FLAG_BETA_0 & i_xgemm_desc->flags)) { /* Beta=0 */
     l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  unsigned int l_n = 0;\n");
     libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  for ( l_n = 0; l_n < %u; l_n++) {\n", (unsigned int)i_xgemm_desc->n);
@@ -148,9 +148,10 @@ void libxsmm_generator_spgemm_csc_bsparse( libxsmm_generated_code*         io_ge
       l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  #pragma vector aligned\n");
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     }
-  } else if ( ( strcmp( i_arch, "knc" ) == 0 ) ||
-              ( strcmp( i_arch, "knl" ) == 0 ) ||
-              ( strcmp( i_arch, "skx" ) == 0 ) ) {
+  } else if ( ( strcmp( i_arch, "knl" ) == 0 ) ||
+              ( strcmp( i_arch, "skx" ) == 0 ) ||
+              ( strcmp( i_arch, "clx" ) == 0 ) ||
+              ( strcmp( i_arch, "cpx" ) == 0 ) ) {
     if ( (i_xgemm_desc->m > 1)          &&
          ((LIBXSMM_GEMM_FLAG_ALIGN_A & i_xgemm_desc->flags) != 0) &&
          ((LIBXSMM_GEMM_FLAG_ALIGN_C & i_xgemm_desc->flags) != 0) ) {
