@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -106,8 +106,9 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
   while (fgets(l_line, l_line_length, l_csc_file_handle) != NULL) {
     if ( strlen(l_line) == l_line_length ) {
       free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_column_idx_id);
-      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_READ_LEN );
+      *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
       fclose( l_csc_file_handle ); /* close mtx file */
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_READ_LEN );
       return;
     }
     /* check if we are still reading comments header */
@@ -116,10 +117,12 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
     } else {
       /* if we are the first line after comment header, we allocate our data structures */
       if ( l_header_read == 0 ) {
-        if ( sscanf(l_line, "%u %u %u", o_row_count, o_column_count, o_element_count) == 3 ) {
+        if (3 == sscanf(l_line, "%u %u %u", o_row_count, o_column_count, o_element_count) &&
+            0 != *o_row_count && 0 != *o_column_count && 0 != *o_element_count)
+        {
           /* allocate CSC data structure matching mtx file */
           *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_element_count));
-          *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_column_count + 1));
+          *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * ((size_t)(*o_column_count) + 1));
           *o_values = (double*) malloc(sizeof(double) * (*o_element_count));
           l_column_idx_id = (unsigned int*) malloc(sizeof(unsigned int) * (*o_column_count));
 
@@ -129,21 +132,22 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
                ( *o_values == NULL )       ||
                ( l_column_idx_id == NULL ) ) {
             free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_column_idx_id);
+            *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+            fclose(l_csc_file_handle); /* close mtx file */
             LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_ALLOC_DATA );
-            fclose( l_csc_file_handle ); /* close mtx file */
             return;
           }
 
           /* set everything to zero for init */
-          memset(*o_row_idx, 0, sizeof(unsigned int)*(*o_element_count));
-          memset(*o_column_idx, 0, sizeof(unsigned int)*(*o_column_count + 1));
-          memset(*o_values, 0, sizeof(double)*(*o_element_count));
-          memset(l_column_idx_id, 0, sizeof(unsigned int)*(*o_column_count));
+          memset(*o_row_idx, 0, sizeof(unsigned int) * (*o_element_count));
+          memset(*o_column_idx, 0, sizeof(unsigned int) * ((size_t)(*o_column_count) + 1));
+          memset(*o_values, 0, sizeof(double) * (*o_element_count));
+          memset(l_column_idx_id, 0, sizeof(unsigned int) * (*o_column_count));
 
           /* init column idx */
-          for ( l_i = 0; l_i < (*o_column_count + 1); l_i++)
-            (*o_column_idx)[l_i] = (*o_element_count);
-
+          for (l_i = 0; l_i <= *o_column_count; ++l_i) {
+            (*o_column_idx)[l_i] = *o_element_count;
+          }
           /* init */
           (*o_column_idx)[0] = 0;
           l_i = 0;
@@ -160,13 +164,14 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
         /* read a line of content */
         if ( sscanf(l_line, "%u %u %lf", &l_row, &l_column, &l_value) != 3 ) {
           free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_column_idx_id);
+          *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+          fclose(l_csc_file_handle); /* close mtx file */
           LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_READ_ELEMS );
-          fclose( l_csc_file_handle ); /* close mtx file */
           return;
         }
         /* adjust numbers to zero termination */
-        l_row--;
-        l_column--;
+        LIBXSMM_ASSERT(0 != l_row && 0 != l_column);
+        l_row--; l_column--;
         /* add these values to row and value structure */
         (*o_row_idx)[l_i] = l_row;
         (*o_values)[l_i] = l_value;
@@ -184,6 +189,7 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
   /* check if we read a file which was consistent */
   if ( l_i != (*o_element_count) ) {
     free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_column_idx_id);
+    *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
     LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_LEN );
     return;
   }
@@ -201,4 +207,3 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
   }
 }
 
-
