@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -64,7 +64,7 @@ void libxsmm_generator_spgemm_csr_asparse( libxsmm_generated_code*         io_ge
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
 
   /* reset C if beta is zero */
-  if ( i_xgemm_desc->beta == 0 ) {
+  if (0 != (LIBXSMM_GEMM_FLAG_BETA_0 & i_xgemm_desc->flags)) { /* Beta=0 */
     l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  unsigned int l_m = 0;\n");
     libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  for ( l_m = 0; l_m < %u; l_m++) {\n", (unsigned int)i_xgemm_desc->m);
@@ -104,9 +104,11 @@ void libxsmm_generator_spgemm_csr_asparse( libxsmm_generated_code*         io_ge
     } else {}
 
 
-  } else if ( ( strcmp( i_arch, "knc" ) == 0 ) ||
-              ( strcmp( i_arch, "knl" ) == 0 ) ||
-              ( strcmp( i_arch, "skx" ) == 0 ) ) {
+  } else if ( ( strcmp( i_arch, "knl" ) == 0 ) ||
+              ( strcmp( i_arch, "knm" ) == 0 ) ||
+              ( strcmp( i_arch, "skx" ) == 0 ) ||
+              ( strcmp( i_arch, "clx" ) == 0 ) ||
+              ( strcmp( i_arch, "cpx" ) == 0 ) ) {
     if ( (i_xgemm_desc->n > 1) ) {
       l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  #pragma simd vectorlength(16)\n");
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
