@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -53,6 +53,8 @@ void libxsmm_generator_spgemm_csr_asparse_soa( libxsmm_generated_code*         i
   if ( strcmp(i_arch, "knl") == 0 ||
        strcmp(i_arch, "knm") == 0 ||
        strcmp(i_arch, "skx") == 0 ||
+       strcmp(i_arch, "clx") == 0 ||
+       strcmp(i_arch, "cpx") == 0 ||
        strcmp(i_arch, "snb") == 0 ||
        strcmp(i_arch, "hsw") == 0 ) {
     libxsmm_generator_spgemm_csr_asparse_soa_n_loop( io_generated_code,
@@ -124,7 +126,9 @@ void libxsmm_generator_spgemm_csr_asparse_soa_n_loop( libxsmm_generated_code*
   if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype )  ) {
     if ( strcmp(i_arch, "knl") == 0 ||
          strcmp(i_arch, "knm") == 0 ||
-         strcmp(i_arch, "skx") == 0 ) {
+         strcmp(i_arch, "skx") == 0 ||
+         strcmp(i_arch, "clx") == 0 ||
+         strcmp(i_arch, "cpx") == 0 ) {
       l_soa_width = 8;
       l_n_max_block = 28;
     } else {
@@ -135,7 +139,9 @@ void libxsmm_generator_spgemm_csr_asparse_soa_n_loop( libxsmm_generated_code*
   } else {
     if ( strcmp(i_arch, "knl") == 0 ||
          strcmp(i_arch, "knm") == 0 ||
-         strcmp(i_arch, "skx") == 0 ) {
+         strcmp(i_arch, "skx") == 0 ||
+         strcmp(i_arch, "clx") == 0 ||
+         strcmp(i_arch, "cpx") == 0 ) {
       l_soa_width = 16;
       l_n_max_block = 28;
     } else {
@@ -164,7 +170,7 @@ void libxsmm_generator_spgemm_csr_asparse_soa_n_loop( libxsmm_generated_code*
   l_n_loop = ( l_n_remain == 0 ) ? (l_n_chunks * l_n_chunksize) : ((l_n_chunks-1) * l_n_chunksize);
 
   /* loop over blocks of n */
-  libxsmm_x86_instruction_register_jump_label( io_generated_code, &l_loop_label_tracker );
+  libxsmm_x86_instruction_register_jump_back_label( io_generated_code, &l_loop_label_tracker );
   libxsmm_x86_instruction_alu_imm( io_generated_code, l_micro_kernel_config.alu_add_instruction, l_gp_reg_mapping.gp_reg_nloop, l_n_chunksize );
 
   /* do matix multiplicatoin for a block of N columns */
@@ -235,7 +241,7 @@ void libxsmm_generator_spgemm_csr_asparse_soa_m_loop( libxsmm_generated_code*
     /* generate M loop */
     if (i_a_is_dense != 0 ) {
       libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_mov_instruction, i_gp_reg_mapping->gp_reg_mloop, 0 );
-      libxsmm_x86_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
+      libxsmm_x86_instruction_register_jump_back_label( io_generated_code, io_loop_label_tracker );
       libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_mloop, 1 );
     }
 
@@ -243,7 +249,7 @@ void libxsmm_generator_spgemm_csr_asparse_soa_m_loop( libxsmm_generated_code*
     if (l_row_elements > 0) {
       /* load C accumulator */
       for ( l_n = 0; l_n < i_num_c_cols; l_n++ ) {
-        if ( i_xgemm_desc->beta == 0 ) {
+        if (0 != (LIBXSMM_GEMM_FLAG_BETA_0 & i_xgemm_desc->flags)) { /* Beta=0 */
           libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
                                                    i_micro_kernel_config->instruction_set,
                                                    i_micro_kernel_config->vxor_instruction,
@@ -257,7 +263,7 @@ void libxsmm_generator_spgemm_csr_asparse_soa_m_loop( libxsmm_generated_code*
                                             LIBXSMM_X86_GP_REG_UNDEF, 0,
                                             l_n*i_soa_width*i_micro_kernel_config->datatype_size,
                                             i_micro_kernel_config->vector_name,
-                                            l_n, 0, 0 );
+                                            l_n, 0, 1, 0 );
         }
         if ( (i_xgemm_desc->prefetch & LIBXSMM_GEMM_PREFETCH_BL2_VIA_C) > 0 ) {
           libxsmm_x86_instruction_prefetch( io_generated_code,
@@ -277,7 +283,7 @@ void libxsmm_generator_spgemm_csr_asparse_soa_m_loop( libxsmm_generated_code*
                                           LIBXSMM_X86_GP_REG_UNDEF, 0,
                                           (i_row_idx[l_m] + l_z) * i_micro_kernel_config->datatype_size,
                                           i_micro_kernel_config->vector_name,
-                                          i_num_c_cols, 0, 0 );
+                                          i_num_c_cols, 0, 1, 0 );
         /* multiply with B */
         for ( l_n = 0; l_n < i_num_c_cols; l_n++ ) {
           l_b_offset = ((i_column_idx[i_row_idx[l_m] + l_z]*i_micro_kernel_config->datatype_size*i_soa_width*i_xgemm_desc->ldb)
@@ -286,6 +292,8 @@ void libxsmm_generator_spgemm_csr_asparse_soa_m_loop( libxsmm_generated_code*
           if ( strcmp(i_arch, "knl") == 0 ||
                strcmp(i_arch, "knm") == 0 ||
                strcmp(i_arch, "skx") == 0 ||
+               strcmp(i_arch, "clx") == 0 ||
+               strcmp(i_arch, "cpx") == 0 ||
                strcmp(i_arch, "hsw") == 0 ) {
             if (l_b_offset >= 8192) {
               l_b_total_offset += l_b_offset;
@@ -337,7 +345,7 @@ void libxsmm_generator_spgemm_csr_asparse_soa_m_loop( libxsmm_generated_code*
                                           LIBXSMM_X86_GP_REG_UNDEF, 0,
                                           l_n*i_soa_width*i_micro_kernel_config->datatype_size,
                                           i_micro_kernel_config->vector_name,
-                                          l_n, 0, 1 );
+                                          l_n, 0, 0, 1 );
       }
     }
     /* advance C pointer */
