@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -54,6 +54,8 @@ void libxsmm_generator_spgemm_csr_bsparse_soa( libxsmm_generated_code*         i
   if ( strcmp(i_arch, "knl") == 0 ||
        strcmp(i_arch, "knm") == 0 ||
        strcmp(i_arch, "skx") == 0 ||
+       strcmp(i_arch, "clx") == 0 ||
+       strcmp(i_arch, "cpx") == 0 ||
        strcmp(i_arch, "hsw") == 0 ||
        strcmp(i_arch, "snb") == 0 ) {
     libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( io_generated_code,
@@ -99,7 +101,9 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
   if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype )  ) {
     if ( strcmp(i_arch, "knl") == 0 ||
          strcmp(i_arch, "knm") == 0 ||
-         strcmp(i_arch, "skx") == 0 ) {
+         strcmp(i_arch, "skx") == 0 ||
+         strcmp(i_arch, "clx") == 0 ||
+         strcmp(i_arch, "cpx") == 0 ) {
       l_soa_width = 8;
       l_max_reg_block = 28;
     } else {
@@ -109,7 +113,9 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
   } else {
     if ( strcmp(i_arch, "knl") == 0 ||
          strcmp(i_arch, "knm") == 0 ||
-         strcmp(i_arch, "skx") == 0 ) {
+         strcmp(i_arch, "skx") == 0 ||
+         strcmp(i_arch, "clx") == 0 ||
+         strcmp(i_arch, "cpx") == 0 ) {
       l_soa_width = 16;
       l_max_reg_block = 28;
     } else {
@@ -168,7 +174,7 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
   libxsmm_x86_instruction_open_stream( io_generated_code, &l_gp_reg_mapping, i_arch, i_xgemm_desc->prefetch );
 
   /* m loop */
-  libxsmm_x86_instruction_register_jump_label( io_generated_code, &l_loop_label_tracker );
+  libxsmm_x86_instruction_register_jump_back_label( io_generated_code, &l_loop_label_tracker );
   libxsmm_x86_instruction_alu_imm( io_generated_code, l_micro_kernel_config.alu_add_instruction, l_gp_reg_mapping.gp_reg_mloop, 1 );
 
   /* loop over n-blocks */
@@ -180,7 +186,7 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
 #endif
     /* load C accumulator */
     for ( l_n = 0; l_n < l_n_limit - l_n_processed; l_n++ ) {
-      if ( i_xgemm_desc->beta == 0 ) {
+      if (0 != (LIBXSMM_GEMM_FLAG_BETA_0 & i_xgemm_desc->flags)) { /* Beta=0 */
         libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
                                                  l_micro_kernel_config.instruction_set,
                                                  l_micro_kernel_config.vxor_instruction,
@@ -194,7 +200,7 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
                                           LIBXSMM_X86_GP_REG_UNDEF, 0,
                                           (l_n_processed + l_n)*l_soa_width*l_micro_kernel_config.datatype_size,
                                           l_micro_kernel_config.vector_name,
-                                          l_n, 0, 0 );
+                                          l_n, 0, 1, 0 );
       }
     }
 
@@ -220,7 +226,7 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
                                           l_k*l_soa_width*l_micro_kernel_config.datatype_size,
                                           l_micro_kernel_config.vector_name,
                                           /*28+(l_k%4), 0, 0 );*/
-                                          l_max_reg_block, 0, 0 );
+                                          l_max_reg_block, 0, 1, 0 );
       }
       /* loop over element in the row of B and multiply*/
       for ( l_z = 0; l_z < l_row_elements; l_z++ ) {
@@ -230,7 +236,9 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
              (i_column_idx[i_row_idx[l_k] + l_z] < l_n_limit) )                        {
           if ( strcmp(i_arch, "knl") == 0 ||
                strcmp(i_arch, "knm") == 0 ||
-               strcmp(i_arch, "skx") == 0 ) {
+               strcmp(i_arch, "skx") == 0 ||
+               strcmp(i_arch, "clx") == 0 ||
+               strcmp(i_arch, "cpx") == 0 ) {
             libxsmm_x86_instruction_vec_compute_mem( io_generated_code,
                                                        l_micro_kernel_config.instruction_set,
                                                        l_micro_kernel_config.vmul_instruction,
@@ -250,7 +258,7 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
                                               LIBXSMM_X86_GP_REG_UNDEF, 0,
                                               (i_row_idx[l_k] + l_z) * l_micro_kernel_config.datatype_size,
                                               l_micro_kernel_config.vector_name,
-                                              15, 0, 0 );
+                                              15, 0, 1, 0 );
             libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
                                                      l_micro_kernel_config.instruction_set,
                                                      l_micro_kernel_config.vmul_instruction,
@@ -266,7 +274,7 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
                                               LIBXSMM_X86_GP_REG_UNDEF, 0,
                                               (i_row_idx[l_k] + l_z) * l_micro_kernel_config.datatype_size,
                                               l_micro_kernel_config.vector_name,
-                                              15, 0, 0 );
+                                              15, 0, 1, 0 );
             libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
                                                      l_micro_kernel_config.instruction_set,
                                                      l_micro_kernel_config.vmul_instruction,
@@ -295,7 +303,7 @@ void libxsmm_generator_spgemm_csr_bsparse_soa_avx256_512( libxsmm_generated_code
                                         LIBXSMM_X86_GP_REG_UNDEF, 0,
                                         (l_n_processed + l_n)*l_soa_width*l_micro_kernel_config.datatype_size,
                                         l_micro_kernel_config.vector_name,
-                                        l_n, 0, 1 );
+                                        l_n, 0, 0, 1 );
     }
 
     /* adjust n progression */
