@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -67,8 +67,9 @@ void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
   while (fgets(l_line, l_line_length, l_csr_file_handle) != NULL) {
     if ( strlen(l_line) == l_line_length ) {
       free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_row_idx_id);
+      *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+      fclose(l_csr_file_handle); /* close mtx file */
       LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSR_READ_LEN );
-      fclose( l_csr_file_handle ); /* close mtx file */
       return;
     }
     /* check if we are still reading comments header */
@@ -77,10 +78,12 @@ void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
     } else {
       /* if we are the first line after comment header, we allocate our data structures */
       if ( l_header_read == 0 ) {
-        if ( sscanf(l_line, "%u %u %u", o_row_count, o_column_count, o_element_count) == 3 ) {
-          /* allocate CSC datastructure matching mtx file */
+        if (3 == sscanf(l_line, "%u %u %u", o_row_count, o_column_count, o_element_count) &&
+            0 != *o_row_count && 0 != *o_column_count && 0 != *o_element_count)
+        {
+          /* allocate CSC data-structure matching mtx file */
           *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_element_count));
-          *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_row_count + 1));
+          *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * ((size_t)(*o_row_count) + 1));
           *o_values = (double*) malloc(sizeof(double) * (*o_element_count));
           l_row_idx_id = (unsigned int*) malloc(sizeof(unsigned int) * (*o_row_count));
 
@@ -90,19 +93,20 @@ void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
                ( *o_values == NULL )       ||
                ( l_row_idx_id == NULL ) ) {
             free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_row_idx_id);
+            *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+            fclose(l_csr_file_handle); /* close mtx file */
             LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_ALLOC_DATA );
-            fclose( l_csr_file_handle ); /* close mtx file */
             return;
           }
 
           /* set everything to zero for init */
-          memset(*o_row_idx, 0, sizeof(unsigned int)*(*o_row_count + 1));
-          memset(*o_column_idx, 0, sizeof(unsigned int)*(*o_element_count));
-          memset(*o_values, 0, sizeof(double)*(*o_element_count));
-          memset(l_row_idx_id, 0, sizeof(unsigned int)*(*o_row_count));
+          memset(*o_row_idx, 0, sizeof(unsigned int) * ((size_t)(*o_row_count) + 1));
+          memset(*o_column_idx, 0, sizeof(unsigned int) * (*o_element_count));
+          memset(*o_values, 0, sizeof(double) * (*o_element_count));
+          memset(l_row_idx_id, 0, sizeof(unsigned int) * (*o_row_count));
 
           /* init column idx */
-          for ( l_i = 0; l_i < (*o_row_count + 1); l_i++)
+          for ( l_i = 0; l_i <= *o_row_count; ++l_i )
             (*o_row_idx)[l_i] = (*o_element_count);
 
           /* init */
@@ -121,13 +125,14 @@ void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
         /* read a line of content */
         if ( sscanf(l_line, "%u %u %lf", &l_row, &l_column, &l_value) != 3 ) {
           free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_row_idx_id);
+          *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+          fclose(l_csr_file_handle); /* close mtx file */
           LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSR_READ_ELEMS );
-          fclose( l_csr_file_handle ); /* close mtx file */
           return;
         }
         /* adjust numbers to zero termination */
-        l_row--;
-        l_column--;
+        LIBXSMM_ASSERT(0 != l_row && 0 != l_column);
+        l_row--; l_column--;
         /* add these values to row and value structure */
         (*o_column_idx)[l_i] = l_column;
         (*o_values)[l_i] = l_value;
@@ -145,6 +150,7 @@ void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
   /* check if we read a file which was consistent */
   if ( l_i != (*o_element_count) ) {
     free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_row_idx_id);
+    *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
     LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSR_LEN );
     return;
   }
@@ -162,4 +168,3 @@ void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
   }
 }
 
-
