@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2018, Intel Corporation                                **
+** Copyright (c) 2015-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -90,6 +90,7 @@ void libxsmm_x86_instruction_vec_mask_move( libxsmm_generated_code* io_generated
  * @param i_vector_name the vector register name prefix (x, y or z)
  * @param i_vec_reg_number_0 the vector register number (xmm/ymm: 0-15, zmm: 0-31)
  * @param i_mask_reg_number the mask register to be used
+ * @param i_use_zero_masking: 0: merge masking ; !=0: zero masking
  * @param i_is_store 0: load semantic, other: store semantic
  */
 LIBXSMM_API_INTERN
@@ -102,7 +103,8 @@ void libxsmm_x86_instruction_vec_move( libxsmm_generated_code* io_generated_code
                                        const int               i_displacement,
                                        const char              i_vector_name,
                                        const unsigned int      i_vec_reg_number_0,
-                                       const unsigned int      i_use_masking,
+                                       const unsigned int      i_mask_reg_number,
+                                       const unsigned int      i_use_zero_masking,
                                        const unsigned int      i_is_store );
 
 /**
@@ -133,8 +135,9 @@ void libxsmm_x86_instruction_vec_compute_reg( libxsmm_generated_code* io_generat
  * @param i_instruction_set requested instruction set to encode
  * @param i_vec_instr actual operation variant
  * @param i_vector_name the vector register name prefix (x,y or z)
- * @param i_vec_reg_src the first/source vector register number (xmm/ymm: 0-15, zmm: 0-31)
- * @param i_vec_reg_dst the second/destination vector register number (xmm/ymm: 0-15, zmm: 0-31)
+ * @param i_vec_reg_src_0 the first source vector register number (xmm/ymm: 0-15, zmm: 0-31)
+ * @param i_vec_reg_src_1 the second source vector register number (xmm/ymm: 0-15, zmm: 0-31)
+ * @param i_vec_reg_dst the destination vector register number (xmm/ymm: 0-15, zmm: 0-31)
  * @param i_shuffle_operand is an immediate (only looked at when needed)
  */
 LIBXSMM_API_INTERN
@@ -142,24 +145,24 @@ void libxsmm_x86_instruction_vec_compute_convert ( libxsmm_generated_code* io_ge
                                               const unsigned int      i_instruction_set,
                                               const unsigned int      i_vec_instr,
                                               const char              i_vector_name,
-                                              const unsigned int      i_vec_reg_src,
+                                              const unsigned int      i_vec_reg_src_0,
+                                              const unsigned int      i_vec_reg_src_1,
                                               const unsigned int      i_vec_reg_dst,
                                               const unsigned int      i_shuffle_operand );
 
 /**
- * Generates (v)XYZpd/(v)XYZps/(v)XYZsd/(v)XYZss instructions with 2 vector registers, memory operand as first operand
+ * Generates (v)XYZpd/(v)XYZps/(v)XYZsd/(v)XYZss instructions with 3 vector registers and masking
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param i_instruction_set requested instruction set to encode
  * @param i_vec_instr actual operation variant
- * @param i_use_broadcast if != 0 memory operand is interpreted as a scalar and broadcasted in fused fashion, only supported on AVX512 and IMCI
- * @param i_gp_reg_base base address register for memory broadcast
- * @param i_gp_reg_idx index register for memory broadcast, can be LIBXSMM_X86_GP_REG_UNDEF -> then regular displacement version is generated
- * @param i_scale scale of index register, ignored if i_gp_reg_idx is LIBXSMM_X86_GP_REG_UNDEF
- * @param i_displacement displacement to SIB address
  * @param i_vector_name the vector register name prefix (z)
  * @param i_vec_reg_number_0 the first vector register number (zmm: 0-31)
  * @param i_vec_reg_number_1 the second vector register number (zmm: 0-31)
+ * @param i_vec_reg_number_3 the second vector register number (zmm: 0-31)
+ * @param i_immediate immediate just as the compare value for a compare instruction
+ * @param i_mask_reg_number the mask register to read/write
+ * @param i_use_zero_masking 0: merge masking, !=0 zero masking
  */
 LIBXSMM_API_INTERN
 void libxsmm_x86_instruction_vec_compute_reg_mask( libxsmm_generated_code* io_generated_code,
@@ -169,7 +172,9 @@ void libxsmm_x86_instruction_vec_compute_reg_mask( libxsmm_generated_code* io_ge
                                               const unsigned int      i_vec_reg_number_0,
                                               const unsigned int      i_vec_reg_number_1,
                                               const unsigned int      i_vec_reg_number_2,
-                                              const unsigned int      i_mask_reg_number );
+                                              const unsigned int      i_immediate,
+                                              const unsigned int      i_mask_reg_number,
+                                              const unsigned int      i_use_zero_masking );
 
 /**
  * @param i_instruction_set requested instruction set to encode
@@ -193,34 +198,39 @@ void libxsmm_x86_instruction_vec_compute_mem( libxsmm_generated_code* io_generat
                                               const unsigned int      i_vec_reg_number_0,
                                               const unsigned int      i_vec_reg_number_1 );
 
- /**
-  * Generates quadmadd instructions added in Knights Mill
-  *
-  * @param io_generated_code pointer to the pointer of the generated code structure
-  * @param i_instruction_set requested instruction set to encode
-  * @param i_vec_instr actual operation variant
-  * @param i_gp_reg_base base address register for memory broadcast
-  * @param i_gp_reg_idx index register for memory broadcast, can be LIBXSMM_X86_GP_REG_UNDEF -> then regular displacement version is generated
-  * @param i_scale scale of index register, ignored if i_gp_reg_idx is LIBXSMM_X86_GP_REG_UNDEF
-  * @param i_displacement displacement to SIB address
-  * @param i_vector_name the vector register name prefix (z)
-  * @param i_vec_reg_number_src the second vector register number (zmm: 0-31), this define a implicit regsiter range
-  * @param i_vec_reg_number_dest the first vector register number (zmm: 0-31)
-  */
+/**
+ * Generates vector instructions which require an immediate and mask. immediate is optional.
+ *
+ * @param io_generated_code pointer to the pointer of the generated code structure
+ * @param i_instruction_set requested instruction set to encode
+ * @param i_vec_instr actual operation variant
+ * @param i_use_broadcast if != 0 memory operand is interpreted as a scalar and broadcasted in fused fashion, only supported on AVX512
+ * @param i_gp_reg_base base address register for memory broadcast
+ * @param i_gp_reg_idx index register for memory broadcast, can be LIBXSMM_X86_GP_REG_UNDEF -> then regular displacement version is generated
+ * @param i_scale scale of index register, ignored if i_gp_reg_idx is LIBXSMM_X86_GP_REG_UNDEF
+ * @param i_displacement displacement to SIB address
+ * @param i_vector_name the vector register name prefix (z)
+ * @param i_vec_reg_number_0 the first vector register number (zmm: 0-31)
+ * @param i_vec_reg_number_1 the second vector register number (zmm: 0-31)
+ * @param i_immediate immediate just as the compare value for a compare instruction
+ * @param i_mask_reg_number the mask register to read/write
+ * @param i_use_zero_masking 0: merge masking; !=0: zero masking
+ */
 LIBXSMM_API_INTERN
 void libxsmm_x86_instruction_vec_compute_mem_mask( libxsmm_generated_code* io_generated_code,
-                                              const unsigned int      i_instruction_set,
-                                              const unsigned int      i_vec_instr,
-                                              const unsigned int      i_use_broadcast,
-                                              const unsigned int      i_gp_reg_base,
-                                              const unsigned int      i_gp_reg_idx,
-                                              const unsigned int      i_scale,
-                                              const int               i_displacement,
-                                              const char              i_vector_name,
-                                              const unsigned int      i_vec_reg_number_0,
-                                              const unsigned int      i_vec_reg_number_1,
-                                              const unsigned int      i_shuffle_operand,
-                                              const unsigned int      i_mask_reg_number );
+                                                   const unsigned int      i_instruction_set,
+                                                   const unsigned int      i_vec_instr,
+                                                   const unsigned int      i_use_broadcast,
+                                                   const unsigned int      i_gp_reg_base,
+                                                   const unsigned int      i_gp_reg_idx,
+                                                   const unsigned int      i_scale,
+                                                   const int               i_displacement,
+                                                   const char              i_vector_name,
+                                                   const unsigned int      i_vec_reg_number_0,
+                                                   const unsigned int      i_vec_reg_number_1,
+                                                   const unsigned int      i_immediate,
+                                                   const unsigned int      i_mask_reg_number,
+                                                   const unsigned int      i_use_zero_masking );
 
  /**
   * Generates quadmadd instructions added in Knights Mill
@@ -233,7 +243,7 @@ void libxsmm_x86_instruction_vec_compute_mem_mask( libxsmm_generated_code* io_ge
   * @param i_scale scale of index register, ignored if i_gp_reg_idx is LIBXSMM_X86_GP_REG_UNDEF
   * @param i_displacement displacement to SIB address
   * @param i_vector_name the vector register name prefix (z)
-  * @param i_vec_reg_number_src the second vector register number (zmm: 0-31), this define a implicit regsiter range
+  * @param i_vec_reg_number_src the second vector register number (zmm: 0-31), this define a implicit register range
   * @param i_vec_reg_number_dest the first vector register number (zmm: 0-31)
   */
 LIBXSMM_API_INTERN
@@ -410,7 +420,7 @@ void libxsmm_x86_instruction_mask_move( libxsmm_generated_code* io_generated_cod
  * Allows for mask move instructions in AVX512
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
- * @param i_mask_instr actual mask compute instruction instruction
+ * @param i_mask_instr actual mask compute instruction
  * @param i_mask_reg_number_src_0 the first operand register number (att syntax) (k1=1...k7=7)
  * @param i_mask_reg_number_src_1 the second operand register number (att syntax) (k1=1...k7=7)
  * @param i_mask_reg_number_dest the third operand register number (att syntax) (k1=1...k7=7)
@@ -423,17 +433,17 @@ void libxsmm_x86_instruction_mask_compute_reg( libxsmm_generated_code* io_genera
                                                const unsigned int      i_mask_reg_number_dest  );
 
 /**
- * Generates regular all instructions with immediates
+ * Generates a label to which one can jump back and pushes it on the loop label stack
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param io_loop_label_tracker data structure to handle loop labels, nested loops are supported, but not overlapping loops
 */
 LIBXSMM_API_INTERN
-void libxsmm_x86_instruction_register_jump_label( libxsmm_generated_code*     io_generated_code,
-                                                  libxsmm_loop_label_tracker* io_loop_label_tracker );
+void libxsmm_x86_instruction_register_jump_back_label( libxsmm_generated_code*     io_generated_code,
+                                                       libxsmm_loop_label_tracker* io_loop_label_tracker );
 
 /**
- * Generates regular all instructions with immediates
+ * Pops the latest from the loop label stack and jumps there
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param i_jmp_instr the particular jump instruction used
@@ -444,12 +454,38 @@ void libxsmm_x86_instruction_jump_back_to_label( libxsmm_generated_code*     io_
                                                  const unsigned int          i_jmp_instr,
                                                  libxsmm_loop_label_tracker* io_loop_label_tracker );
 
+/**
+ * Generates a label to which one can jump back and pushes it on the loop label stack
+ *
+ * @param io_generated_code pointer to the pointer of the generated code structure
+ * @parma i_labal_no position in the jump label tracker to set
+ * @param io_jump_forward_label_tracker forward jump tracker structure for tracking the jump addresses/labels
+*/
+LIBXSMM_API_INTERN
+void libxsmm_x86_instruction_register_jump_label( libxsmm_generated_code*     io_generated_code,
+                                                  const unsigned int          i_label_no,
+                                                  libxsmm_jump_label_tracker* io_jump_label_tracker );
+
+/**
+ * Jumps to the address/label stored a specific position
+ *
+ * @param io_generated_code pointer to the pointer of the generated code structure
+ * @param i_jmp_instr the particular jump instruction used
+ * @param i_label_no position in the jump label tracker to jump to
+ * @param io_jump_label_tracker data structures that tracks arbitrary jump labels
+*/
+LIBXSMM_API_INTERN
+void libxsmm_x86_instruction_jump_to_label( libxsmm_generated_code*     io_generated_code,
+                                            const unsigned int          i_jmp_instr,
+                                            const unsigned int          i_label_no,
+                                            libxsmm_jump_label_tracker* io_jump_label_tracker );
+
 /**
  * Generates an insertion of constants into the code stream and loads them into
  * into a vector register
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
- * @param i_data pointer to an array of bytes that should be be loaded, length needs to match registerlength spezified in i_vector_name (x=16, y=32, z=64)
+ * @param i_data pointer to an array of bytes that should be loaded, length needs to match registerlength specified in i_vector_name (x=16, y=32, z=64)
  * @param i_id global identifier of constants to load.
  * @param i_vector_name the vector register name prefix (x,y or z)
  * @param i_vec_reg_number the destination(gather)/source(scatter) vec register (xmm/ymm: 0-15, zmm: 0-31)
