@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Google Inc.                                      **
+** Copyright (c) 2015-2019, Google Inc.                                      **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -171,7 +171,7 @@ LIBXSMM_API_INTERN void libxsmm_perf_init(void)
   header.elf_mach   = 62;  /* EM_X86_64 */
   header.total_size = sizeof(header);
   header.pid        = pid;
-  header.timestamp  = libxsmm_timer_tick_rdtsc();
+  header.timestamp  = libxsmm_timer_tick();
   header.flags      = JITDUMP_FLAGS_ARCH_TIMESTAMP;
 
   res = fwrite(&header, sizeof(header), 1, internal_perf_fp);
@@ -214,7 +214,7 @@ LIBXSMM_API_INTERN void libxsmm_perf_finalize(void)
   memset(&hdr, 0, sizeof(hdr));
   hdr.id = JITDUMP_CODE_CLOSE;
   hdr.total_size = sizeof(hdr);
-  hdr.timestamp = libxsmm_timer_tick_rdtsc();
+  hdr.timestamp = libxsmm_timer_tick();
   res = fwrite(&hdr, sizeof(hdr), 1, internal_perf_fp);
 
   if (res != 1) {
@@ -267,7 +267,7 @@ LIBXSMM_API_INTERN void libxsmm_perf_dump_code(const void* memory, size_t size,
 
     hdr.id = JITDUMP_CODE_LOAD;
     hdr.total_size = sizeof(hdr) + sizeof(rec) + name_len + size;
-    hdr.timestamp = libxsmm_timer_tick_rdtsc();
+    hdr.timestamp = libxsmm_timer_tick();
 
     rec.code_size = size;
     rec.vma = (uintptr_t) memory;
