@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2018, Intel Corporation                                **
+** Copyright (c) 2016-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -17,7 +17,7 @@
 ** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
 ** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
 ** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
+** a PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
 ** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
 ** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
 ** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
@@ -35,11 +35,9 @@
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
-#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#include <math.h>
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
 #endif
@@ -56,15 +54,15 @@
 #endif
 
 
-/* function pointer for the CPUID-dispatched implementation */
-LIBXSMM_APIVAR(void (*internal_spmdm_createSparseSlice_fp32_thread)(const libxsmm_spmdm_handle*, char,
-  const float*, libxsmm_CSR_sparseslice*, int, int, int));
-LIBXSMM_APIVAR(void (*internal_spmdm_createSparseSlice_bfloat16_thread)(const libxsmm_spmdm_handle*, char,
-  const uint16_t*, libxsmm_CSR_sparseslice*, int, int, int));
-LIBXSMM_APIVAR(void (*internal_spmdm_compute_fp32_thread)(const libxsmm_spmdm_handle*, char, char,
-  const float*, libxsmm_CSR_sparseslice*, const float*, char, const float*, float*, int, int, int));
-LIBXSMM_APIVAR(void (*internal_spmdm_compute_bfloat16_thread)(const libxsmm_spmdm_handle*, char, char,
-  const uint16_t*, libxsmm_CSR_sparseslice*, const uint16_t*, char, const uint16_t*, float*, int, int, int));
+/* function pointer for the CPUID-dispatched implementation (separate typedef for legacy Cray C++ needed) */
+typedef void (*internal_spmdm_createSparseSlice_fp32_thread_fn)(const libxsmm_spmdm_handle*, char, const float*, libxsmm_CSR_sparseslice*, int, int, int);
+LIBXSMM_APIVAR(internal_spmdm_createSparseSlice_fp32_thread_fn internal_spmdm_createSparseSlice_fp32_thread);
+typedef void (*internal_spmdm_createSparseSlice_bfloat16_thread_fn)(const libxsmm_spmdm_handle*, char, const libxsmm_bfloat16*, libxsmm_CSR_sparseslice*, int, int, int);
+LIBXSMM_APIVAR(internal_spmdm_createSparseSlice_bfloat16_thread_fn internal_spmdm_createSparseSlice_bfloat16_thread);
+typedef void (*internal_spmdm_compute_fp32_thread_fn)(const libxsmm_spmdm_handle*, char, char, const float*, libxsmm_CSR_sparseslice*, const float*, char, const float*, float*, int, int, int);
+LIBXSMM_APIVAR(internal_spmdm_compute_fp32_thread_fn internal_spmdm_compute_fp32_thread);
+typedef void (*internal_spmdm_compute_bfloat16_thread_fn)(const libxsmm_spmdm_handle*, char, char, const libxsmm_bfloat16*, libxsmm_CSR_sparseslice*, const libxsmm_bfloat16*, char, const libxsmm_bfloat16*, float*, int, int, int);
+LIBXSMM_APIVAR(internal_spmdm_compute_bfloat16_thread_fn internal_spmdm_compute_bfloat16_thread);
 
 #if defined(LIBXSMM_SPMDM_AVX)
 LIBXSMM_APIVAR(__m256i* internal_spmdm_shufmasks_32);
@@ -108,31 +106,35 @@ LIBXSMM_API_INLINE void internal_spmdm_allocate_csr_a(libxsmm_spmdm_handle* hand
   int m_blocks = handle->mb;
   int k_blocks = handle->kb;
 
-  size_t sz_block = ((handle->bm + 1)*sizeof(uint16_t) + (handle->bm)*(handle->bk)*sizeof(uint16_t) + (handle->bm)*(handle->bk)*sizeof(float) + sizeof(libxsmm_CSR_sparseslice));
+  const size_t sz_block = (((size_t)handle->bm + 1) * sizeof(uint16_t)
+    + (size_t)handle->bm * handle->bk * sizeof(uint16_t)
+    + (size_t)handle->bm * handle->bk * sizeof(float)
+    + sizeof(libxsmm_CSR_sparseslice));
   size_t sz_all_blocks = sz_block * handle->mb * handle->kb;
   char* memory_block = 0;
+  void *const pv = &memory_block;
 
   /* use low-level scratch memory allocation since life-time of this buffer is unknown */
-  if (EXIT_SUCCESS == libxsmm_xmalloc((void**)&memory_block, sz_all_blocks, 2097152,
-    LIBXSMM_MALLOC_FLAG_SCRATCH, 0/*extra*/, 0/*extra_size*/))
+  if (EXIT_SUCCESS == libxsmm_xmalloc((void**)pv, sz_all_blocks, 2097152,
+    LIBXSMM_MALLOC_FLAG_SCRATCH | LIBXSMM_MALLOC_FLAG_PRIVATE, 0/*extra*/, 0/*extra_size*/))
   {
     char* memory_head  = memory_block;
     libxsmm_CSR_sparseslice* libxsmm_output_csr_a = (libxsmm_CSR_sparseslice*)(memory_head);
-    memory_head += handle->mb * handle->kb * sizeof(libxsmm_CSR_sparseslice);
-    assert(0 != libxsmm_output_csr_a/*sanity check*/);
+    memory_head += (size_t)handle->mb * handle->kb * sizeof(libxsmm_CSR_sparseslice);
+    LIBXSMM_ASSERT(0 != libxsmm_output_csr_a/*sanity check*/);
 
     for (kb = 0; kb < k_blocks; kb++) {
       for (mb = 0; mb < m_blocks; mb++) {
         int i = kb*m_blocks + mb;
-        libxsmm_output_csr_a[i].rowidx = (uint16_t *)(memory_head);
-        memory_head += (handle->bm + 1)*sizeof(uint16_t);
-        libxsmm_output_csr_a[i].colidx = (uint16_t *)(memory_head);
-        memory_head += (handle->bm)*(handle->bk)*sizeof(uint16_t);
+        libxsmm_output_csr_a[i].rowidx = (uint16_t*)(memory_head);
+        memory_head += ((size_t)handle->bm + 1) * sizeof(uint16_t);
+        libxsmm_output_csr_a[i].colidx = (uint16_t*)(memory_head);
+        memory_head += (size_t)handle->bm * handle->bk * sizeof(uint16_t);
         libxsmm_output_csr_a[i].values = (float*)(memory_head);
-        memory_head += (handle->bm)*(handle->bk)*sizeof(float);
+        memory_head += (size_t)handle->bm * handle->bk * sizeof(float);
       }
     }
-    assert(memory_head == (memory_block + sz_all_blocks));
+    LIBXSMM_ASSERT(memory_head == (memory_block + sz_all_blocks));
     *libxsmm_output_csr = libxsmm_output_csr_a;
   }
   else if (0 != libxsmm_verbosity) { /* library code is expected to be mute */
@@ -145,14 +147,17 @@ LIBXSMM_API_INLINE void internal_spmdm_allocate_csr_a(libxsmm_spmdm_handle* hand
 
 LIBXSMM_API_INLINE void internal_spmdm_allocate_scratch(libxsmm_spmdm_handle* handle, int max_threads)
 {
-  size_t sz_memory_for_scratch_per_thread = ((handle->bm)*(handle->bn)*sizeof(float) + (handle->bk)*(handle->bn)*sizeof(float)), sz_total_memory;
+  void *const pv = &handle->base_ptr_scratch_B_scratch_C;
+  size_t sz_total_memory, sz_memory_for_scratch_per_thread =
+    (size_t)handle->bm * handle->bn * sizeof(float) +
+    (size_t)handle->bk * handle->bn * sizeof(float);
   sz_memory_for_scratch_per_thread = LIBXSMM_UP2(sz_memory_for_scratch_per_thread, 4096);
   sz_total_memory = sz_memory_for_scratch_per_thread * max_threads;
   handle->base_ptr_scratch_B_scratch_C = 0;
 
   /* use low-level scratch memory allocation since life-time of this buffer is unknown */
-  if (EXIT_SUCCESS == libxsmm_xmalloc((void**)&handle->base_ptr_scratch_B_scratch_C, sz_total_memory, 2097152,
-    LIBXSMM_MALLOC_FLAG_SCRATCH, 0/*extra*/, 0/*extra_size*/))
+  if (EXIT_SUCCESS == libxsmm_xmalloc((void**)pv, sz_total_memory, 2097152,
+    LIBXSMM_MALLOC_FLAG_SCRATCH | LIBXSMM_MALLOC_FLAG_PRIVATE, 0/*extra*/, 0/*extra_size*/))
   {
     handle->memory_for_scratch_per_thread = (int)sz_memory_for_scratch_per_thread;
   }
@@ -168,7 +173,7 @@ LIBXSMM_API_INLINE void internal_spmdm_allocate_scratch(libxsmm_spmdm_handle* ha
 LIBXSMM_API_INLINE void internal_spmdm_deallocate_csr_a(libxsmm_spmdm_handle* handle)
 {
   libxsmm_xfree(handle->base_ptr_scratch_A);
-  handle->base_ptr_scratch_A= NULL;
+  handle->base_ptr_scratch_A = NULL;
   libxsmm_xfree(handle->base_ptr_scratch_B_scratch_C);
   handle->base_ptr_scratch_B_scratch_C = NULL;
 }
@@ -195,8 +200,8 @@ LIBXSMM_API int libxsmm_spmdm_get_num_compute_blocks(const libxsmm_spmdm_handle*
 LIBXSMM_API_INLINE
 void internal_spmdm_createSparseSlice_fp32_thread_sw(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  const float* A,
+  char transa,
+  const float* a,
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a,
   int block_id,
   int tid, int nthreads)
@@ -210,8 +215,8 @@ void internal_spmdm_createSparseSlice_fp32_thread_sw(
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
 LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_fp32_thread_avx2(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  const float* A,
+  char transa,
+  const float* a,
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a,
   int block_id,
   int tid, int nthreads)
@@ -221,7 +226,7 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_fp32_thread_avx2(
 # include "template/libxsmm_spmdm_createSparseSlice_fp32_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-  internal_spmdm_createSparseSlice_fp32_thread_sw(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+  internal_spmdm_createSparseSlice_fp32_thread_sw(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
 
@@ -229,8 +234,8 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_fp32_thread_avx2(
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
 LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_fp32_thread_avx512_core(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  const float* A,
+  char transa,
+  const float* a,
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a,
   int block_id,
   int tid, int nthreads)
@@ -240,7 +245,7 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_fp32_thread_avx51
 # include "template/libxsmm_spmdm_createSparseSlice_fp32_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-  internal_spmdm_createSparseSlice_fp32_thread_avx2(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+  internal_spmdm_createSparseSlice_fp32_thread_avx2(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
 
@@ -248,25 +253,21 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_fp32_thread_avx51
 LIBXSMM_API
 void libxsmm_spmdm_createSparseSlice_fp32_thread(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  const float* A,
+  char transa,
+  const float* a,
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a,
   int block_id,
   int tid, int nthreads)
 {
   /* if highest implemented code path is statically present, no need for an indirect call (function pointer) */
 #if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
-  internal_spmdm_createSparseSlice_fp32_thread_avx512_core(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
-#elif (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH) /* eventually no need for an indirect call */
-# if (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
-  internal_spmdm_createSparseSlice_fp32_thread_avx2(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
-# else /* pointer based function call */
-  assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
-  internal_spmdm_createSparseSlice_fp32_thread(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
-# endif
+  internal_spmdm_createSparseSlice_fp32_thread_avx512_core(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
+#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH) && /* eventually no need for an indirect call */ \
+      (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  internal_spmdm_createSparseSlice_fp32_thread_avx2(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
 #else /* pointer based function call */
-  assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
-  internal_spmdm_createSparseSlice_fp32_thread(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+  LIBXSMM_ASSERT(0 != internal_spmdm_createSparseSlice_fp32_thread);
+  internal_spmdm_createSparseSlice_fp32_thread(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
 
@@ -274,8 +275,8 @@ void libxsmm_spmdm_createSparseSlice_fp32_thread(
 LIBXSMM_API_INLINE
 void internal_spmdm_createSparseSlice_bfloat16_thread_sw(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  const uint16_t* A,
+  char transa,
+  const libxsmm_bfloat16* a,
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a,
   int block_id,
   int tid, int nthreads)
@@ -289,8 +290,8 @@ void internal_spmdm_createSparseSlice_bfloat16_thread_sw(
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
 LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_bfloat16_thread_avx2(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  const uint16_t* A,
+  char transa,
+  const libxsmm_bfloat16* a,
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a,
   int block_id,
   int tid, int nthreads)
@@ -300,7 +301,7 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_bfloat16_thread_a
 # include "template/libxsmm_spmdm_createSparseSlice_bfloat16_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-  internal_spmdm_createSparseSlice_bfloat16_thread_sw(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+  internal_spmdm_createSparseSlice_bfloat16_thread_sw(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
 
@@ -308,8 +309,8 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_bfloat16_thread_a
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
 LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  const uint16_t* A,
+  char transa,
+  const libxsmm_bfloat16* a,
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a,
   int block_id,
   int tid, int nthreads)
@@ -319,7 +320,7 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_bfloat16_thread_a
 # include "template/libxsmm_spmdm_createSparseSlice_bfloat16_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-  internal_spmdm_createSparseSlice_bfloat16_thread_avx2(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+  internal_spmdm_createSparseSlice_bfloat16_thread_avx2(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
 
@@ -327,25 +328,21 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_bfloat16_thread_a
 LIBXSMM_API
 void libxsmm_spmdm_createSparseSlice_bfloat16_thread(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  const uint16_t* A,
+  char transa,
+  const libxsmm_bfloat16* a,
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a,
   int block_id,
   int tid, int nthreads)
 {
   /* if highest implemented code path is statically present, no need for an indirect call (function pointer) */
 #if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
-  internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
-#elif (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH) /* eventually no need for an indirect call */
-# if (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
-  internal_spmdm_createSparseSlice_bfloat16_thread_avx2(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
-# else /* pointer based function call */
-  assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
-  internal_spmdm_createSparseSlice_bfloat16_thread(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
-# endif
+  internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
+#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH) && /* eventually no need for an indirect call */ \
+      (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  internal_spmdm_createSparseSlice_bfloat16_thread_avx2(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
 #else /* pointer based function call */
-  assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
-  internal_spmdm_createSparseSlice_bfloat16_thread(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+  LIBXSMM_ASSERT(0 != internal_spmdm_createSparseSlice_fp32_thread);
+  internal_spmdm_createSparseSlice_bfloat16_thread(handle, transa, a, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
 
@@ -353,14 +350,14 @@ void libxsmm_spmdm_createSparseSlice_bfloat16_thread(
 LIBXSMM_API_INLINE
 void internal_spmdm_compute_fp32_thread_sw(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  char transB,
+  char transa,
+  char transb,
   const float* alpha,
-  libxsmm_CSR_sparseslice* A_sparse,
-  const float* B,
-  char transC,
+  libxsmm_CSR_sparseslice* a_sparse,
+  const float* b,
+  char transc,
   const float* beta,
-  float* C,
+  float* c,
   int block_id,
   int tid, int nthreads)
 {
@@ -373,14 +370,14 @@ void internal_spmdm_compute_fp32_thread_sw(
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
 LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_fp32_thread_avx2(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  char transB,
+  char transa,
+  char transb,
   const float* alpha,
-  libxsmm_CSR_sparseslice* A_sparse,
-  const float* B,
-  char transC,
+  libxsmm_CSR_sparseslice* a_sparse,
+  const float* b,
+  char transc,
   const float* beta,
-  float* C,
+  float* c,
   int block_id,
   int tid, int nthreads)
 {
@@ -389,7 +386,7 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_fp32_thread_avx2(
 # include "template/libxsmm_spmdm_compute_fp32_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-  internal_spmdm_compute_fp32_thread_sw(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+  internal_spmdm_compute_fp32_thread_sw(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
 #endif
 }
 
@@ -397,14 +394,14 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_fp32_thread_avx2(
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
 LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_fp32_thread_avx512_core(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  char transB,
+  char transa,
+  char transb,
   const float* alpha,
-  libxsmm_CSR_sparseslice* A_sparse,
-  const float* B,
-  char transC,
+  libxsmm_CSR_sparseslice* a_sparse,
+  const float* b,
+  char transc,
   const float* beta,
-  float* C,
+  float* c,
   int block_id,
   int tid, int nthreads)
 {
@@ -413,7 +410,7 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_fp32_thread_avx512_core(
 # include "template/libxsmm_spmdm_compute_fp32_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-  internal_spmdm_compute_fp32_thread_avx2(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+  internal_spmdm_compute_fp32_thread_avx2(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
 #endif
 }
 
@@ -421,23 +418,26 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_fp32_thread_avx512_core(
 LIBXSMM_API
 void libxsmm_spmdm_compute_fp32_thread(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  char transB,
+  char transa,
+  char transb,
   const float* alpha,
-  libxsmm_CSR_sparseslice* A_sparse,
-  const float* B,
-  char transC,
+  libxsmm_CSR_sparseslice* a_sparse,
+  const float* b,
+  char transc,
   const float* beta,
-  float* C,
+  float* c,
   int block_id,
   int tid, int nthreads)
 {
   /* if highest implemented code path is statically present, no need for an indirect call (function pointer) */
 #if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
-  internal_spmdm_compute_fp32_thread_avx512_core(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+  internal_spmdm_compute_fp32_thread_avx512_core(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
+#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH) && /* eventually no need for an indirect call */ \
+      (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  internal_spmdm_compute_fp32_thread_avx2(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
 #else /* pointer based function call */
-  assert(0 != internal_spmdm_compute_fp32_thread);
-  internal_spmdm_compute_fp32_thread(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+  LIBXSMM_ASSERT(0 != internal_spmdm_compute_fp32_thread);
+  internal_spmdm_compute_fp32_thread(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
 #endif
 }
 
@@ -445,14 +445,14 @@ void libxsmm_spmdm_compute_fp32_thread(
 LIBXSMM_API_INLINE
 void internal_spmdm_compute_bfloat16_thread_sw(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  char transB,
-  const uint16_t* alpha,
-  libxsmm_CSR_sparseslice* A_sparse,
-  const uint16_t* B,
-  char transC,
-  const uint16_t* beta,
-  float* C,
+  char transa,
+  char transb,
+  const libxsmm_bfloat16* alpha,
+  libxsmm_CSR_sparseslice* a_sparse,
+  const libxsmm_bfloat16* b,
+  char transc,
+  const libxsmm_bfloat16* beta,
+  float* c,
   int block_id,
   int tid, int nthreads)
 {
@@ -465,14 +465,14 @@ void internal_spmdm_compute_bfloat16_thread_sw(
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
 LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_bfloat16_thread_avx2(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  char transB,
-  const uint16_t* alpha,
-  libxsmm_CSR_sparseslice* A_sparse,
-  const uint16_t* B,
-  char transC,
-  const uint16_t* beta,
-  float* C,
+  char transa,
+  char transb,
+  const libxsmm_bfloat16* alpha,
+  libxsmm_CSR_sparseslice* a_sparse,
+  const libxsmm_bfloat16* b,
+  char transc,
+  const libxsmm_bfloat16* beta,
+  float* c,
   int block_id,
   int tid, int nthreads)
 {
@@ -481,7 +481,7 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_bfloat16_thread_avx2(
 # include "template/libxsmm_spmdm_compute_bfloat16_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-  internal_spmdm_compute_bfloat16_thread_sw(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+  internal_spmdm_compute_bfloat16_thread_sw(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
 #endif
 }
 
@@ -489,14 +489,14 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_bfloat16_thread_avx2(
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
 LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_bfloat16_thread_avx512_core(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  char transB,
-  const uint16_t* alpha,
-  libxsmm_CSR_sparseslice* A_sparse,
-  const uint16_t* B,
-  char transC,
-  const uint16_t* beta,
-  float* C,
+  char transa,
+  char transb,
+  const libxsmm_bfloat16* alpha,
+  libxsmm_CSR_sparseslice* a_sparse,
+  const libxsmm_bfloat16* b,
+  char transc,
+  const libxsmm_bfloat16* beta,
+  float* c,
   int block_id,
   int tid, int nthreads)
 {
@@ -505,7 +505,7 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_bfloat16_thread_avx512_core
 # include "template/libxsmm_spmdm_compute_bfloat16_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-  internal_spmdm_compute_bfloat16_thread_avx2(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+  internal_spmdm_compute_bfloat16_thread_avx2(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
 #endif
 }
 
@@ -513,45 +513,30 @@ LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_bfloat16_thread_avx512_core
 LIBXSMM_API
 void libxsmm_spmdm_compute_bfloat16_thread(
   const libxsmm_spmdm_handle* handle,
-  char transA,
-  char transB,
-  const uint16_t* alpha,
-  libxsmm_CSR_sparseslice* A_sparse,
-  const uint16_t* B,
-  char transC,
-  const uint16_t* beta,
-  float* C,
+  char transa,
+  char transb,
+  const libxsmm_bfloat16* alpha,
+  libxsmm_CSR_sparseslice* a_sparse,
+  const libxsmm_bfloat16* b,
+  char transc,
+  const libxsmm_bfloat16* beta,
+  float* c,
   int block_id,
   int tid, int nthreads)
 {
   /* if highest implemented code path is statically present, no need for an indirect call (function pointer) */
 #if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
-  internal_spmdm_compute_bfloat16_thread_avx512_core(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
-#elif (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH) /* eventually no need for an indirect call */
-# if (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
-  internal_spmdm_compute_bfloat16_thread_avx2(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
-# else /* pointer based function call */
-  assert(0 != internal_spmdm_compute_bfloat16_thread);
-  internal_spmdm_compute_bfloat16_thread(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
-# endif
+  internal_spmdm_compute_bfloat16_thread_avx512_core(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
+#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH) && /* eventually no need for an indirect call */ \
+      (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  internal_spmdm_compute_bfloat16_thread_avx2(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
 #else /* pointer based function call */
-  assert(0 != internal_spmdm_compute_bfloat16_thread);
-  internal_spmdm_compute_bfloat16_thread(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+  LIBXSMM_ASSERT(0 != internal_spmdm_compute_bfloat16_thread);
+  internal_spmdm_compute_bfloat16_thread(handle, transa, transb, alpha, a_sparse, b, transc, beta, c, block_id, tid, nthreads);
 #endif
 }
 
 
-LIBXSMM_API_INLINE void internal_spmdm_init_check(int archid)
-{
-  if (archid < libxsmm_target_archid && 0 != libxsmm_verbosity) { /* library code is expected to be mute */
-    static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM ERROR: missed to enter \"%s\" code path due to the compiler used!\n", libxsmm_get_target_arch());
-    }
-  }
-}
-
-
 LIBXSMM_API void libxsmm_spmdm_init(int M, int N, int K, int max_threads,
   libxsmm_spmdm_handle* handle, libxsmm_CSR_sparseslice** libxsmm_output_csr)
 {
@@ -562,8 +547,9 @@ LIBXSMM_API void libxsmm_spmdm_init(int M, int N, int K, int max_threads,
   double avg_blocks_per_thread;
   double load_imbalance_1, load_imbalance_2, load_imbalance;
 
-  /* initialize internal library structures */
-  LIBXSMM_INIT
+  libxsmm_init(); /* !LIBXSMM_INIT */
+  LIBXSMM_ATOMIC_ADD_FETCH(&libxsmm_statistic_num_spmdm, 1,
+    LIBXSMM_ATOMIC_RELAXED); /* count number of invocations */
 
   handle->m  = M;
   handle->n  = N;
@@ -572,7 +558,6 @@ LIBXSMM_API void libxsmm_spmdm_init(int M, int N, int K, int max_threads,
 
 #if defined(LIBXSMM_SPMDM_AVX512_CORE)
   if (LIBXSMM_X86_AVX512_CORE <= libxsmm_target_archid || LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH) {
-    internal_spmdm_init_check(LIBXSMM_X86_AVX512_CORE);
     internal_spmdm_createSparseSlice_fp32_thread = internal_spmdm_createSparseSlice_fp32_thread_avx512_core;
     internal_spmdm_createSparseSlice_bfloat16_thread = internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core;
     internal_spmdm_compute_fp32_thread = internal_spmdm_compute_fp32_thread_avx512_core;
@@ -583,7 +568,6 @@ LIBXSMM_API void libxsmm_spmdm_init(int M, int N, int K, int max_threads,
 #endif
 #if defined(LIBXSMM_SPMDM_AVX2)
   if (LIBXSMM_X86_AVX2 <= libxsmm_target_archid || LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH) {
-    internal_spmdm_init_check(LIBXSMM_X86_AVX512_MIC);
     internal_spmdm_createSparseSlice_fp32_thread = internal_spmdm_createSparseSlice_fp32_thread_avx2;
     internal_spmdm_createSparseSlice_bfloat16_thread = internal_spmdm_createSparseSlice_bfloat16_thread_avx2;
     internal_spmdm_compute_fp32_thread = internal_spmdm_compute_fp32_thread_avx2;
@@ -593,7 +577,6 @@ LIBXSMM_API void libxsmm_spmdm_init(int M, int N, int K, int max_threads,
   else
 #endif
   {
-    internal_spmdm_init_check(LIBXSMM_X86_AVX);
     internal_spmdm_createSparseSlice_fp32_thread = internal_spmdm_createSparseSlice_fp32_thread_sw;
     internal_spmdm_createSparseSlice_bfloat16_thread = internal_spmdm_createSparseSlice_bfloat16_thread_sw;
     internal_spmdm_compute_fp32_thread = internal_spmdm_compute_fp32_thread_sw;
@@ -606,7 +589,7 @@ LIBXSMM_API void libxsmm_spmdm_init(int M, int N, int K, int max_threads,
   handle->kb = (handle->k + handle->bk - 1) / handle->bk;
 
   max_work_per_block    = handle->bm * handle->bn;
-  avg_work_per_block    = (double)(handle->m * handle->n) / (handle->mb * handle->nb);
+  avg_work_per_block    = (double)((size_t)handle->m * handle->n) / ((size_t)handle->mb * handle->nb);
   load_imbalance_1      = max_work_per_block / avg_work_per_block;
   max_blocks_per_thread = (handle->mb * handle->nb + max_threads - 1) / max_threads;
   avg_blocks_per_thread = (double)handle->mb * handle->nb / max_threads;
@@ -621,28 +604,27 @@ LIBXSMM_API void libxsmm_spmdm_init(int M, int N, int K, int max_threads,
     avg_blocks_per_thread = (double)handle->mb * handle->nb / max_threads;
     load_imbalance_2      = max_blocks_per_thread / avg_blocks_per_thread;
     max_work_per_block    = handle->bm * handle->bn;
-    avg_work_per_block    = (double)(handle->m * handle->n) / (handle->mb * handle->nb);
+    avg_work_per_block    = (double)((size_t)handle->m * handle->n) / ((size_t)handle->mb * handle->nb);
     load_imbalance_1      = max_work_per_block / avg_work_per_block;
     load_imbalance        = load_imbalance_1 * load_imbalance_2;
   }
 
-  /* This is temporary space needed; allocate for each different size of A */
+  /* This is temporary space needed; allocate for each different size of a */
   internal_spmdm_allocate_csr_a(handle, libxsmm_output_csr);
   internal_spmdm_allocate_scratch(handle, max_threads);
 
   /* Initialize shuffle masks for the computation */
 #if defined(LIBXSMM_SPMDM_AVX)
-  if (LIBXSMM_X86_AVX <= libxsmm_target_archid) {
+  if (LIBXSMM_X86_AVX <= libxsmm_target_archid || LIBXSMM_X86_AVX <= LIBXSMM_STATIC_TARGET_ARCH) {
     internal_spmdm_init_shufmask_avx();
+    LIBXSMM_ASSERT(0 != internal_spmdm_shufmasks_32);
+    LIBXSMM_ASSERT(0 != internal_spmdm_shufmasks_16);
   }
-  assert(0 != internal_spmdm_shufmasks_32);
-  assert(0 != internal_spmdm_shufmasks_16);
 #endif
-
   /* post-conditions */
-  assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
-  assert(0 != internal_spmdm_createSparseSlice_bfloat16_thread);
-  assert(0 != internal_spmdm_compute_fp32_thread);
-  assert(0 != internal_spmdm_compute_bfloat16_thread);
+  LIBXSMM_ASSERT(0 != internal_spmdm_createSparseSlice_fp32_thread);
+  LIBXSMM_ASSERT(0 != internal_spmdm_createSparseSlice_bfloat16_thread);
+  LIBXSMM_ASSERT(0 != internal_spmdm_compute_fp32_thread);
+  LIBXSMM_ASSERT(0 != internal_spmdm_compute_bfloat16_thread);
 }
 
