@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2018, Intel Corporation                                **
+** Copyright (c) 2016-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -60,13 +60,13 @@
 #define _MM_PREFETCH(x, y) _mm_prefetch(x, y)
 #define TRANSPOSE_SIMD_WIDTH_KERNEL(ptr_A, ldA, ptr_B, ldB) { \
   __m256 ymm9  = _mm256_loadu_ps(ptr_A); \
-  __m256 ymm10 = _mm256_loadu_ps(ptr_A + ldA); \
-  __m256 ymm11 = _mm256_loadu_ps(ptr_A + 2*ldA); \
-  __m256 ymm12 = _mm256_loadu_ps(ptr_A + 3*ldA); \
-  __m256 ymm13 = _mm256_loadu_ps(ptr_A + 4*ldA); \
-  __m256 ymm14 = _mm256_loadu_ps(ptr_A + 5*ldA); \
-  __m256 ymm15 = _mm256_loadu_ps(ptr_A + 6*ldA); \
-  __m256 ymm2  = _mm256_loadu_ps(ptr_A + 7*ldA); \
+  __m256 ymm10 = _mm256_loadu_ps(ptr_A + (size_t)ldA); \
+  __m256 ymm11 = _mm256_loadu_ps(ptr_A + (size_t)ldA*2); \
+  __m256 ymm12 = _mm256_loadu_ps(ptr_A + (size_t)ldA*3); \
+  __m256 ymm13 = _mm256_loadu_ps(ptr_A + (size_t)ldA*4); \
+  __m256 ymm14 = _mm256_loadu_ps(ptr_A + (size_t)ldA*5); \
+  __m256 ymm15 = _mm256_loadu_ps(ptr_A + (size_t)ldA*6); \
+  __m256 ymm2  = _mm256_loadu_ps(ptr_A + (size_t)ldA*7); \
   __m256 ymm6  = _mm256_unpacklo_ps(ymm9, ymm10); \
   __m256 ymm1  = _mm256_unpacklo_ps(ymm11, ymm12); \
   __m256 ymm8  = _mm256_unpackhi_ps(ymm9, ymm10); \
@@ -84,40 +84,40 @@
          ymm13 = _mm256_blend_ps(ymm6, ymm9, 0xCC);{ \
   __m256 ymm4  = _mm256_unpackhi_ps(ymm15, ymm2); \
          ymm2  = _mm256_permute2f128_ps(ymm12, ymm13, 0x20); \
-         _mm256_storeu_ps(ptr_B + ldB, ymm2); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB, ymm2); \
          ymm14 = _mm256_shuffle_ps(ymm8, ymm7, 0x4E); \
          ymm15 = _mm256_blend_ps(ymm14, ymm7, 0xCC); \
          ymm7  = _mm256_shuffle_ps(ymm5, ymm4, 0x4E); \
          ymm8  = _mm256_blend_ps(ymm8, ymm14, 0xCC); \
          ymm5  = _mm256_blend_ps(ymm5, ymm7, 0xCC); \
          ymm6  = _mm256_permute2f128_ps(ymm8, ymm5, 0x20); \
-         _mm256_storeu_ps(ptr_B + 2*ldB, ymm6); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*2, ymm6); \
          ymm4  = _mm256_blend_ps(ymm7, ymm4, 0xCC); \
          ymm7  = _mm256_permute2f128_ps(ymm15, ymm4, 0x20); \
-         _mm256_storeu_ps(ptr_B + 3*ldB, ymm7); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*3, ymm7); \
          ymm1  = _mm256_permute2f128_ps(ymm10, ymm11, 0x31); \
          ymm0  = _mm256_permute2f128_ps(ymm12, ymm13, 0x31); \
-         _mm256_storeu_ps(ptr_B + 4*ldB, ymm1); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*4, ymm1); \
          ymm5  = _mm256_permute2f128_ps(ymm8, ymm5, 0x31); \
          ymm4  = _mm256_permute2f128_ps(ymm15, ymm4, 0x31); \
-         _mm256_storeu_ps(ptr_B + 5*ldB, ymm0); \
-         _mm256_storeu_ps(ptr_B + 6*ldB, ymm5); \
-         _mm256_storeu_ps(ptr_B + 7*ldB, ymm4);}}}} \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*5, ymm0); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*6, ymm5); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*7, ymm4);}}}} \
 }
 
 #define TRANSPOSE_SIMD_WIDTH_KERNEL_BFLOAT16(ptr_A, ldA, ptr_B, ldB) { \
   __m256 ymm9, ymm10, ymm11, ymm12, ymm13, ymm14, ymm15, ymm2; \
   __m256i vload_1 =  _mm256_castsi128_si256(_mm_loadu_si128((const __m128i*)(ptr_A))); \
-  vload_1 =  _mm256_inserti128_si256(vload_1, _mm_loadu_si128((const __m128i*)(ptr_A + ldA)), 1); \
+  vload_1 =  _mm256_inserti128_si256(vload_1, _mm_loadu_si128((const __m128i*)(ptr_A + (size_t)ldA)), 1); \
   EXPAND_BFLOAT16(vload_1, ymm9, ymm10);{ \
-  __m256i vload_2 =  _mm256_castsi128_si256(_mm_loadu_si128((const __m128i*)(ptr_A + 2*ldA))); \
-  vload_2 =  _mm256_inserti128_si256(vload_2, _mm_loadu_si128((const __m128i*)(ptr_A + 3*ldA)), 1); \
+  __m256i vload_2 =  _mm256_castsi128_si256(_mm_loadu_si128((const __m128i*)(ptr_A + (size_t)ldA*2))); \
+  vload_2 =  _mm256_inserti128_si256(vload_2, _mm_loadu_si128((const __m128i*)(ptr_A + (size_t)ldA*3)), 1); \
   EXPAND_BFLOAT16(vload_2, ymm11, ymm12);{ \
-  __m256i vload_3 =  _mm256_castsi128_si256(_mm_loadu_si128((const __m128i*)(ptr_A + 4*ldA))); \
-  vload_3 =  _mm256_inserti128_si256(vload_3, _mm_loadu_si128((const __m128i*)(ptr_A + 5*ldA)), 1); \
+  __m256i vload_3 =  _mm256_castsi128_si256(_mm_loadu_si128((const __m128i*)(ptr_A + (size_t)ldA*4))); \
+  vload_3 =  _mm256_inserti128_si256(vload_3, _mm_loadu_si128((const __m128i*)(ptr_A + (size_t)ldA*5)), 1); \
   EXPAND_BFLOAT16(vload_3, ymm13, ymm14);{ \
-  __m256i vload_4 =  _mm256_castsi128_si256(_mm_loadu_si128((const __m128i*)(ptr_A + 6*ldA))); \
-  vload_4 =  _mm256_inserti128_si256(vload_4, _mm_loadu_si128((const __m128i*)(ptr_A + 7*ldA)), 1); \
+  __m256i vload_4 =  _mm256_castsi128_si256(_mm_loadu_si128((const __m128i*)(ptr_A + (size_t)ldA*6))); \
+  vload_4 =  _mm256_inserti128_si256(vload_4, _mm_loadu_si128((const __m128i*)(ptr_A + (size_t)ldA*7)), 1); \
   EXPAND_BFLOAT16(vload_4, ymm15, ymm2);{ \
   __m256 ymm6  = _mm256_unpacklo_ps(ymm9, ymm10); \
   __m256 ymm1  = _mm256_unpacklo_ps(ymm11, ymm12); \
@@ -136,25 +136,25 @@
          ymm13 = _mm256_blend_ps(ymm6, ymm9, 0xCC);{ \
   __m256 ymm4  = _mm256_unpackhi_ps(ymm15, ymm2); \
          ymm2  = _mm256_permute2f128_ps(ymm12, ymm13, 0x20); \
-         _mm256_storeu_ps(ptr_B + ldB, ymm2); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB, ymm2); \
          ymm14 = _mm256_shuffle_ps(ymm8, ymm7, 0x4E); \
          ymm15 = _mm256_blend_ps(ymm14, ymm7, 0xCC); \
          ymm7  = _mm256_shuffle_ps(ymm5, ymm4, 0x4E); \
          ymm8  = _mm256_blend_ps(ymm8, ymm14, 0xCC); \
          ymm5  = _mm256_blend_ps(ymm5, ymm7, 0xCC); \
          ymm6  = _mm256_permute2f128_ps(ymm8, ymm5, 0x20); \
-         _mm256_storeu_ps(ptr_B + 2*ldB, ymm6); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*2, ymm6); \
          ymm4  = _mm256_blend_ps(ymm7, ymm4, 0xCC); \
          ymm7  = _mm256_permute2f128_ps(ymm15, ymm4, 0x20); \
-         _mm256_storeu_ps(ptr_B + 3*ldB, ymm7); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*3, ymm7); \
          ymm1  = _mm256_permute2f128_ps(ymm10, ymm11, 0x31); \
          ymm0  = _mm256_permute2f128_ps(ymm12, ymm13, 0x31); \
-         _mm256_storeu_ps(ptr_B + 4*ldB, ymm1); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*4, ymm1); \
          ymm5  = _mm256_permute2f128_ps(ymm8, ymm5, 0x31); \
          ymm4  = _mm256_permute2f128_ps(ymm15, ymm4, 0x31); \
-         _mm256_storeu_ps(ptr_B + 5*ldB, ymm0); \
-         _mm256_storeu_ps(ptr_B + 6*ldB, ymm5); \
-         _mm256_storeu_ps(ptr_B + 7*ldB, ymm4);}}}}}}}} \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*5, ymm0); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*6, ymm5); \
+         _mm256_storeu_ps(ptr_B + (size_t)ldB*7, ymm4);}}}}}}}} \
 }
 
 #define COMPRESS_FP32(v, k, m, cnt) { \
