@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2014-2018, Intel Corporation                                **
+** Copyright (c) 2014-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -32,7 +32,7 @@
 ******************************************************************************/
 #include "libxsmm_main.h"
 
-#if !defined(LIBXSMM_SYNC_FUTEX) && defined(__linux__)
+#if !defined(LIBXSMM_SYNC_FUTEX) && defined(__linux__) && defined(_GNU_SOURCE)
 # define LIBXSMM_SYNC_FUTEX
 #endif
 
@@ -42,12 +42,11 @@
 #include <stdint.h>
 #include <string.h>
 #include <stdio.h>
-#include <math.h>
 #if defined(_WIN32)
 # include <windows.h>
 # include <process.h>
 #else
-# if defined(LIBXSMM_SYNC_FUTEX) && defined(__linux__)
+# if defined(LIBXSMM_SYNC_FUTEX) && defined(__linux__) && defined(_GNU_SOURCE)
 #   include <linux/futex.h>
 # endif
 # include <unistd.h>
@@ -77,7 +76,7 @@ struct LIBXSMM_RETARGETABLE libxsmm_barrier {
   internal_sync_core_tag** cores;
   internal_sync_thread_tag** threads;
   int ncores, nthreads_per_core;
-  int nthreads, ncores_log2;
+  int nthreads, ncores_nbits; /* nbits(ncores) != log2(ncores) */
   /* internal counter type which is guaranteed to be atomic when using certain methods */
   volatile int threads_waiting;
   /* thread-safety during initialization */
@@ -87,107 +86,112 @@ struct LIBXSMM_RETARGETABLE libxsmm_barrier {
 
 LIBXSMM_API libxsmm_barrier* libxsmm_barrier_create(int ncores, int nthreads_per_core)
 {
-  libxsmm_barrier *const barrier = (libxsmm_barrier*)libxsmm_aligned_malloc(
-    sizeof(libxsmm_barrier), LIBXSMM_CACHELINE);
-#if !defined(LIBXSMM_NO_SYNC)
-  barrier->ncores = ncores;
-  barrier->ncores_log2 = LIBXSMM_LOG2((ncores << 1) - 1);
-  barrier->nthreads_per_core = nthreads_per_core;
-  barrier->nthreads = ncores * nthreads_per_core;
-
-  barrier->threads = (internal_sync_thread_tag**)libxsmm_aligned_malloc(
-    barrier->nthreads * sizeof(internal_sync_thread_tag*), LIBXSMM_CACHELINE);
-  barrier->cores = (internal_sync_core_tag**)libxsmm_aligned_malloc(
-    barrier->ncores * sizeof(internal_sync_core_tag*), LIBXSMM_CACHELINE);
-
-  barrier->threads_waiting = barrier->nthreads; /* atomic */
-  barrier->init_done = 0; /* false */
-#else
+  libxsmm_barrier *const barrier = (libxsmm_barrier*)malloc(sizeof(libxsmm_barrier));
+#if (0 == LIBXSMM_SYNC)
   LIBXSMM_UNUSED(ncores); LIBXSMM_UNUSED(nthreads_per_core);
+#else
+  if (NULL != barrier && 1 < ncores && 1 <= nthreads_per_core) {
+    barrier->ncores = ncores;
+    barrier->ncores_nbits = (int)LIBXSMM_NBITS(ncores);
+    barrier->nthreads_per_core = nthreads_per_core;
+    barrier->nthreads = ncores * nthreads_per_core;
+    barrier->threads = (internal_sync_thread_tag**)libxsmm_aligned_malloc(
+      barrier->nthreads * sizeof(internal_sync_thread_tag*), LIBXSMM_CACHELINE);
+    barrier->cores = (internal_sync_core_tag**)libxsmm_aligned_malloc(
+      barrier->ncores * sizeof(internal_sync_core_tag*), LIBXSMM_CACHELINE);
+    barrier->threads_waiting = barrier->nthreads; /* atomic */
+    barrier->init_done = 0; /* false */
+  }
+  else
 #endif
+  if (NULL != barrier) {
+    barrier->nthreads = 1;
+  }
   return barrier;
 }
 
 
 LIBXSMM_API void libxsmm_barrier_init(libxsmm_barrier* barrier, int tid)
 {
-#if !defined(LIBXSMM_NO_SYNC)
-  const int cid = tid / barrier->nthreads_per_core; /* this thread's core ID */
-  internal_sync_core_tag* core = 0;
-  int i;
-  internal_sync_thread_tag* thread;
-
-  /* we only initialize the barrier once */
-  if (barrier->init_done == 2) {
-    return;
-  }
-
-  /* allocate per-thread structure */
-  thread = (internal_sync_thread_tag*)libxsmm_aligned_malloc(
-    sizeof(internal_sync_thread_tag), LIBXSMM_CACHELINE);
-  barrier->threads[tid] = thread;
-  thread->core_tid = tid - (barrier->nthreads_per_core * cid); /* mod */
-
-  /* each core's thread 0 does all the allocations */
-  if (0 == thread->core_tid) {
-    core = (internal_sync_core_tag*)libxsmm_aligned_malloc(
-      sizeof(internal_sync_core_tag), LIBXSMM_CACHELINE);
-    core->id = (uint8_t)cid;
-    core->core_sense = 1;
-
-    core->thread_senses = (uint8_t*)libxsmm_aligned_malloc(
-      barrier->nthreads_per_core * sizeof(uint8_t), LIBXSMM_CACHELINE);
-    for (i = 0; i < barrier->nthreads_per_core; ++i) core->thread_senses[i] = 1;
-
-    for (i = 0; i < 2; ++i) {
-      core->my_flags[i] = (uint8_t*)libxsmm_aligned_malloc(
-        barrier->ncores_log2 * sizeof(uint8_t) * LIBXSMM_CACHELINE,
-        LIBXSMM_CACHELINE);
-      core->partner_flags[i] = (uint8_t**)libxsmm_aligned_malloc(
-        barrier->ncores_log2 * sizeof(uint8_t*),
-        LIBXSMM_CACHELINE);
-    }
-
-    core->parity = 0;
-    core->sense = 1;
-    barrier->cores[cid] = core;
-  }
-
-  /* barrier to let all the allocations complete */
-  if (0 == LIBXSMM_ATOMIC_SUB_FETCH(&barrier->threads_waiting, 1, LIBXSMM_ATOMIC_RELAXED)) {
-    barrier->threads_waiting = barrier->nthreads; /* atomic */
-    barrier->init_done = 1; /* true */
-  }
-  else {
-    while (0/*false*/ == barrier->init_done);
-  }
-
-  /* set required per-thread information */
-  thread->core = barrier->cores[cid];
-
-  /* each core's thread 0 completes setup */
-  if (0 == thread->core_tid) {
-    int di;
-    for (i = di = 0; i < barrier->ncores_log2; ++i, di += LIBXSMM_CACHELINE) {
-      /* find dissemination partner and link to it */
-      const int dissem_cid = (cid + (1 << i)) % barrier->ncores;
-      assert(0 != core); /* initialized under the same condition; see above */
-      core->my_flags[0][di] = core->my_flags[1][di] = 0;
-      core->partner_flags[0][i] = (uint8_t*)&barrier->cores[dissem_cid]->my_flags[0][di];
-      core->partner_flags[1][i] = (uint8_t*)&barrier->cores[dissem_cid]->my_flags[1][di];
-    }
-  }
-
-  /* barrier to let initialization complete */
-  if (0 == LIBXSMM_ATOMIC_SUB_FETCH(&barrier->threads_waiting, 1, LIBXSMM_ATOMIC_RELAXED)) {
-    barrier->threads_waiting = barrier->nthreads; /* atomic */
-    barrier->init_done = 2;
-  }
-  else {
-    while (2 != barrier->init_done);
-  }
-#else
+#if (0 == LIBXSMM_SYNC)
   LIBXSMM_UNUSED(barrier); LIBXSMM_UNUSED(tid);
+#else
+  if (NULL != barrier && 1 < barrier->nthreads) {
+    const int cid = tid / barrier->nthreads_per_core; /* this thread's core ID */
+    internal_sync_core_tag* core = 0;
+    int i;
+    internal_sync_thread_tag* thread;
+
+    /* we only initialize the barrier once */
+    if (barrier->init_done == 2) {
+      return;
+    }
+
+    /* allocate per-thread structure */
+    thread = (internal_sync_thread_tag*)libxsmm_aligned_malloc(
+      sizeof(internal_sync_thread_tag), LIBXSMM_CACHELINE);
+    barrier->threads[tid] = thread;
+    thread->core_tid = tid - (barrier->nthreads_per_core * cid); /* mod */
+
+    /* each core's thread 0 does all the allocations */
+    if (0 == thread->core_tid) {
+      core = (internal_sync_core_tag*)libxsmm_aligned_malloc(
+        sizeof(internal_sync_core_tag), LIBXSMM_CACHELINE);
+      core->id = (uint8_t)cid;
+      core->core_sense = 1;
+
+      core->thread_senses = (uint8_t*)libxsmm_aligned_malloc(
+        barrier->nthreads_per_core * sizeof(uint8_t), LIBXSMM_CACHELINE);
+      for (i = 0; i < barrier->nthreads_per_core; ++i) core->thread_senses[i] = 1;
+
+      for (i = 0; i < 2; ++i) {
+        core->my_flags[i] = (uint8_t*)libxsmm_aligned_malloc(
+          barrier->ncores_nbits * sizeof(uint8_t) * LIBXSMM_CACHELINE,
+          LIBXSMM_CACHELINE);
+        core->partner_flags[i] = (uint8_t**)libxsmm_aligned_malloc(
+          barrier->ncores_nbits * sizeof(uint8_t*),
+          LIBXSMM_CACHELINE);
+      }
+
+      core->parity = 0;
+      core->sense = 1;
+      barrier->cores[cid] = core;
+    }
+
+    /* barrier to let all the allocations complete */
+    if (0 == LIBXSMM_ATOMIC_SUB_FETCH(&barrier->threads_waiting, 1, LIBXSMM_ATOMIC_RELAXED)) {
+      barrier->threads_waiting = barrier->nthreads; /* atomic */
+      barrier->init_done = 1; /* true */
+    }
+    else {
+      while (0/*false*/ == barrier->init_done);
+    }
+
+    /* set required per-thread information */
+    thread->core = barrier->cores[cid];
+
+    /* each core's thread 0 completes setup */
+    if (0 == thread->core_tid) {
+      int di;
+      for (i = di = 0; i < barrier->ncores_nbits; ++i, di += LIBXSMM_CACHELINE) {
+        /* find dissemination partner and link to it */
+        const int dissem_cid = (cid + (1 << i)) % barrier->ncores;
+        assert(0 != core); /* initialized under the same condition; see above */
+        core->my_flags[0][di] = core->my_flags[1][di] = 0;
+        core->partner_flags[0][i] = (uint8_t*)&barrier->cores[dissem_cid]->my_flags[0][di];
+        core->partner_flags[1][i] = (uint8_t*)&barrier->cores[dissem_cid]->my_flags[1][di];
+      }
+    }
+
+    /* barrier to let initialization complete */
+    if (0 == LIBXSMM_ATOMIC_SUB_FETCH(&barrier->threads_waiting, 1, LIBXSMM_ATOMIC_RELAXED)) {
+      barrier->threads_waiting = barrier->nthreads; /* atomic */
+      barrier->init_done = 2;
+    }
+    else {
+      while (2 != barrier->init_done);
+    }
+  }
 #endif
 }
 
@@ -195,106 +199,110 @@ LIBXSMM_API void libxsmm_barrier_init(libxsmm_barrier* barrier, int tid)
 LIBXSMM_API LIBXSMM_INTRINSICS(LIBXSMM_X86_GENERIC)
 void libxsmm_barrier_wait(libxsmm_barrier* barrier, int tid)
 {
-#if !defined(LIBXSMM_NO_SYNC)
-  internal_sync_thread_tag *const thread = barrier->threads[tid];
-  internal_sync_core_tag *const core = thread->core;
+#if (0 == LIBXSMM_SYNC)
+  LIBXSMM_UNUSED(barrier); LIBXSMM_UNUSED(tid);
+#else
+  if (NULL != barrier && 1 < barrier->nthreads) {
+    internal_sync_thread_tag *const thread = barrier->threads[tid];
+    internal_sync_core_tag *const core = thread->core;
 
-  /* first let's execute a memory fence */
-  LIBXSMM_ATOMIC_SYNC(LIBXSMM_ATOMIC_SEQ_CST);
+    /* first let's execute a memory fence */
+    LIBXSMM_ATOMIC_SYNC(LIBXSMM_ATOMIC_SEQ_CST);
 
-  /* first signal this thread's arrival */
-  core->thread_senses[thread->core_tid] = (uint8_t)(0 == core->thread_senses[thread->core_tid] ? 1 : 0);
+    /* first signal this thread's arrival */
+    core->thread_senses[thread->core_tid] = (uint8_t)(0 == core->thread_senses[thread->core_tid] ? 1 : 0);
 
-  /* each core's thread 0 syncs across cores */
-  if (0 == thread->core_tid) {
-    int i;
-    /* wait for the core's remaining threads */
-    for (i = 1; i < barrier->nthreads_per_core; ++i) {
-      uint8_t core_sense = core->core_sense, thread_sense = core->thread_senses[i];
-      while (core_sense == thread_sense) { /* avoid evaluation in unspecified order */
+    /* each core's thread 0 syncs across cores */
+    if (0 == thread->core_tid) {
+      int i;
+      /* wait for the core's remaining threads */
+      for (i = 1; i < barrier->nthreads_per_core; ++i) {
+        uint8_t core_sense = core->core_sense, thread_sense = core->thread_senses[i];
+        while (core_sense == thread_sense) { /* avoid evaluation in unspecified order */
+          LIBXSMM_SYNC_PAUSE;
+          core_sense = core->core_sense;
+          thread_sense = core->thread_senses[i];
+        }
+      }
+
+      if (1 < barrier->ncores) {
+        int di;
+# if defined(__MIC__)
+        /* cannot use LIBXSMM_ALIGNED since attribute may not apply to local non-static arrays */
+        uint8_t sendbuffer[LIBXSMM_CACHELINE+LIBXSMM_CACHELINE-1];
+        uint8_t *const sendbuf = LIBXSMM_ALIGN(sendbuffer, LIBXSMM_CACHELINE);
+        __m512d m512d;
+        _mm_prefetch((const char*)core->partner_flags[core->parity][0], _MM_HINT_ET1);
+        sendbuf[0] = core->sense;
+        m512d = LIBXSMM_INTRINSICS_MM512_LOAD_PD(sendbuf);
+# endif
+
+        for (i = di = 0; i < barrier->ncores_nbits - 1; ++i, di += LIBXSMM_CACHELINE) {
+# if defined(__MIC__)
+          _mm_prefetch((const char*)core->partner_flags[core->parity][i+1], _MM_HINT_ET1);
+          _mm512_storenrngo_pd(core->partner_flags[core->parity][i], m512d);
+# else
+          *core->partner_flags[core->parity][i] = core->sense;
+# endif
+          while (core->my_flags[core->parity][di] != core->sense) LIBXSMM_SYNC_PAUSE;
+        }
+
+# if defined(__MIC__)
+        _mm512_storenrngo_pd(core->partner_flags[core->parity][i], m512d);
+# else
+        *core->partner_flags[core->parity][i] = core->sense;
+# endif
+        while (core->my_flags[core->parity][di] != core->sense) LIBXSMM_SYNC_PAUSE;
+        if (1 == core->parity) {
+          core->sense = (uint8_t)(0 == core->sense ? 1 : 0);
+        }
+        core->parity = (uint8_t)(1 - core->parity);
+      }
+
+      /* wake up the core's remaining threads */
+      core->core_sense = core->thread_senses[0];
+    }
+    else { /* other threads wait for cross-core sync to complete */
+      uint8_t core_sense = core->core_sense, thread_sense = core->thread_senses[thread->core_tid];
+      while (core_sense != thread_sense) { /* avoid evaluation in unspecified order */
         LIBXSMM_SYNC_PAUSE;
         core_sense = core->core_sense;
-        thread_sense = core->thread_senses[i];
+        thread_sense = core->thread_senses[thread->core_tid];
       }
     }
-
-    if (1 < barrier->ncores) {
-      int di;
-#if defined(__MIC__)
-      /* cannot use LIBXSMM_ALIGNED since attribute may not apply to local non-static arrays */
-      uint8_t sendbuffer[LIBXSMM_CACHELINE+LIBXSMM_CACHELINE-1];
-      uint8_t *const sendbuf = LIBXSMM_ALIGN(sendbuffer, LIBXSMM_CACHELINE);
-      __m512d m512d;
-      _mm_prefetch((const char*)core->partner_flags[core->parity][0], _MM_HINT_ET1);
-      sendbuf[0] = core->sense;
-      m512d = LIBXSMM_INTRINSICS_MM512_LOAD_PD(sendbuf);
-#endif
-
-      for (i = di = 0; i < barrier->ncores_log2 - 1; ++i, di += LIBXSMM_CACHELINE) {
-#if defined(__MIC__)
-        _mm_prefetch((const char*)core->partner_flags[core->parity][i+1], _MM_HINT_ET1);
-        _mm512_storenrngo_pd(core->partner_flags[core->parity][i], m512d);
-#else
-        *core->partner_flags[core->parity][i] = core->sense;
-#endif
-        while (core->my_flags[core->parity][di] != core->sense) LIBXSMM_SYNC_PAUSE;
-      }
-
-#if defined(__MIC__)
-      _mm512_storenrngo_pd(core->partner_flags[core->parity][i], m512d);
-#else
-      *core->partner_flags[core->parity][i] = core->sense;
-#endif
-      while (core->my_flags[core->parity][di] != core->sense) LIBXSMM_SYNC_PAUSE;
-      if (1 == core->parity) {
-        core->sense = (uint8_t)(0 == core->sense ? 1 : 0);
-      }
-      core->parity = (uint8_t)(1 - core->parity);
-    }
-
-    /* wake up the core's remaining threads */
-    core->core_sense = core->thread_senses[0];
-  }
-  else { /* other threads wait for cross-core sync to complete */
-    uint8_t core_sense = core->core_sense, thread_sense = core->thread_senses[thread->core_tid];
-    while (core_sense != thread_sense) { /* avoid evaluation in unspecified order */
-      LIBXSMM_SYNC_PAUSE;
-      core_sense = core->core_sense;
-      thread_sense = core->thread_senses[thread->core_tid];
-    }
   }
-#else
-  LIBXSMM_UNUSED(barrier); LIBXSMM_UNUSED(tid);
 #endif
 }
 
 
 LIBXSMM_API void libxsmm_barrier_destroy(const libxsmm_barrier* barrier)
 {
-#if !defined(LIBXSMM_NO_SYNC)
-  int i;
-  if (2 == barrier->init_done) {
-    for (i = 0; i < barrier->ncores; ++i) {
-      int j;
-      libxsmm_free((const void*)barrier->cores[i]->thread_senses);
-      for (j = 0; j < 2; ++j) {
-        libxsmm_free((const void*)barrier->cores[i]->my_flags[j]);
-        libxsmm_free(barrier->cores[i]->partner_flags[j]);
+#if (0 != LIBXSMM_SYNC)
+  if (NULL != barrier && 1 < barrier->nthreads) {
+    if (2 == barrier->init_done) {
+      int i;
+      for (i = 0; i < barrier->ncores; ++i) {
+        int j;
+        libxsmm_free((const void*)barrier->cores[i]->thread_senses);
+        for (j = 0; j < 2; ++j) {
+          libxsmm_free((const void*)barrier->cores[i]->my_flags[j]);
+          libxsmm_free(barrier->cores[i]->partner_flags[j]);
+        }
+        libxsmm_free(barrier->cores[i]);
+      }
+      for (i = 0; i < barrier->nthreads; ++i) {
+        libxsmm_free(barrier->threads[i]);
       }
-      libxsmm_free(barrier->cores[i]);
-    }
-    for (i = 0; i < barrier->nthreads; ++i) {
-      libxsmm_free(barrier->threads[i]);
     }
+    libxsmm_free(barrier->threads);
+    libxsmm_free(barrier->cores);
   }
-  libxsmm_free(barrier->threads);
-  libxsmm_free(barrier->cores);
 #endif
-  libxsmm_free(barrier);
+  free((libxsmm_barrier*)barrier);
 }
 
 
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 enum {
   INTERNAL_SYNC_LOCK_FREE = 0,
   INTERNAL_SYNC_LOCK_LOCKED = 1,
@@ -329,7 +337,7 @@ LIBXSMM_API libxsmm_spinlock* libxsmm_spinlock_create(void)
     LIBXSMM_LOCK_ATTR_INIT(LIBXSMM_LOCK_SPINLOCK, &attr);
     LIBXSMM_LOCK_INIT(LIBXSMM_LOCK_SPINLOCK, &result->impl, &attr);
     LIBXSMM_LOCK_ATTR_DESTROY(LIBXSMM_LOCK_SPINLOCK, &attr);
-#elif !defined(LIBXSMM_NO_SYNC)
+#elif (0 != LIBXSMM_SYNC)
     result->state = INTERNAL_SYNC_LOCK_FREE;
 #endif
   }
@@ -348,7 +356,7 @@ LIBXSMM_API void libxsmm_spinlock_destroy(const libxsmm_spinlock* spinlock)
 
 LIBXSMM_API int libxsmm_spinlock_trylock(libxsmm_spinlock* spinlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 # if defined(LIBXSMM_LOCK_SYSTEM_SPINLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   assert(0 != spinlock);
   return LIBXSMM_LOCK_TRYLOCK(LIBXSMM_LOCK_SPINLOCK, &spinlock->impl);
@@ -370,7 +378,7 @@ LIBXSMM_API int libxsmm_spinlock_trylock(libxsmm_spinlock* spinlock)
 
 LIBXSMM_API void libxsmm_spinlock_acquire(libxsmm_spinlock* spinlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 # if defined(LIBXSMM_LOCK_SYSTEM_SPINLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   assert(0 != spinlock);
   LIBXSMM_LOCK_ACQUIRE(LIBXSMM_LOCK_SPINLOCK, &spinlock->impl);
@@ -391,7 +399,7 @@ LIBXSMM_API void libxsmm_spinlock_acquire(libxsmm_spinlock* spinlock)
 
 LIBXSMM_API void libxsmm_spinlock_release(libxsmm_spinlock* spinlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
   assert(0 != spinlock);
 # if defined(LIBXSMM_LOCK_SYSTEM_SPINLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   LIBXSMM_LOCK_RELEASE(LIBXSMM_LOCK_SPINLOCK, &spinlock->impl);
@@ -407,7 +415,7 @@ LIBXSMM_API void libxsmm_spinlock_release(libxsmm_spinlock* spinlock)
 
 #if defined(LIBXSMM_LOCK_SYSTEM_MUTEX) && defined(LIBXSMM_SYNC_SYSTEM)
 typedef LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK_MUTEX) libxsmm_mutex_state;
-#elif defined(LIBXSMM_SYNC_FUTEX) && defined(__linux__)
+#elif defined(LIBXSMM_SYNC_FUTEX) && defined(__linux__) && defined(_GNU_SOURCE)
 typedef int libxsmm_mutex_state;
 #else
 typedef char libxsmm_mutex_state;
@@ -431,7 +439,7 @@ LIBXSMM_API libxsmm_mutex* libxsmm_mutex_create(void)
     LIBXSMM_LOCK_ATTR_INIT(LIBXSMM_LOCK_MUTEX, &attr);
     LIBXSMM_LOCK_INIT(LIBXSMM_LOCK_MUTEX, &result->impl, &attr);
     LIBXSMM_LOCK_ATTR_DESTROY(LIBXSMM_LOCK_MUTEX, &attr);
-#elif !defined(LIBXSMM_NO_SYNC)
+#elif (0 != LIBXSMM_SYNC)
     result->state = INTERNAL_SYNC_LOCK_FREE;
 #endif
   }
@@ -450,7 +458,7 @@ LIBXSMM_API void libxsmm_mutex_destroy(const libxsmm_mutex* mutex)
 
 LIBXSMM_API int libxsmm_mutex_trylock(libxsmm_mutex* mutex)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
   assert(0 != mutex);
 # if defined(LIBXSMM_LOCK_SYSTEM_MUTEX) && defined(LIBXSMM_SYNC_SYSTEM)
   return LIBXSMM_LOCK_TRYLOCK(LIBXSMM_LOCK_MUTEX, &mutex->impl);
@@ -466,7 +474,7 @@ LIBXSMM_API int libxsmm_mutex_trylock(libxsmm_mutex* mutex)
 
 LIBXSMM_API void libxsmm_mutex_acquire(libxsmm_mutex* mutex)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 # if defined(LIBXSMM_LOCK_SYSTEM_MUTEX) && defined(LIBXSMM_SYNC_SYSTEM)
   assert(0 != mutex);
   LIBXSMM_LOCK_ACQUIRE(LIBXSMM_LOCK_MUTEX, &mutex->impl);
@@ -511,13 +519,13 @@ LIBXSMM_API void libxsmm_mutex_acquire(libxsmm_mutex* mutex)
 
 LIBXSMM_API void libxsmm_mutex_release(libxsmm_mutex* mutex)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
   assert(0 != mutex);
 # if defined(LIBXSMM_LOCK_SYSTEM_MUTEX) && defined(LIBXSMM_SYNC_SYSTEM)
   LIBXSMM_LOCK_RELEASE(LIBXSMM_LOCK_MUTEX, &mutex->impl);
 # else
   LIBXSMM_ATOMIC_SYNC(LIBXSMM_ATOMIC_SEQ_CST);
-#   if defined(LIBXSMM_SYNC_FUTEX) && defined(__linux__)
+#   if defined(LIBXSMM_SYNC_FUTEX) && defined(__linux__) && defined(_GNU_SOURCE)
   if (INTERNAL_SYNC_LOCK_CONTESTED == LIBXSMM_ATOMIC_FETCH_SUB(&mutex->state, 1, LIBXSMM_ATOMIC_RELAXED)) {
     mutex->state = INTERNAL_SYNC_LOCK_FREE;
     syscall(INTERNAL_SYNC_FUTEX, &mutex->state, FUTEX_WAKE, 1, NULL, NULL, 0);
@@ -532,7 +540,7 @@ LIBXSMM_API void libxsmm_mutex_release(libxsmm_mutex* mutex)
 }
 
 
-#if !defined(LIBXSMM_NO_SYNC) && !(defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM))
+#if (0 != LIBXSMM_SYNC) && !(defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM))
 LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE internal_sync_counter {
   struct {
     uint16_t writer;
@@ -544,12 +552,12 @@ LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE internal_sync_counter {
 
 
 LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_rwlock {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 # if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK_RWLOCK) impl;
 # else
-  volatile internal_sync_counter requests;
   volatile internal_sync_counter completions;
+  volatile internal_sync_counter requests;
 # endif
 #else
   int dummy;
@@ -561,11 +569,16 @@ LIBXSMM_API libxsmm_rwlock* libxsmm_rwlock_create(void)
 {
   libxsmm_rwlock *const result = (libxsmm_rwlock*)malloc(sizeof(libxsmm_rwlock));
   if (0 != result) {
-#if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
+#if (0 != LIBXSMM_SYNC)
+# if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
     LIBXSMM_LOCK_ATTR_TYPE(LIBXSMM_LOCK_RWLOCK) attr;
     LIBXSMM_LOCK_ATTR_INIT(LIBXSMM_LOCK_RWLOCK, &attr);
     LIBXSMM_LOCK_INIT(LIBXSMM_LOCK_RWLOCK, &result->impl, &attr);
     LIBXSMM_LOCK_ATTR_DESTROY(LIBXSMM_LOCK_RWLOCK, &attr);
+# else
+    memset((void*)&result->completions, 0, sizeof(internal_sync_counter));
+    memset((void*)&result->requests, 0, sizeof(internal_sync_counter));
+# endif
 #else
     memset(result, 0, sizeof(libxsmm_rwlock));
 #endif
@@ -583,7 +596,7 @@ LIBXSMM_API void libxsmm_rwlock_destroy(const libxsmm_rwlock* rwlock)
 }
 
 
-#if !defined(LIBXSMM_NO_SYNC) && !(defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM))
+#if (0 != LIBXSMM_SYNC) && !(defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM))
 LIBXSMM_API_INLINE int internal_rwlock_trylock(libxsmm_rwlock* rwlock, internal_sync_counter* prev)
 {
   internal_sync_counter next;
@@ -603,7 +616,7 @@ LIBXSMM_API_INLINE int internal_rwlock_trylock(libxsmm_rwlock* rwlock, internal_
 
 LIBXSMM_API int libxsmm_rwlock_trylock(libxsmm_rwlock* rwlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 # if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   assert(0 != rwlock);
   return LIBXSMM_LOCK_TRYLOCK(LIBXSMM_LOCK_RWLOCK, &rwlock->impl);
@@ -620,7 +633,7 @@ LIBXSMM_API int libxsmm_rwlock_trylock(libxsmm_rwlock* rwlock)
 
 LIBXSMM_API void libxsmm_rwlock_acquire(libxsmm_rwlock* rwlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 # if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   assert(0 != rwlock);
   LIBXSMM_LOCK_ACQUIRE(LIBXSMM_LOCK_RWLOCK, &rwlock->impl);
@@ -639,7 +652,7 @@ LIBXSMM_API void libxsmm_rwlock_acquire(libxsmm_rwlock* rwlock)
 
 LIBXSMM_API void libxsmm_rwlock_release(libxsmm_rwlock* rwlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
   assert(0 != rwlock);
 # if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   LIBXSMM_LOCK_RELEASE(LIBXSMM_LOCK_RWLOCK, &rwlock->impl);
@@ -654,10 +667,10 @@ LIBXSMM_API void libxsmm_rwlock_release(libxsmm_rwlock* rwlock)
 }
 
 
-#if !defined(LIBXSMM_NO_SYNC) && !(defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM))
+#if (0 != LIBXSMM_SYNC) && !(defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM))
 LIBXSMM_API_INLINE int internal_rwlock_tryread(libxsmm_rwlock* rwlock, internal_sync_counter* prev)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
   assert(0 != rwlock && 0 != prev);
 # if defined(_WIN32)
   prev->bits = InterlockedExchangeAdd((volatile LONG*)&rwlock->requests.bits, INTERNAL_SYNC_RWLOCK_READINC);
@@ -677,7 +690,7 @@ LIBXSMM_API_INLINE int internal_rwlock_tryread(libxsmm_rwlock* rwlock, internal_
 
 LIBXSMM_API int libxsmm_rwlock_tryread(libxsmm_rwlock* rwlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 # if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   assert(0 != rwlock);
   return LIBXSMM_LOCK_TRYREAD(LIBXSMM_LOCK_RWLOCK, &rwlock->impl);
@@ -694,7 +707,7 @@ LIBXSMM_API int libxsmm_rwlock_tryread(libxsmm_rwlock* rwlock)
 
 LIBXSMM_API void libxsmm_rwlock_acqread(libxsmm_rwlock* rwlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
 # if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   assert(0 != rwlock);
   LIBXSMM_LOCK_ACQREAD(LIBXSMM_LOCK_RWLOCK, &rwlock->impl);
@@ -713,7 +726,7 @@ LIBXSMM_API void libxsmm_rwlock_acqread(libxsmm_rwlock* rwlock)
 
 LIBXSMM_API void libxsmm_rwlock_relread(libxsmm_rwlock* rwlock)
 {
-#if !defined(LIBXSMM_NO_SYNC)
+#if (0 != LIBXSMM_SYNC)
   assert(0 != rwlock);
 # if defined(LIBXSMM_LOCK_SYSTEM_RWLOCK) && defined(LIBXSMM_SYNC_SYSTEM)
   LIBXSMM_LOCK_RELREAD(LIBXSMM_LOCK_RWLOCK, &rwlock->impl);
