@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2009-2018, Intel Corporation                                **
+** Copyright (c) 2009-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -45,7 +45,7 @@
 # pragma offload_attribute(pop)
 #endif
 
-#if (defined(__GNUC__) || defined(__INTEL_COMPILER)) && (64 <= (LIBXSMM_BITS))
+#if (defined(__GNUC__) || defined(LIBXSMM_INTEL_COMPILER)) && (64 <= (LIBXSMM_BITS))
 # define LIBXSMM_TIMER_RDTSC(CYCLE) { libxsmm_timer_tickint libxsmm_timer_rdtsc_hi_; \
     __asm__ __volatile__ ("rdtsc" : "=a"(CYCLE), "=d"(libxsmm_timer_rdtsc_hi_)); \
     CYCLE |= libxsmm_timer_rdtsc_hi_ << 32; \
@@ -55,9 +55,13 @@
 #endif
 
 
-LIBXSMM_API_INLINE libxsmm_timer_tickint internal_timer_tick(void)
+LIBXSMM_APIVAR(int internal_timer_init_rtc);
+
+
+LIBXSMM_API_INTERN libxsmm_timer_tickint libxsmm_timer_tick_rtc(void)
 {
   libxsmm_timer_tickint result;
+  int dummy;
 #if defined(_WIN32)
   LARGE_INTEGER t;
   QueryPerformanceCounter(&t);
@@ -71,19 +75,8 @@ LIBXSMM_API_INLINE libxsmm_timer_tickint internal_timer_tick(void)
   gettimeofday(&t, 0);
   result = 1000000ULL * t.tv_sec + t.tv_usec;
 #endif
-  return result;
-}
-
-
-LIBXSMM_API_INTERN LIBXSMM_INTRINSICS(LIBXSMM_X86_GENERIC)
-libxsmm_timer_tickint libxsmm_timer_tick_rdtsc(void)
-{
-  libxsmm_timer_tickint result;
-#if defined(LIBXSMM_TIMER_RDTSC)
-  LIBXSMM_TIMER_RDTSC(result);
-#else
-  result = internal_timer_tick();
-#endif
+  dummy = LIBXSMM_ATOMIC_ADD_FETCH(&internal_timer_init_rtc, 1, LIBXSMM_ATOMIC_RELAXED);
+  LIBXSMM_UNUSED(dummy);
   return result;
 }
 
@@ -93,22 +86,29 @@ libxsmm_timer_tickint libxsmm_timer_tick(void)
 {
   libxsmm_timer_tickint result;
 #if defined(LIBXSMM_TIMER_RDTSC)
-  if (0 < libxsmm_timer_scale) {
-    LIBXSMM_TIMER_RDTSC(result);
-  }
-  else
+  LIBXSMM_TIMER_RDTSC(result);
+#else
+  result = libxsmm_timer_tick_rtc();
 #endif
-  {
-    result = internal_timer_tick();
-  }
   return result;
 }
 
 
+LIBXSMM_API libxsmm_timer_tickint libxsmm_timer_cycles(libxsmm_timer_tickint tick0, libxsmm_timer_tickint tick1)
+{
+  return LIBXSMM_DELTA(tick0, tick1);
+}
+
+
 LIBXSMM_API double libxsmm_timer_duration(libxsmm_timer_tickint tick0, libxsmm_timer_tickint tick1)
 {
-  double result = (double)LIBXSMM_DIFF(tick0, tick1);
+  double result = (double)LIBXSMM_DELTA(tick0, tick1);
 #if defined(LIBXSMM_TIMER_RDTSC)
+# if defined(LIBXSMM_INIT_COMPLETED)
+  LIBXSMM_ASSERT_MSG(0 != internal_timer_init_rtc, "LIBXSMM is not initialized");
+# else
+  if (0 == internal_timer_init_rtc) libxsmm_init();
+# endif
   if (0 < libxsmm_timer_scale) {
     result *= libxsmm_timer_scale;
   }
