@@ -7,6 +7,7 @@
 #define LIBXSMM_CONFIG_VERSION_MINOR $MINOR
 #define LIBXSMM_CONFIG_VERSION_UPDATE $UPDATE
 #define LIBXSMM_CONFIG_VERSION_PATCH $PATCH
+#define LIBXSMM_CONFIG_BUILD_DATE $DATE
 
 #define LIBXSMM_CONFIG_CACHELINE $CACHELINE
 #define LIBXSMM_CONFIG_ALIGNMENT $CACHELINE
@@ -16,12 +17,11 @@
 
 #define LIBXSMM_CONFIG_PREFETCH $PREFETCH
 #define LIBXSMM_CONFIG_MAX_MNK $MAX_MNK
+#define LIBXSMM_CONFIG_MAX_DIM $MAX_DIM
+#define LIBXSMM_CONFIG_AVG_DIM $AVG_DIM
 #define LIBXSMM_CONFIG_MAX_M $MAX_M
 #define LIBXSMM_CONFIG_MAX_N $MAX_N
 #define LIBXSMM_CONFIG_MAX_K $MAX_K
-#define LIBXSMM_CONFIG_AVG_M $AVG_M
-#define LIBXSMM_CONFIG_AVG_N $AVG_N
-#define LIBXSMM_CONFIG_AVG_K $AVG_K
 #define LIBXSMM_CONFIG_FLAGS $FLAGS
 #define LIBXSMM_CONFIG_ALPHA $ALPHA
 #define LIBXSMM_CONFIG_BETA $BETA
