@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2018, Intel Corporation                                **
+** Copyright (c) 2016-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -17,7 +17,7 @@
 ** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
 ** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
 ** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
+** a PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
 ** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
 ** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
 ** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
@@ -38,7 +38,7 @@ const __m256i *const shufmasks = internal_spmdm_shufmasks_32;
 const __m256i *const shufmasks2 = internal_spmdm_shufmasks_16;
 #endif
 int block_offset_base, block_offset;
-int index[16];
+int idx_array[16];
 SIMDTYPE_INT32 vindex = _MM_SETZERO_INT32();
 
 LIBXSMM_UNUSED(nthreads);
@@ -46,12 +46,12 @@ LIBXSMM_UNUSED(tid);
 
 kb = block_id / handle->mb;
 mb = block_id % handle->mb;
-if ('T' == transA || 't' == transA) {
+if ('T' == transa || 't' == transa) {
   int kk;
   block_offset_base = mb * handle->bm;
   block_offset = block_offset_base + kb * handle->m * handle->bk;
-  for (kk = 0; kk < SIMD_WIDTH_FP32; kk++) index[kk] = kk*handle->m;
-  vindex = _MM_LOADU_INT32(index);
+  for (kk = 0; kk < SIMD_WIDTH_FP32; kk++) idx_array[kk] = kk*handle->m;
+  vindex = _MM_LOADU_INT32(idx_array);
 }
 else {
   block_offset_base = kb * handle->bk;
@@ -64,7 +64,7 @@ else {
   /*printf("nrows: %d, ncols: %d\n", nrows, ncols);*/
   int ncols_aligned = ncols / (4*SIMD_WIDTH_FP32)*(4*SIMD_WIDTH_FP32);
   int ncols_aligned_2 = ncols / (SIMD_WIDTH_FP32)*(SIMD_WIDTH_FP32);
-  const float * input_ptr = A + block_offset;
+  const float * input_ptr = a + block_offset;
   uint16_t * rowidx_ptr = slice.rowidx;
   uint16_t * colidx_ptr = slice.colidx;
   float    * values_ptr = (float *)(slice.values);
@@ -76,12 +76,12 @@ else {
 #endif
   for (i = 0; i < nrows; i++) {
     rowidx_ptr[i] = cnt;
-    if ('T' == transA || 't' == transA) {
+    if ('T' == transa || 't' == transa) {
       for (k = 0; k < ncols_aligned; k += 4*SIMD_WIDTH_FP32) {
-        SIMDTYPE_FP32 v1 = _MM_GATHER_FP32(input_ptr + k*handle->m + i, vindex, 4);
-        SIMDTYPE_FP32 v2 = _MM_GATHER_FP32(input_ptr + (k+SIMD_WIDTH_FP32)*handle->m + i, vindex, 4);
-        SIMDTYPE_FP32 v3 = _MM_GATHER_FP32(input_ptr + (k+2*SIMD_WIDTH_FP32)*handle->m + i, vindex, 4);
-        SIMDTYPE_FP32 v4 = _MM_GATHER_FP32(input_ptr + (k+3*SIMD_WIDTH_FP32)*handle->m + i, vindex, 4);
+        SIMDTYPE_FP32 v1 = _MM_GATHER_FP32(input_ptr + (size_t)k * handle->m + i, vindex, 4);
+        SIMDTYPE_FP32 v2 = _MM_GATHER_FP32(input_ptr + ((size_t)k+1*SIMD_WIDTH_FP32) * handle->m + i, vindex, 4);
+        SIMDTYPE_FP32 v3 = _MM_GATHER_FP32(input_ptr + ((size_t)k+2*SIMD_WIDTH_FP32) * handle->m + i, vindex, 4);
+        SIMDTYPE_FP32 v4 = _MM_GATHER_FP32(input_ptr + ((size_t)k+3*SIMD_WIDTH_FP32) * handle->m + i, vindex, 4);
         SIMDMASKTYPE_FP32 m1 = _MM_CMPNEQ_FP32(v1, vzero);
         SIMDMASKTYPE_FP32 m2 = _MM_CMPNEQ_FP32(v2, vzero);
         SIMDMASKTYPE_FP32 m3 = _MM_CMPNEQ_FP32(v3, vzero);
@@ -92,7 +92,7 @@ else {
         COMPRESS_FP32(v4, k + 3*SIMD_WIDTH_FP32, m4, cnt);
       }
       for (k = ncols_aligned; k < ncols_aligned_2; k += SIMD_WIDTH_FP32) {
-        SIMDTYPE_FP32 v1 = _MM_GATHER_FP32(input_ptr + k*handle->m + i, vindex, 4);
+        SIMDTYPE_FP32 v1 = _MM_GATHER_FP32(input_ptr + (size_t)k * handle->m + i, vindex, 4);
         SIMDMASKTYPE_FP32 m1 = _MM_CMPNEQ_FP32(v1, vzero);
         COMPRESS_FP32(v1, k, m1, cnt);
       }
@@ -106,14 +106,14 @@ else {
       for (k = 0; k < ncols_aligned; k += 4*SIMD_WIDTH_FP32) {
         SIMDTYPE_FP32 v1, v2, v3, v4;
         SIMDMASKTYPE_FP32 m1, m2, m3, m4;
-        v1 = _MM_LOADU_FP32(input_ptr + i*handle->k + k);
-        _MM_PREFETCH((char *)(input_ptr + (i+2)*handle->k + k), _MM_HINT_T0);
-        v2 = _MM_LOADU_FP32(input_ptr + i*handle->k + k + SIMD_WIDTH_FP32);
-        _MM_PREFETCH((char *)(input_ptr + (i+2)*handle->k + k + SIMD_WIDTH_FP32), _MM_HINT_T0);
-        v3 = _MM_LOADU_FP32(input_ptr + i*handle->k + k + 2*SIMD_WIDTH_FP32);
-        _MM_PREFETCH((char *)(input_ptr + (i+2)*handle->k + k + 2*SIMD_WIDTH_FP32), _MM_HINT_T0);
-        v4 = _MM_LOADU_FP32(input_ptr + i*handle->k + k + 3*SIMD_WIDTH_FP32);
-        _MM_PREFETCH((char *)(input_ptr + (i+2)*handle->k + k + 3*SIMD_WIDTH_FP32), _MM_HINT_T0);
+        v1 = _MM_LOADU_FP32(input_ptr + ((size_t)i)   * handle->k + (size_t)k);
+        _MM_PREFETCH((char*)input_ptr + ((size_t)i+2) * handle->k + (size_t)k, _MM_HINT_T0);
+        v2 = _MM_LOADU_FP32(input_ptr + ((size_t)i)   * handle->k + (size_t)k + (size_t)SIMD_WIDTH_FP32);
+        _MM_PREFETCH((char*)input_ptr + ((size_t)i+2) * handle->k + (size_t)k + (size_t)SIMD_WIDTH_FP32, _MM_HINT_T0);
+        v3 = _MM_LOADU_FP32(input_ptr + ((size_t)i)   * handle->k + (size_t)k + (size_t)2 * SIMD_WIDTH_FP32);
+        _MM_PREFETCH((char*)input_ptr + ((size_t)i+2) * handle->k + (size_t)k + (size_t)2 * SIMD_WIDTH_FP32, _MM_HINT_T0);
+        v4 = _MM_LOADU_FP32(input_ptr + ((size_t)i)   * handle->k + (size_t)k + (size_t)3 * SIMD_WIDTH_FP32);
+        _MM_PREFETCH((char*)input_ptr + ((size_t)i+2) * handle->k + (size_t)k + (size_t)3 * SIMD_WIDTH_FP32, _MM_HINT_T0);
         m1 = _MM_CMPNEQ_FP32(v1, vzero);
         m2 = _MM_CMPNEQ_FP32(v2, vzero);
         m3 = _MM_CMPNEQ_FP32(v3, vzero);
@@ -126,8 +126,8 @@ else {
       for (k = ncols_aligned; k < ncols_aligned_2; k += SIMD_WIDTH_FP32) {
         SIMDTYPE_FP32 v1;
         SIMDMASKTYPE_FP32 m1;
-        v1 = _MM_LOADU_FP32(input_ptr + i*handle->k + k);
-        _MM_PREFETCH((char *)(input_ptr + (i+2)*handle->k + k), _MM_HINT_T0);
+        v1 = _MM_LOADU_FP32(input_ptr + ((size_t)i)   * handle->k + (size_t)k);
+        _MM_PREFETCH((char*)input_ptr + ((size_t)i+2) * handle->k + (size_t)k, _MM_HINT_T0);
         m1 = _MM_CMPNEQ_FP32(v1, vzero);
         COMPRESS_FP32(v1, k, m1, cnt);
       }
@@ -139,14 +139,5 @@ else {
     }
   }
   rowidx_ptr[nrows] = cnt;
-#if 0
-  printf("cnt: %d\n", cnt);
-  for (i = 0; i <= nrows; i++) {
-    int j;
-    for (j = slice.rowidx[i]; j < slice.rowidx[i+1]; j++) {
-      printf("(%d, %d): %f ", i, colidx_ptr[j], values_ptr[j]);
-    }
-  }
-#endif
 }
 
