@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2018, Intel Corporation                                **
+** Copyright (c) 2016-2019, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -30,7 +30,6 @@
 ******************************************************************************/
 #include <libxsmm_macros.h>
 #include <stdlib.h>
-#include <assert.h>
 
 #if !defined(ELEM_TYPE)
 # define ELEM_TYPE short
@@ -40,15 +39,19 @@
 int main(/*int argc, char* argv[]*/)
 {
   int ni = 9, nj = 7, nk = 3, i, j, k, linear = 0, result = EXIT_SUCCESS;
-  ELEM_TYPE* in1 = (ELEM_TYPE*)malloc(ni * nj * nk * sizeof(ELEM_TYPE));
-  LIBXSMM_VLA_DECL(3, const ELEM_TYPE, in3, in1, nj, nk);
+  ELEM_TYPE *const input = (ELEM_TYPE*)malloc(sizeof(ELEM_TYPE) * ni * nj * nk);
+  LIBXSMM_VLA_DECL(1, const ELEM_TYPE, in1, input);
+  LIBXSMM_VLA_DECL(3, const ELEM_TYPE, in3, input, nj, nk);
 
-  assert(0 != in1);
-  for (i = 0; i < (ni * nj * nk); ++i) in1[i] = (ELEM_TYPE)i;
+  LIBXSMM_ASSERT(NULL != input);
+  for (i = 0; i < (ni * nj * nk); ++i) input[i] = (ELEM_TYPE)i;
   for (i = 0; i < ni; ++i) {
     for (j = 0; j < nj; ++j) {
       for (k = 0; k < nk; ++k) {
-        if (in1[linear] != LIBXSMM_VLA_ACCESS(3, in3, i, j, k, nj, nk)) {
+        const ELEM_TYPE gold = input[linear];
+        if (gold != LIBXSMM_VLA_ACCESS(3, in3, i, j, k, nj, nk) ||
+            gold != LIBXSMM_VLA_ACCESS(1, in1, linear))
+        {
           result = EXIT_FAILURE;
           i = ni; j = nj;
           break;
@@ -58,6 +61,6 @@ int main(/*int argc, char* argv[]*/)
     }
   }
 
-  free(in1);
+  free(input);
   return result;
 }
